#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs-extra");
var child_process = require("child_process");
var glob = require("glob");
var path = require("path");
var yaml = require("js-yaml");
var program = require("commander");
var src;
var repoRelativePath;
var dest = 'docs-ref-autogen';
var packageWhiteList = [];
var pjson = require('../package.json');
program
    .version("v" + pjson.version)
    .description('A tool to go through source code and convert it to yaml data model for docfx.')
    .option('--whiteList [value]', 'white list some package name for extraction.')
    .arguments('<sourceFolder> [repoConfigPath]')
    .action(function (sourceFolder, repoConfig) {
    src = sourceFolder;
    repoRelativePath = repoConfig;
})
    .parse(process.argv);
if (!src) {
    console.log('Error: The source file folder is not specified!');
    program.help();
}
if (program.whiteList) {
    try {
        var tempList = JSON.parse(program.whiteList);
        if (tempList && tempList.length) {
            packageWhiteList = tempList;
        }
    }
    catch (e) {
    }
}
// 1. prepare
fs.removeSync(dest);
fs.mkdirpSync(dest);
var tsconfigs = glob.sync(path.join(src, '**/tsconfig.json'));
var toc = [];
tsconfigs.forEach(function (tsconfig) {
    var packagePath = tsconfig.replace('tsconfig.json', 'package.json');
    generatePackageDoc(packagePath, dest);
});
toc = JSON.parse(JSON.stringify(toc));
fs.writeFileSync(dest + "/toc.yml", yaml.safeDump(toc));
process.exit(0);
function generatePackageDoc(packagePath, dest) {
    var dir = path.dirname(packagePath);
    if (!fs.existsSync(packagePath)) {
        return;
    }
    var packageName = fs.readJsonSync(packagePath).name;
    if (packageWhiteList.length === 0 || packageWhiteList.indexOf(packageName) >= 0) {
        console.log(packageName);
        console.log(dir);
        var sourceCodeBasePath = dir + "/src";
        if (fs.existsSync(sourceCodeBasePath)) {
            var packageDest = dest + "/" + packageName;
            fs.mkdirpSync(packageDest);
            try {
                // child_process.execSync(`typedoc --json ${dir}/api.json ${dir}/src --module commonjs --ignoreCompilerErrors`);
                child_process.execFileSync('node', ['node_modules/typedoc/bin/typedoc', '--json', dir + "/api.json",
                    dir + "/src", '--module', 'commonjs', '--ignoreCompilerErrors']);
                var basePath = sourceCodeBasePath.replace(src + "/", '');
                child_process.execFileSync('node', ['node_modules/type2docfx/dist/main', dir + "/api.json", packageDest]);
                if (fs.existsSync(packageDest + "/toc.yml")) {
                    var subToc = yaml.safeLoad(fs.readFileSync(packageDest + "/toc.yml").toString());
                    toc.push(subToc[0]);
                    fs.removeSync(packageDest + "/toc.yml");
                }
            }
            catch (e) {
                console.log(packageName + ": TypeDoc Parse Error.");
            }
        }
        else {
            console.log("No source file for " + packageName);
        }
    }
}
