# [sp-client](./README.md)

The *sp-client* repo is the home for the [SharePoint Framework](http://aka.ms/spfx) (SPFx) and the first-party
components built on this platform, which includes: the canvas, modern publishing pages, the modern
team site experience, the tenant admin portal, various client-side web parts, and many related libraries.


## Help! :-)

➜ For general questions when working in this repo, join the modern group "[SP-Client Help and News](mailto:SP-Client%20Help%20and%20News%20&lt;spchan@service.microsoft.com&gt;)" (spchan).

➜ For status of ongoing issues, or to open a new ticket:  [https://aka.ms/spc-ticket](https://aka.ms/spc-ticket)

➜ The [Support Contacts](./common/docs/Onboarding/SupportContacts.md) page has a full breakdown of people who can help you.


## Onboarding  (your first day on the job)

1. [Setting up your dev box](common/docs/Onboarding/SettingUpYourDevBox.md): Step 1 for getting started with sp-client.

2. [Building projects](common/docs/Onboarding/BuildingProjects.md): The overall pull->install->build->run loop with Rush

3. Debugging your code:
   - [Debugging in EDog](common/docs/Onboarding/DebuggingInEdog.md)
   - Debugging with "gulp serve"
   - Debugging unit tests

## How to...  (everyday activities)

- [...add a package dependency](common/docs/HowTo/AddAPackageDependency.md): Read this if you need approval for a change to **browser-approved-packages.json** or **nonbrowser-approved-packages.json**.

- [...break the rules](common/docs/HowTo/BreakTheRules.md): The correct way to bypass VSO branch policy rules in an emergency

- [...create a localized string](common/docs/HowTo/CreateALocalizedString.md): Any text that's shown to an end user must be reviewed by UA and then translated to other languages

- [...create a new manifest](common/docs/HowTo/CreateANewManifest.md): Registering the manifest for a new client-side component (i.e. library, application, web part, or extension)

- [...create a new project](common/docs/HowTo/CreateANewProject.md): Creating a new project (i.e. NPM package) in this repo

- [...manage TAB tests](common/docs/HowTo/ManageTabTests.md): What is "TAB", how to interpret failures, how to disable a misbehaving test, etc.

- [...test the Yeoman generator](common/docs/HowTo/TestTheYeomanGenerator.md): Running the integration test for the SPFx Yeoman generator

- [...use internal npm feeds](common/docs/HowTo/UseInternalNPMFeeds.md): What our internal NPM feeds are for and how to consume them.

- [...write a change log](common/docs/HowTo/WriteAChangeLog.md): What to do if your PR build fails because you need to run "rush change"


## Best Practices  (for good citizens)

- [TypeScript Syntax Conventions](common/docs/BestPractices/TypeScriptSyntaxConventions.md): SharePoint agreed upon standards for the appearance of your code (capitalization, indentation, naming, syntaxes that are difficult to read, etc)

- [Recommended Coding Practices](common/docs/BestPractices/RecommendedCodingPractices.md): SharePoint best practices for writing better code (path-based imports, common security vulnerabilities, avoiding mistakes with promises, etc)

- [React Style Guide](common/docs/BestPractices/ReactStyleGuide.md): Coding conventions for projects using
  [ReactJS](https://facebook.github.io/react/)

- [Strict Null Checks](common/docs/BestPractices/StrictNullChecks.md): What is this, whether you need it, how to enable it for your project

- [Tslint Rules](common/docs/BestPractices/TslintRules.md): Explains certain [tslint](https://github.com/palantir/tslint) rules that we adopted, with recommended fixes when you encounter a warning


## Specialized Procedures  (for people who do those things)

- [Build Agent Troubleshooting](common/docs/Procedures/BuildAgentTroubleshooting.md): What to do when the lab machines are broken

- [Publishing API Documentation](common/docs/Procedures/PublishingApiDocs.md): Using api-documenter to update the [SharePoint API Reference](https://docs.microsoft.com/en-us/javascript/api/sp-application-base) web site

- [Publishing JSON Schemas](common/docs/Procedures/PublishingJsonSchemas.md): How to publish JSON schemas to dev.office.com

- [Rolling Code to MSIT and Prod](https://msft.spoppe.com/sites/SPEXDogfood/SitePages/Rolling%20Code%20to%20MSIT%20and%20Prod.aspx): Instructions for the OCE, who validates the weekly build and rolls it out in production

- [Shrinkwrap Reviews](common/docs/Procedures/ShrinkwrapReviews.md): Read this if you're a person who reviews changes to **npm-shrinkwrap.json**.

- [SPFx Release History](common/docs/Procedures/SpfxReleaseHistory.md): past NPM releases and the associated branches.

- [SPFx Release Process](common/docs/Procedures/SpfxReleaseProcess.md): Doing a public release of the NPM packages for the SharePoint Framework


## Reference Material  (answers if you need them)

- [API Extractor](common/docs/Reference/ApiExtractor.md): When to use release tags (@internal / @alpha / @beta / @public), why do I need an "API Review", etc.

- [Flight Performance Testing](common/docs/Reference/FlightPerformanceTesting.md): If you want to measure the performance impact of your SharePoint flight

- [SPFx Assemblies](common/docs/Reference/SpfxAssemblies.md): Technical info about SPFx's "assembly" construct, which is a bootloader script with other libraries embedded in it

- [SPFx Query Parameters](common/docs/Reference/SpfxQueryParameters.md): An inventory of all URL query parameters
  that are reserved by our framework, and what they do

- [Telemetry](common/docs/Reference/Telemetry.md): What is telemetry data, how to add it, how to view it, how to report it,
  how to alert on it.

- [Third-party client-side applications](common/docs/Reference/ThirdPartyClientSideApplications.md): Bundling a client-side application in an *.sppkg and running it on a TDS box

- [Verifying Code Coverage](common/docs/Reference/VerifyingCodeCoverage.md): The VerifyCoverageTask allows you to detect when people introduce source code with insufficient unit test coverage


## Project Inventory

If you are adding a new project:

- Don't forget to add it to the inventory below, in alphabetical order, with an informative description

- Buildable projects should always be at the 2nd level, following the
  **/&lt;category&gt;/&lt;project&gt;** convention.  This makes it easy to discover and build them from
  the command line.

- Do not add 1st level folders for any other reason.  If something is shared between projects,
  it should go in the **/common** folder.

- Do not introduce cross-project references that would prevent a project folder from being buildable
  in isolation.  It should be easy to move any project to github.com at any time.

### "apps" category

**announcement-list-demo**:
> *A sample client side application using the "list-based" model*

**article-site-demo**:
> *A sample client side application using the "page-based" model
> It hosts a canvas with web parts and implements in-place navigation
> (similar to a "Single-Page Application").*

**listview-spfx-host**:
> *A client-side application that is used by the SharePoint list view to host
> SPFx to enable extensibility scenarios.*

**sp-classic-page**:
> *The small "client-side application" that loads SPFx on a classic page, used when a
> client-side is added there.*

**sp-pages**:
> *The WEX team's client-side application that provides a page authoring
> experience for team sites (and eventually publishing sites).*

**sp-teams-tab**
> *The SharePoint Tab in Microsoft Teams*

**sp-tenant-admin**:
> *The modern tenant admin page.*

**sp-webpart-workbench**:
> *A visual design surface for testing web parts*

**yeoman-list-application**:
> *The skeleton list-based application generated by our Yeoman template.*

**yeoman-page-application**:
> *The skeleton page-based application generated by our Yeoman template.*


### "integration-tests" category
*NOTE: DON'T PUT UNIT TEST LIBRARIES HERE -- USE THE `src/test/index.ts` ENTRY POINT IN THE NON-TEST PACKAGE*

**integration-tests/sp-generator-test**
> *Integration test for the SPFx Yeoman generator*

**integration-tests/tab-test-extensions**
> *Client-side extensions deployed for use by partner TAB tests*

**integration-tests/sp-active-monitors**
> *Project that deploys TAB Tests and dependent libraries for use in active monitors*

### "libraries" category

**odsp-utilities-bundle**:
> *An SPFx-compatible bundle of the internal **odsp-utilites** package.*

**sp-bingmap**:
> *Module and API set for embedding bing map functionalities*

**sp-client-shared**: (deprecated)
> *Utility code that is shared throughout ODSP.
> Currently just the TokenUtility class used for making REST calls
> to external Microsoft servcies such as Exchange.*

**sp-component-layouts**:
> *Shared layouts for first-party components (webparts) in the **sp-client** repo*

**sp-component-utilties**:
> *Shared utilities for first-party components (apps, libraries, and webparts) in the **sp-client** repo*

**sp-dataproviders**:
> *Shared data providers for first-party components (webparts) in the **sp-client** repo*

**sp-dragzone**:
> *A react based control for drag drop capability in the **sp-client** repo*

**sp-filepicker**:
> *A control for selecting files within a SharePoint site*

**sp-html-embed**:
> *Render html code or preview content about one URL*

**sp-pagepicker**:
> *A control for selecting a site page in current site or an external page*

**sp-safehtml**
> *An HTML sanitization library*

**sp-canvas-toolbox**:
> *A control for adding web parts or sections to page*

### "project-templates" category

**empty-web-library**
> *A template that can be used when following the instructions in [AddingANewProject.md](./common/docs/AddingANewProject.md)*

### "prototypes" category

**package-tombstone**
> *Used to mark the last release of an **@ms** package that was
> graduated to the **@microsoft** public NPM scope.*

**sp-client-projects**
> *Some old Dev Kitchen webparts being kept around for sentimental reasons.*

**demo-application-extensions**
> *Some demo application extensions.

**demo-application**
> *A demo application to help test application extensions.

**test-extensions-application**
> *A localhost-served test application for debugging the client-side extension libraries.*

**test-extensions**
> *Some test client-side extensions used for debugging the SPFx extension support.*


### "spfx-core" category

**[decorators](spfx-core/decorators/README.md)** (public API)
> *Standard TypeScript decorators supported by SPFx.*

**sp-application-base**: (public API)
> *Coming soon -- future home of shell, ClientSideApplication, etc *

**sp-canvas**: (public API)
> *Coming soon -- future home of the Canvas control. *

**sp-client-preview**: (public API)
> *Preview features that have not been moved into an official package yet.
> Currently this includes a lot of web part APIs that we are working on migrating
> into the **sp-webpart-base** package.*

**sp-component-base**: (public API)
> *Common base classes shared between web parts, extensions, and applications.*

**sp-component-data**: (public API)
> *his package enables a client-side component to access its associated system data such as manifest properties and preloaded objects.*

**sp-core-library**: (public API)
> *Base SPFx APIs such as ServiceScope, Guid, SPFlight, etc.*

**sp-embed-webpart-base**: (public API)
> *Framework for creating embed web parts.*

**sp-extensions-base**: (public API)
> *Support for building SharePoint client-side extensions.*

**sp-http** (public API)
> *SPFx communication layer: HttpClient, SPHttpClient, OAuthTokenProvider, etc. *

**sp-module-interfaces**: (public API)
> *SPFx manifest interfaces that are shared between browser packages and the toolchain.*

**sp-loader** (public API)
> *The first thing to load on the page; it uses manifests to find bundles, and then boots the framework*

**sp-odata-types** (public API)
> *TypeScript interfaces for working with SharePoint REST APIs*

**sp-page-context** (public API)
> *SPFx PageContext and related APIs such as SPSite, SPWeb, SPList, SPUser, etc.*

**sp-telemetry**: (private API)
> *SPFx Telemetry and related internal APIs*

**sp-webpart-base**: (public API)
> *Support for building SharePoint client-side web parts: BaseClientSideWebPart,
> WebPartManager, property pane, etc.*

**sp-polyfills**:
> *Browser polyfills required by the SharePoint Framework*

### "spfx-libraries" category

**i18n-utilities**
> *Internationalization utilities, including a wrapper for the globalize library.*

**office-ui-fabric-react-bundle**:
> *An SPFx-compatible bundle of the **office-ui-fabric-react** package.*

**sp-external-libraries**
> Reads external packages containing manifests and prepares them for deployment to the CDN and the manifest server.

**sp-commandbar-extensibility** (public API)
> *APIs allowing third-party developers to customize the SharePoint command bar.
> Includes the BaseCommandBarCustomizer client-side extension support.*

**sp-jquery**
> *Provides support for third-party spfx developers to load jquery*

**sp-listview-extensibility** (public API)
> *APIs allowing third-party developers to customize the SharePoint ListView.
> Includes the BaseFieldCustomizer client-side extension support.*

**sp-lodash-subset** (public API)
> *An SPFx-compatible bundle of [Lodash](https://lodash.com/).  Only includes a
> conservative subset of essential functions.*

**sp-office-ui-fabric-core**
> *An SPFx compatible bundle of [office-ui-fabric-core](https://dev.office.com/fabric).
> Only includes set of functionalities which are supported by the SPFx.

**react-bundle**:
> *An SPFx-compatible bundle of **react**.

**react-dom-bundle**:
> *An SPFx-compatible bundle of **react-dom**.


### "spfx-tools" category

**api-lint**:
> *A utility that analyzes a project, detects common JSDoc problems, and generates
> a report of the exported Public API.*

**generator-sharepoint**:
> *The [Yeoman](http://yeoman.io/) generator for SharePoint client-side
> web parts and applications*

**[loader-cased-file](spfx-tools/loader-cased-file/README.md)**:
> *A Webpack loader for modifying filename casing*

**sp-build-common**
> *Common task infrastructure shared between **sp-build-node** and **sp-build-web** rigs*

**sp-build-core-tasks**:
> *Core gulp tasks used for SharePoint projects*

**sp-build-node**
> *SharePoint Framework build rig for NodeJS projects*

**sp-build-web**
> *SharePoint Framework build rig for web projects*

**sp-check**
> *A command-line tool that ensures a developer's PC has the right SharePoint Framework dependencies*

**sp-tslint-rules**:
> *Common TSLint overrides for all SharePoint client projects.*


### "tools" category

**csharp/ClientAppProvisioner**
> *A tool that provisions an ASPX page for a client-side application*

**csharp/ClientAppProvisioner**
> *A tool that provisions an ASPX page for a client-side application*

**sp-post-build-tasks**
> *Tasks that happen after the core component build has completed such as assembly bundling and size auditing*

**sp-build-internal-tasks**
> *First-party build tasks*

**sp-build-internal-web**
> *Internal build tasks for typescript, html, sass, etc.*

**sp-client-utilities**
> *Special build tasks that operate on the entire sp-client repo,
> e.g. the localization import/export task.*

**sp-selenium-utilities**
> *Utilities for launching tests via [Selenium](http://docs.seleniumhq.org/)*

**sp-special-internal-tasks**
> *A collection of tasks that aren't part of the normal build and are used by few projects.*

**sp-tab-tasklib**
> *Task libraries for use by TAB tests*

**[sp-visual-tests](tools/sp-visual-tests/README.md)**:
> *Set of visual regression tests for **sp-client** projects*

**[vsts-deploy](tools/vsts-deploy/README.md)**:
> *Used by our CI system to continuously deploy master to our CDNs and manifest service.*

**webpart-index**
> *The listing of first-party web parts which are uploaded to manifest server*


### "webparts" category

**clientwebpart**
> *The client-side web part used to host AppParts on modern pages*

**sp-chart-webparts**
> *A collection of chart webparts (e.g. QuickChart)*

**sp-connector-webparts**
> The base O365 Connector web part and webparts derived from it to access services from external providers such as Bing, Github, Mail Chimp, Trello, Twitter, etc.

**sp-embed-webparts**
> *A collection of embed webparts (e.g. Twitter, YouTube, PayPal)*

**sp-other-webparts**
> *The WorldClock web part; a prototype web part that is not currently shipping*

**sp-people-webparts**
> *A collection of people webparts (e.g. People)*

**sp-publishing-webparts**
> *Web parts for publishing sites: Document Embed, Content Rollup, Image,
> Quick Links, BingMap, Yammer Embed, etc.*

**sp-sample-webparts**
> *Web parts that serve as code samples for third-party developers*

**sp-teamsite-webparts**
> *Web parts for team sites: Newsfeed, Newsreel, Pinned Items, Site Activity*

**yeoman-webpart**:
> *The skeleton web part project generated by our Yeoman template.*


### "integration-tests" category

**generator-sharepoint-test**
> Integration test for SharePoint generator
