# [sp-client](../../../README.md) › How To › Break The Rules

Most of our folders have various branch policies that require approvals for
certain changes.  These approvals serve many purposes:

- **Prevent pushing directly to master** -- Back when we allowed this, mistakes
  sometimes resulted in major disruptions of productivity.

- **Require code reviews** --  Code reviews make a difference! Even for experienced devs!
  If you discuss your work with at least one person who takes the time to ask insightful
  questions, and if you make time to implement their feedback, the effect is indisputable.
  Conversely, when we look back at the PRs that caused great suffering, they almost always
  were lacking a meaningful discussion.

- **Avoid common mistakes** -- Certain source files (e.g. *npm-shrinkwrap.json*)
  have non-obvious considerations.  Having a knowledgeable person check your changes
  avoids common mistakes.

- **Protect feature owners** -- Feature owners have the responsibility to
  support certain areas of the code base.  Branch policies allow them to
  be aware of activity in their area, and to have a voice about changes
  that may be difficult to support.

# When to break the rules

Despite all these benefits of branch policies, there are certain recognized
scenarios where they get in the way:

- **Reverting PRs in an emergency** - If master is failing to build, or if EDog is
  broken, *report it immediately!*  (Send an e-mail to
  [spchan](mailto:SP-Client%20Help%20and%20News%20&lt;spchan@service.microsoft.com&gt;),
  and mark your e-mail as "High Importance".)  **Our default policy is to REVERT
  the PR that caused the problem.**  If this requires reverting another PR that
  depends on it, do it.  Getting the branch into a healthy state for everyone
  is more important than preserving one person's changes; they can always resubmit
  their PR later.  If you are reverting PRs to get back to a healthy state, you
  do not need to wait around for someone to approve your changes.  Get it fixed.
  If you are sure that the CI build is going to pass, you can bypass that as well.

- **Critical fixes** - An OCE is woken up at 1am in the morning by a sev1 incident.
  The solution is a small fix that is obviously low risk, but nobody is available
  to do a code review.  In this situation, go ahead and merge your fix to unblock
  the customer, and then review it with someone in the morning.  Be sure to reference
  the ICM incident number in your PR description.

- **Implicit approval** - Suppose your next step is blocked by a PR that you
  need to merge.  You discussed the changes with your tech lead, but they were
  in a hurry and didn't have time to do the official signoff before they left
  the office.  In this situation, rather than waiting for them to get home and
  connect remotely, you can add a PR comment saying "verbally approved by X".
  (NOTE: Here we are assuming that the person actually reviewed your
  final iteration -- it is not meaningful to "approve" a PR without even looking
  at the code!)

- **Irrelevant policy** - Suppose that the "Localization Approvers" branch policy
  was triggered because you edited a file under /src/localization/*, however your
  change is just updating a code comment.  You are certain that the Localization Approvers
  are not interested in this kind of change.  In this situation, you can add a
  PR comment such as "bypassed Localization Approvers -- this PR does not contain
  any actual changes to localization".

- **A "waiting" person who is unreachable** - A person added some comments to
  iteration #1 of your PR, then set the "Waiting" state, and went on vacation
  for a week.  It's not reasonable for your PR to be blocked until they return.
  Instead, address their feedback as best you can, then you can simply delete their
  name by clicking the "X" on the VSO web page, and find someone else to signoff.
  (If you can't find anyone else, send an e-mail to all group members notifying
  them that you made a reasonable effort to find a reviewer but nobody responded,
  and therefore you are bypassing their policy.)

**IMPORTANT: WHEN YOU BYPASS SAFEGUARDS THAT YOUR TEAM HAS PUT IN PLACE, YOU ARE TAKING
PERSONAL RESPONSIBILITY FOR ANY CONSEQUENCES.  If something is broken, people
will look back at your PR and ask "why did this happen?"  The safeguards exist
to protect *your* reputation, not just the team.  Be careful and conscientious!**

# How to break the rules

If a single person is in the "Waiting" state, but all branch policies are satisfied
by other approvals, then you can simply click the "Complete" button; our VSO configuration
will allow you to merge the PR.  (Do not delete the "Waiting" person, since then there would be
no record of their dissenting opinion.)

Otherwise, you will need to contact a member of the **"SP-Client Policy Bypassers"** VSTS group.
These people all have permissions to click the "Complete" button and bypass any branch policies.
Because this permission also allows accidentally pushing directly to the master branch,
the people in this group should not be everyday sp-client developers.  In a critical situation,
members of this group are also technically allowed to bypass policies for their own PR,
provided another person approves it.  We try to ensure that this group contains someone from
every time zone.  Feel free to nominate additional members.

Here is a list of common **"SP-Client Policy Bypassers"** contacts:
- Chris Hackmann (chackman)
- Cyrus Balsara (cyrusb)
- Daniel Kogan (dkogan)
- Dmitriy Meyerzon (dmitriym)
- Evgeniia Firsova (evfirs)
- Jeremy Kelley (jeremyke)
- John Fan (johnfan)
- John Sanders (josander)
- Luca Bandinelli (lucaband)
- Pat Miller (patmill)
- Paula Wing (paulac)
- Robert Chen (rchen)
- Shannon Koh (shank)
- Vesa Juvonen (vesaj)
- William Russell More (wrmore)
- Yimo Tao (yimot)

To see the full roster of members, go to the [Security Settings](https://onedrive.visualstudio.com/DefaultCollection/SPPPlat/_admin/_security?_a=members)
page, and click on the "SP-Client Policy Bypassers" VSTS group.
