# [sp-client](../../../README.md) › How To › Manage TAB Tests

Table of Contents:

  * [Run Tab in a project where Tab is already set up](#run-tab-in-a-project-where-it-is-already-set-up)
    * [How to run TAB with the General set of tests](#if-you-want-to-run-tab-with-the-general-set-of-tests)
    * [How to open the test runner, select some tests, and execute them](#if-you-want-to-run-tab-and-select-the-tests)
    * [How to run TAB against a different environment (edog, spdf, prodbubble)](#if-you-want-to-run-tab-and-select-the-tests-but-run-against-the-spdf-environment)

  * [Write a new TAB test](#write-a-new-tab-test)
	  * [Create test file](#create-test-file)
	  * [Create tasks](#create-tasks)
    * [Checking in a new or changed TAB Test](#checking-in-a-new-or-changed-tab-test)

  * [Adding Tab to a New Project](#adding-tab-to-a-new-project)

  * [Add Tab Task for a project to the PR Build](#add-tab-task-for-a-project-to-the-pr-build)

  * [Investigating Tab Test Failures](#investigating-tab-test-failures)

  * [**Important** Note about Flights and TAB Tests](#flights-and-tab-tests)

---
## Run Tab in a project where it is already set up
 Navigate to the project directory where you are running your test.

#### If you want to run Tab with the General set of tests:
	gulp runtab --browser=chrome

#### If you want to run tab and select the tests:
	 gulp runtab --browser=chrome --noAutoRun

#### If you want to run tab and select the tests but run against the SPDF environment:
	 gulp runtab --browser=chrome --noAutoRun --config=spdf

This will deploy all your files to Azure and start a window with a 'Test Harness'. You will see a dialog screen when the Harness is starting, click 'Ok'.

Once the Test Harness window is open, on the left you will see 3 options 'General', 'InProgress' and 'Needs Development'.

You will most likely want to run the tests in 'General'. Click the '+' icon next to General and select the tests you want to have run.
Alternatively, you can simply click the checkbox next to 'General' to have all the tests ran.

Finally, at the bottom left corner of the window click 'Run Tests'. This will open a seperate window to run the tests. There will be another dialog box that appears in this new Dialog window, click 'Ok' and now the automated tests will run.

---

## Write a new TAB test

#### Create Test File
Navigate to the root directory of the project you want to add a Tab test to.
If the directory ```src/tab``` does not exist below the root directory, create it.
Now, create your test file in ```src/tab```.

* To view an example of the structure of this file view, [WorkbenchBasic.ts](https://onedrive.visualstudio.com/SPPPlat/_git/sp-client?path=%2Fapps%2Fsp-webpart-workbench%2Fsrc%2Ftab%2FWorkbenchBasic.ts&version=GBmaster&_a=contents).


#### Create Tasks
To create new tasks we need to navigate over to the [libraries\sp-tab-tasklib]( https://onedrive.visualstudio.com/SPPPlat/_git/sp-client?path=%2Flibraries%2Fsp-tab-tasklib&version=GBmaster&_a=contents) project directory.

Navigate to the ```src/tab``` folder, this is where we will add new tasks.

If there is a not in this directory that encapsulates the domain of your test then create a new file.

* To view an example of a task file, take a look at [Workbench.ts](https://onedrive.visualstudio.com/SPPPlat/_git/sp-client?path=%2Flibraries%2Fsp-tab-tasklib%2Fsrc%2Ftasklib%2FWorkbench.ts&version=GBmaster&_a=contents)


##### Components of an individual task
1. *task* - this is where you define the action you want to take place. (like the click of a button, input of text, etc)
2. *waiter* - this is where you define a condition that must be ```true``` for the task to be successful and subsequently for the the TAB test to advance to the next task. When the task is executed, execution will not pass until the waiter condition is true. If it takes too long, the waiter will time out and your tab test will fail.

##### Example of a simple task
```
   public static AddTaskClearSecurityDialog(): void {
      TAB.AddTask(
        undefined,
        TAB.MakeWaiter(() => {
          return TAB.ElementExists(TAB.Win, TAB.searchBy.customQuery, `[data-automation-id='debugManifestPromptDialog']`);
        })
      );
      TAB.AddTask(
        TAB.MakeTask(() => {
          TAB.Log.AddTaskComment('Clear security dialog');
          const confirmButton: HTMLElement = Next.GetElementByDataAutomationId(TAB.Win as any, 'debugManifestConfirmButton');
          TAB.ClickElement(confirmButton);
        }),
        Workbench.PageReady()
      );
    }
```


##### Steps to create new tasks:

1. make sure there is a way to load up the page you want to test. There may be an existing way to create the page you want and if there is you will find it in [PageUtil.ts](https://onedrive.visualstudio.com/SPPPlat/_git/sp-client?path=%2Flibraries%2Fsp-tab-tasklib%2Fsrc%2Ftasklib%2FPageUtil.ts&version=GBmaster&_a=contents). If there is not, then you will need to create a function to load the page you want to run tasks on. You can find example's of how to do so in [PageUtil.ts](https://onedrive.visualstudio.com/SPPPlat/_git/sp-client?path=%2Flibraries%2Fsp-tab-tasklib%2Fsrc%2Ftasklib%2FPageUtil.ts&version=GBmaster&_a=contents).

##### Common Pitfall
* After editing files in [sp-tab-tasklib](https://onedrive.visualstudio.com/SPPPlat/_git/sp-client?path=%2Flibraries%2Fsp-tab-tasklib&version=GBmaster&_a=contents), you will most likely navigate back to the project you want to test and run the tab test. If you do not run  ```gulp build``` from the project directory you wish to test after editing this file, you will not see the changes.



##### Getting elements to test in your task
The functions you will use the most are:

*  ```function TAB.ElementExists(searchObj: any, criteria: TAB.searchBy, value: any): boolean```
* ``` function TAB.GetElement(searchObj: any, criteria: TAB.searchBy, value: any): HTMLElement```

The second parameter in both of the functions allows several ways of locating the DOM element that you wish to inspect. The parameter value depends on how TAB should find the DOM element.

Possible values for the TAB.searchBy parameter:

* TAB.searchBy.hasClassName - use this if the DOM element has multiple class names.
* TAB.searchBy.id - use this if the unique identifier is located in the id name.
* TAB.searchBy.customQuery - use this if you wish to search for a 'data-automation-id'.
	* Ex: ```TAB.ElementExists(TAB.Win, TAB.searchBy.customQuery, `[data-automation-id='debugManifestPromptDialog']`);```

 ---

## Adding Tab to a New Project

 Add @ms/sp-tab-tasklib to dependencies in package.json


	"dependencies": {
    	...
        "@ms/sp-tab-tasklib": "~0.0.1",
    	...
    }
Run ```rush install```.

Add sp-tab-tasklib to the config file under externals

  	"externals": {
    	"sp-tab-tasklib": "node_modules/@ms/sp-tab-tasklib/dist/sp-tab-tasklib.js"
    },

We need to tell the compiler to copy our tab test files to the dist folder.
     Create ```/config/post-copy.json``` and paste the following

    {
     	"$schema": "node_modules/@ms/sp-build-internal-web/lib/schemas/copy.schema.json",
         "copyTo": {
         	"dist": [
            	"lib/tab/*.js"
            ]
         }
    }


 Exclude the Tab test files from from code coverage enforcement by creating ```verify-coverage.json``` in the config folder and setting an option to skip the 'tab' folder.

	// file: config/verify-coverage.json
    {
      "skipFoldersOrFiles": [

        // TAB tests are excluded for unit tests
        "tab"
      ]
    }

Finally, create a file ```tab-config.json``` in the config folder. This is where you will specify tests that should appear in the TestHarness.

Below is the template you can paste into your file now.
```
{
  "TestUI": {
    "General": {},
    "InProgress": {},
    "NeedsInvestigation": {}
  }
}
```
As you add tests, you update this file like this:
```
{
  "TestUI": {
    "General": {
      "Example": ["<AzureTestLocation>/test.js", 0, false],
      "PanelsBasic": ["<AzureTestLocation>/PanelsBasic.js", 0, false]
    },
    "InProgress": {},
    "NeedsInvestigation": {}
  }
}
```
For now, you can ignore what the semantics of the '<AzureTestLocation>', '0' and 'false' are.'

Separate tests should be in separate files
An individual test is made up of setup, run, and teardown tasks.

Create a ```tab``` folder under the ```src``` folder.

Create your test in ```src/tab/```. An example name is ```WorkbenchBasic.ts```.
You can find take a look at the [Workbench Basic test for reference](https://onedrive.visualstudio.com/SPPPlat/_git/sp-client?path=%2Fapps%2Fsp-webpart-workbench%2Fsrc%2Ftab%2FWorkbenchBasic.ts&version=GBmaster&_a=contents)

---

## Checking in a new or changed Tab Test

* New or modified TAB Tests go through the **TAB Approvers** group for sign off.
* Your TAB test has been run **50x locally** in Chrome using the local TAB harness (gulp runtab --noAutoRun --browser=chrome) and you save/send the log as proof before you get to check in your test.
  * You will find the setting to change the number of runs under Settings --> Debugging --> #Runs (check this) --> change # to 50, then start your test.
  * Logs are here after hitting “Save Log” on the local TAB harness: ```\sp-client\webparts\sp-test-webparts\node_modules\@ms\sp-build-internal-web\node_modules\@ms\sp-build-internal-tasks\node_modules\@ms\onedrive-buildtools-binaries\tab\bin\service\logs```

---

##  Add Tab Task for a project to the PR Build

Before you add a Tab Test to the master PR build definition, you will want to test it on a clone of the master PR build definition.

#### Test new TAB Task on a clone of master PR
If you already have a test build definition you can tamper with you can skip this clone step.

Visit [the master PR build definition](https://onedrive.visualstudio.com/SPPPlat/_build?path=%5Csp-client%20Pull%20Requests&_a=allDefinitions) and click *clone* by first clicking the 3 horizontal dots to the right of the definition name. You will want to clone the build definition into the 'Experimental' folder.

Once you have a build definition, click 'Edit'.
Here you will find other Batch Scripts and items for the PR build definition.

Scroll to the bottom of the left list and click 'Add New', a list will open on the right side of the page and you then will want to hover the mouse over 'Batch Script' and the 'Add' button will appear, click the 'Add' button.

At this time, a new configurable script with name 'Run Script' will appear at the bottom of the left list. Click this new item to edit it.

#####Enter the following values:

**Display Name**: ``` Run TAB in TabStop - <your-project-name> (<environment>, <browser>)```
**Path**:``` <path-to-your-project's-node_modules-relative-to-the-root>\node_modules\.bin\gulp.cmd ```

* Example: ``` sp-webpart-workbench\node_modules\.bin\gulp.cmd ```

**Arguments**:
```
runtab --browser=chrome --tabstopAsync --tabstopUser=$(TabStopUsername) --tabstopPwd=$(TabStopPassword) --tabstopServer=$(TabStopServer) --tabstopDb=$(TabStopDatabase) --tabstopHostname=$(TabStopHostname) --tabstopJobFile=tabstopChrome.json --ciMode
```
Click the drop down menu for advanced:

Path: ```<path to your project's root folder from sp-client> ```
Example: ``` apps/sp-webpart-workbench```

Click the drop down menu for Control Options:

Make sure 'Enabled' is checked.
You can optionally click 'Proceed on Failure'. This means that the build won't stop if your tab test fails. Note that running tasks should not be in a 'Proceed on Failure' state for more than a few days. This is because tasks in this state aren't protecting the build and if the tests are failing and are not fixed, they delay the build time for everyone. Tasks left in this state will be disabled.

Additionally, we need to create a duplicate script with all the same options except a different name.

This duplicate is to check the results. If you look closely at the tabstopJobFile argument it tells tabStop to create a job file ```--tabstopJobFile=tabstopChrome.json```.
TabStop will first check if this file exists, if it doesn't not it will create one and start the automation. If it does exist it will watch the results.
In this case, the second script will watch the results from the first batch script.

Enter the following:
Display Name:
```Check TabStop job result - <your-project-name> (<environment>, <browser>)```
Fill in all the other options will the same as the previously made script.

Click 'Save & Queue' at the top right of the page.

Now, a dialog will be presented:
Make sure the 'Agent-queue' is set to 'odsp-lab'.
Set 'branch' to your working branch.
Finally, click 'Queue'.

At this time, a new build will queue up and you can watch to see if your Tab task succeeded or failed.
After successful builds, you can get approval to add the new TAB task to the sp-client-pullrequest-master build definition.

---

## Investigating Tab Test Failures

### Finding the TAB Test Logs

* If you are running from the local TAB test harness, then the logs should be right in the harness.
* If you are investigating a TAB test task failure from a PR Build, you can drill into the failures by clicking on the failing task, then clicking the tabstop url in the logs to get detailed logs of the tests themselves.

### Initial Considerations

Tab tests could be failing for many reasons.

Note that PR Test Tasks can fail for other non-TAB reasons, like a dev-deploy failure, etc.

* Look at the logs to see what the last successful comment was and what the next one should have been.
* Look at the screenshot in the logs after the error.
  * Does it look like a log in error, server error, or other environmental error?
* Look at the video of the test run.
* Is there a flight change that could be causing the failure?
* Is there expected content that the test expects/requires missing in the target environment/test tenant?
* Are you using a data automation id to select elements to ensure you protect yourself against css/etc changes?

### PR Build Issues

* If the build is failing for just one person, it's probably just their change.
* If the build is failing for 3+ different people with the same failure, it's most likely a breaking change or an environmental outage.
* If you are already on an older build, run against the build definition environment/browser on the local harness.
  * If it doesn't work, it must be environmental, since your older build doesn't have any current changes.
  * Sync to the current/suspect build, run against the build definition environment/browser on the local harness.
    * If it doesn't work, try it against another environment (--config=spdf, --config=prodbubble, etc)
      * If it works there, it's probably environmental.
      * If it doesn't work, there's a breaking change.
Compare the build history (looking for the first chronological failure) with the chronological merges.

### Debugging a TAB test

You can only debug using the local TAB test harness, however, if you are debugging a PR build test failure, you can use the Investigate link from the logs to kick off a run using the same deployed code and environment that the failed test used.

In the TAB Test harness, click on the Debug checkbox, then open the browser developer tools (F12). If you then start the test you want to run, it will immediately break for debugging in the browser. You can now set breakpoints in your code and then continue and debug.

---

## Broken or Flaky TAB tests

### Broken/Blocking TAB test

* A blocking test is one that is failing 100% across all users in recent PR history, thus blocking checkins for others.
  * The onus is on the test owner to immediately begin an investigation of what is failing (product? Test?) and why, and what the resolution will be.
  * This is basically a sev 1.
    * There should be acknowledgement and updates every hour at least until resolution.
  * If the test owner isn’t available, should fall to the EM to have someone look at it, etc.
  * If the owner can’t find a resolution, the test needs to be disabled (moved from General to Needs Investigation).

### Flaky TAB test

* A flaky test is one that is consistently failing at least once (out of the three times it runs) across most PRs in recent PR history. While this doesn’t block it does cause build times to increase unnecessarily.
  * The onus is on the test owner to begin an investigation of what is failing (product? Test?) and why, and what the resolution will be.
  * This is basically a sev 2.
    * There should be acknowledgement and updates at least every day and should be resolved within days.
  * If the test owner isn’t available, should fall to the EM to have someone look at it, etc.
  * If the owner can’t find a resolution, the test needs to be disabled (moved from General to Needs Investigation).

---

## Flights and TAB tests

If your feature has a TAB test and your change introduces a behavioral change  to the feature - the TAB test should be updated to handle it. Otherwise, if TAB tenant doesn’t yet have a flight – your change passes TAB and then it suddenly breaks for multiple engineers when flight is turned on for everyone. If you are just beginning next version development and you anticipate the flight rollout will take a while it’s important to also not regress existing behavior. You should do this in your local environment cheaply to avoid using CI/the build system.

---

## TAB Test Tenants

### Configuration

* Enable the ability to run debug code:
  * Add the tenant to a localscope for the following flight:
    * https://flightsonrails/Flight/Edit/SppplatDevCookiesEnabled
    * Flight name: SppplatDevCookiesEnabled
    * Flightid: 554
    * Only 5 tenants per ring can have this flight enabled at a time.