# [sp-client](../../../README.md) › How To › Write A Change Log

## Introduction

It is common practice to release a change log along with every release of
your software in order for your users to be aware of bug fixes,
new features, and changed or removed functionality. In SP-Client, this is
done by utilizing the `rush change` command. This command should
be run once you are ready to merge your PR. It analyzes the changes
in your branch and (when necessary) prompts you to write human-readable
descriptions of your changes.

**Changefile descriptions are public!** External developers **will**
see and read your changefile, so the way in which you phrase your
description is very important. You don't want to be overly concise
or specific, you don't want to reveal private information, and you
want the description to be as helpful as possible.

Err on the side of readability. Ask yourself: "How is my change relevant to a third-party developer?"
Could it break them?  Does it fix a bug that's been annoying them?  Is it a new feature for them to try?

A human editor will review the change logs before they are pubished, but
everyone should do their best to ensure the editing job is not a painful
chore.

## Style Guide
### Rules and Tips
* Use the [present simple tense](http://www.englishtenses.com/tenses/present_simple) using the
  [imperative ("command") mood](http://grammarist.com/grammar/english-moods/). E.g.

  "Sit on the sofa."

  "Let me go to sleep."

  "Keep reading."

* Start with a verb.

* Ideally, use one of the following words:
  * **Add** - when you introduce or expose a new feature, property, class, UI, etc.
  * **Remove** - when you fully removed something and it can no longer be used.
  * **Deprecate** - when you plan on removing something, but it is still accessible.
  * **Fix an issue with/where...** - when you fixed a bug.
  * **Improve** - when you made an existing thing better.
  * **Update** - when you refresh something, but don't necessarily make it better.
  * **Upgrade** - when upgrading the version of a dependency.
  * **Initial/Beta release of ...** - when releasing a brand-new feature

* Use official terminology. Only use capitals for proper nouns. For class names, follow guidance below.
Pascal casing should only be used for type names.
  * "web parts" not "WebParts" or "Web Parts"
  * "property pane" not "Property Pane" or "PropertyPane"
  * "developer dashboard" not "dev dashboard" or "Developer Dashboard"

* Don't talk about things that are marked `@alpha`.

* Don't use the word **bug**. Use **issue** instead.

* Don't use shorthand words. Avoid obscure acronyms where possible. Obvious acronyms are reasonable
as long as it does not significantly detract from readability. Examples:
  * prop - **BAD**
  * property - **GOOD!!**
  * dev - **BAD**
  * developer - **GOOD!!**
  * IE - **BAD**
  * Internet Explorer - **GOOD!!**
  * XHR - **BAD**
  * XMLHttpRequest - **GOOD!!**

* Use () to indicate a function name, e.g. `setSomethingOnWebpart()`

* Use "double-quotes" around class, function, property names. However, avoid talking about classes,
and try to focus on end-user features. We should not mention any changes to internal function or
internal property or internal class. Only mention public interface changes.
For changes to internal objects, only mention it if there are any significant behavior changes.

* If doing an upgrade, write "Upgraded `PACKAGE` from `OLD_VERSION` to `NEW_VERSION`"

* If fixing a GitHub issue, consider adding the URL to the issue in parenthesis.

* Remember that real external customers will read what you write. Your changefile is not a git commit message.

* **Don't copy and paste the same message!** Only describe changes that are relevant to
that particular package.

* Your sentence structure will often be "`ACTION` `THING` to `THING`"

* Don't add a trailing period unless you have more than two (2) sentences.

* Use the word "function" instead of "method".

### Examples

* **Add** "safeWithCustomScriptDisabled" to the web part manifest schema
* **Remove** support for Internet Explorer releases older than IE9
* **Deprecate** the loadScript() function of "SPModuleLoader". Use loadComponent() instead.
* **Fix** an issue with "SPTimeZone" where the timezone was off by one
* **Fix** an issue where the module loader fails to load React during full moons
* **Improve** the debug manifests security confirmation dialog
* **Update** loadScript() to use XMLHttpRequest instead of \<script\> tags
* **Upgrade** @microsoft/office-ui-fabric-react from 1.0.0 to 2.0.0
* **Initial release of** SharePoint Framework Extensions

## Conclusion

Consistency is key! Conciseness is key! Our changelog is a direct message to our customers
and can either frustrate or delight them. Take some time to craft the kind of message you would
want to read!