# [sp-client](../../../README.md) › Onboarding › Building Projects

This document describes the daily workflow for working in the the
**sp-client** git repository.  If you haven't done so already, set up
your dev box following the instructions from the
[Setting Up Your Dev Box](./SettingUpYourDevBox.md) page.

## Rush concept

The buildable projects in this repo are always 2nd level folders, following the
**/&lt;category&gt;/&lt;project&gt;** convention.  Each project folder corresponds
to an NPM package.  In principle you can run "npm install" and "gulp" inside any
single folder to build it independently of the others.  However, this is impractical
for our everyday usage scenario, which has these requirements:

1.  **Bulk builds:** If you change an API that affects lots of projects, you want
    to build everything using a single command

2.  **Common npm install:** Running "npm install" takes several minutes and downloads
    hundreds of thousands of files.  If you had to do that separately for 30 different
    project folders, you would have to wait more than an hour!

3.  **No upgrade victims**: If someone makes an API or functional change that
    will break consuming projects, ideally they should be the one who pays the
    cost to fix what they broke, and that fix should occur *before* committing,
    rather than lagging it behind "git commit" -> "npm publish" -> "npm update".
    Our API contracts should be stable, but they can't be frozen in time, or
    resistant to refactoring.  Wherever it makes sense, we should avoid a model
    that hides the impact from the person who introduced the break, or creates
    victims who cannot upgrade to a new major version without facing a mixture
    of random breaks that they have to debug and then pass along to their own
    consumers.  Obviously it's more efficient for external team relationships
    to be decoupled, but for a group of people who all work at the same company
    (mostly on the same building floor) and talk to each other every day,
    this decoupling can be easily avoided.  :-)

4.  **Automatic rebuilding**: When you run "gulp serve" in a particular project,
    it should detect changes made in any of the sibling projects that it depends
    on.

The "Rush" tool extends the basic NPM model to address these advanced scenarios.
We'll give you a quick crash course, but you can find full documentation on
the [Rush web site](https://aka.ms/rush).


## Initial Build

Once you have installed the tools, you can setup Rush as follows:

1.  Open an sp-client command prompt by running the appropriate script from
    the root of your Git working folder:

    * **CommandPrompt-Cmd.cmd**: The classic Windows command prompt.
    * **CommandPrompt-Bash.cmd**: The Bourne-again shell that was installed with
      **Git for Windows**.
    * **CommandPrompt-PS.cmd**: A Windows PowerShell command prompt.
    * **CommandPrompt-Mac.command**: The Mac OS X terminal.

    This will map your PATH environment variable to find tools such as "gulp" in
    the "common/node_modules" folder.  (If you don't do this, then you may
    encounter build failures, e.g. if your globally installed version of "gulp" is
    too old or too new.)  You might consider creating nice desktop shortcuts to
    open your command prompt, maybe with different background colors to distinguish
    each enlistment folder.

2.  Run "**npm install -g @microsoft/rush**" to ensure you have the latest Rush tool.

    *NOTE: Make sure to specify "@microsoft/rush" and not "@ms/rush" or "rush"!*

3.  From anywhere in your git working folder, run "**rush install**".  This
    will install NPM modules in your "sp-client/common" folder.

    NOTE: If you are troubleshooting build issues, try "**rush install --full-clean**"
    instead.

4.  Do your initial build by running "**rush rebuild**" .  This will
    recurse through each project folder and run "gulp clean", "gulp",
    and "gulp test", and then give you a report of anything that failed to build.

    You could also do "**rush build**" if you want to just do an incremental build.


## Pull -> Edit -> Build -> Run -> Push

The above steps are only necessary when you need to do a clean full build (e.g.
after pulling changes that affected common/package.json).  Otherwise, you can
run "gulp" in individual project folders as usual.  Your daily workflow will
look like this:

1.  Pull the latest changes from git.

2.  If you pulled new changes, then you should run **rush install** to update your
    dependencies:

    > C:\GitRepos\sp-client> **rush install**

3.  If you pulled new changes, here's how to rebuild everything to make sure you're
    up to date:

    > C:\GitRepos\sp-client> **rush build**

4.  Run your code.  How you do this depends on the type of project.
    (The next stup [Debugging In EDog](./DebuggingInEdog.md) will discuss
    the most common debugging scenario.)

Commands such as "gulp clean" and "gulp test" can also be run
from within a project folder.  Rush is only required for your first
build, or after pulling new changes from Git.

## If you need to modify your package.json

The Rush wiki has a useful note about ["Updating a package.json file"](https://github.com/Microsoft/web-build-tools/wiki/Rush-~-Everyday-commands#updating-a-packagejson-file),
which covers most needs.

If you want to introduce a new library package, please follow the
[HowTo/AddAPackageDependency](../HowTo/AddAPackageDependency.md)
instructions instead.


## Next Steps

Great job!  Now that you've successfully compiled the code, please continue to
[Debugging In EDog](./DebuggingInEdog.md) to see how to debug a web part or
first-party application.  The top-level [README.md](../../../README.md) has links
to instructions for other debugging scenarios.
