# [sp-client](../../../README.md) › Onboarding › Debugging In EDog

To debug on eDog, you need to replace one or more production manifests with manifests for your local dev box. This means
that instead of using the manifest from the manifest server that points to our CDN, you're using a manifest that points
to scripts being served from a gulp server. We've introduced a new way to share manifests by pointing to a URL serving
a manifest initialization script. The format of the URL with the parameter is
`msft.spoppe.com/sites/wex/SitePages/P.aspx?noredir=true&debugManifestsFile=<path to debug manfiests init script>`

The build tools will automatically drop a debug manfiests init script called `manifests.js` in the `temp` directory on
every successful build, and this script contains manifests for all projects that are linked by Rush and are listed in
the `cumulativeManifestOptions/referencedProjects` object of the `write-manifests.json` config file. This means that
built changes made to all listed projects will automaticaly be present on the page being debugged. For example, if
you are running `gulp serve` in `sp-pages`, and you've made changes to both `sp-pages` and `sp-client-preview`, because
`sp-client-preview` is referenced by `sp-pages` and linked locally by Rush, you'll automatically see your changes made to
both `sp-pages` and `sp-client-preview`.

For local debugging, the full URL should look something like:

> `https://msft.spoppe.com/sites/wex/SitePages/P.aspx?debug=true&debugManifestsFile=https://localhost:4321/temp/manifests.js`

## Flags for debugging

Any component can be replaced for debug one, but there are some flags that enable/disable certain behavior.

### ?debug=true
Use `?debug=true` when you are modifying components necessary for SPFx or the application to load.


This includes things like sp-application-base or sp-http for SPPPLAT, and sp-pages, sp-component-library, sp-canvas for WEX.
If `debug` flag is not true, those components will use the original implementation.

This flag is not required when debugging web parts (not libraries that are re-used by sp-pages, though), or new applications
like Tenant Admin page, or extensions application (not sp-extension-base, though).

Note: The `debug` flag will replace the assembly with a minimal assembly that loads all components from its own bundle.
This allows the system to replace any of those components. The fact that there is an assembly loaded doesn't mean that
the other components are loaded from there.

### ?noredir=true
Use `?noredir=true` to avoid going to an error page when trying to load an application, if it fails.

This allows debugging through the initialization code. The expected behavior when something goes wrong is to end up
in an empty page. From there the page can be refreshed to execute the code again.

### ?moduleloader=\<url\>
Use `?moduleloader=<url>` to replace the assembly for a different loader code. This can be used to replace the assembly
with another assembly, or with the sp-loader bundle.

This flag is only required when modifying the following packages: sp-loader, sp-core-library, sp-telemetry, sp-lodash-subset
and odsp-utilities.

If there are no modifications to these packages, *DO NOT USE* the `moduleloader` flag. This involves changes in both the
server-side and the client-side code and might impact the experience.

Please note that when using the `moduleloader` flag, the `debug` flag is useless.

## Assemblies

Assemblies are JS files that includes the bootstrapping code for SPFx and other components.

There are several assemblies, like the sp-loader assembly, the default assembly, and sp-pages assembly.

Using `?debug=true` will load the sp-loader assembly, which contains only components relevant for the sp-loader, and all
other components are loaded based on their manifest information.

The default assembly (currently used by any application different than sp-pages, classic pages and workbench) loads all
components needed by SPFx to load an application, but not the application itself.

The other assemblies include both the components needed by SPFx and the components needed by the application itself.
This applies to sp-pages, classic pages and workbench.

The assembly in use can be identified in the F12 Developer Tools, by looking at the loaded file that follows the pattern:
`<name>-assembly-<locale>.js`

## Sharing local builds

To share the current state of a project on your local enlistment, run `gulp dev-deploy` and share the URL fragment
emitted after all files have been uploaded to Azure. Append the `--rush` parameter to deploy every project in your
enlistment (so the full command is `gulp dev-deploy --rush`). Without the `--rush` parameter, only the current
project is shared, however this method follows the same semantics as the method described above wherein all dependent
projects are also deployed. Therefore, if you want to share multiple projects without deploying the entire enlistment,
ensure that all of the projects you intend to share are in the dependency graph of the project from which you run
`gulp dev-deploy`.

Only one dev-deploy may be shared per enlistment per branch, so if you run `gulp dev-deploy` more than once in your
enlistment while on a certain branch, the last one taskes precedence.

Keep in mind that dev-deploy only deploys, it doesn't build anything. For `gulp dev-deploy` to work properly, ensure
that the project you're deploying, and its dependency tree, are fully built. For `gulp dev-deploy --rush` to work
properly, ensure your entire enlistment is built.
