# [sp-client](../../../README.md) › Onboarding › Setting Up Your Dev Box

This document describes how to configure your dev machine to build the projects
in the **sp-client** git repository.

## Join the e-mail groups

First, make sure you've subscribed to the relevant discussion groups listed on the
[Support Contacts](./SupportContacts,md) page.  This is where you can find about
any known issues, and ask for help if you encounter problems.


## Installing the Tools

1.  Install 64-bit Git from here: https://git-scm.com/

    Recommended installer settings for Windows:

    - Disable "Windows Explorer integration"

    - "Use Git from from the Windows Command Prompt"

    - "Checkout Windows-style, commit Unix-style line endings"

    *NOTE: The Git installer includes a terrible "Git Gui" application, that adds
    terrible Windows Explorer context menu commands.  You probably don't
    want that - it is not very good!*

2.  It is strongly recommend that you install a graphical tool for Git, rather
    than trying to view source code diffs as cryptic ASCII art in the console.
    The most common choices:

    - [SourceTree](https://www.sourcetreeapp.com/) - freeware (but requires you to
      register your e-mail address), standalone window.  For people who like
      something simple and shrink-wrapped.

    - [TortoiseGit](https://tortoisegit.org/) - open source, integrated directly
      into Windows Explorer.  For people who like something advanced and flexible.
      (If you use TortoiseGit, you'll want to combine it with an industrial strength
      differ like
      [WinMerge](http://thoai-nguyen.blogspot.com/2012/03/setup-tortoise-git-and-winmerge.html)
      or [BeyondCompare](http://www.scootersoftware.com/support.php?zz=kb_vcs#tortoisegit).)

3.  Install the plugin to enable Git support for Redmond domain credentials.
    (More details including Mac OS X instructions are
    [here](https://onedrive.visualstudio.com/DefaultCollection/OneDriveWeb/_git/odsp-next?path=%2FREADME.md)).

    a. If you are using SourceTree, you will need to go in SourceTree Tools->Options->GIT(tab)]
       and click on "**Use System Git**" button.

    b. Make sure the git you are using is 2.6.3 or higher.  Run "`git --version`" to see
       your current version.  (See above instructions for installing Git.)

    c. Run the *Setup.exe* from here - make sure to use the latest release version

       https://github.com/Microsoft/Git-Credential-Manager-for-Windows/releases/

4.  Choose an IDE:

    - **VS Code**: If you like hip new stuff you can install
      [Visual Studio Code](https://code.visualstudio.com/),
      and then use File->Open to open the "D:\GitRepos\sp-client" folder.

        *NOTE: Several people have found that VSO's IntelliSense is significantly
        slower than regular Visual Studio.*

    - **Visual Studio**:  If you prefer the full Visual Studio 2017, then you
      MUST install the latest
      [TypeScript for Visual Studio 2017](https://www.typescriptlang.org/#download-links)
      plug-in.

      During installation of Visual Studio, we recommend to **install the C++ compiler option**.
      For the best experience, it's also recommended to install these plug-ins for Visual Studio:

      - [Markdown Editor](https://visualstudiogallery.msdn.microsoft.com/eaab33c3-437b-4918-8354-872dfe5d1bfe), which
        provides a visualizer for the markdown file format (e.g. when viewing/authoring README.md files)

      - [Trailing Whitespace Visualizer](https://visualstudiogallery.msdn.microsoft.com/a204e29b-1778-4dae-affd-209bea658a59)
        to avoid tslint errors

5.  Install Node.js version 6.10.2 LTS or newer.  We recommend [nvm-windows](https://github.com/coreybutler/nvm-windows) (Windows) or [nvm](https://github.com/creationix/nvm) (Mac/Linux) so you can easily work with repos that require different Node.js engine versions.  (Don't use *nodist*.)

    **Instructions for NVM:**

    a. If you previously installed the mainline from https://nodejs.org/, first uninstall it, and remove all traces of NPM and Node.js from your PATH, "C:\Program Files\", and "%APPDATA%" folders.

    b. Determine the latest "[Long Term Stable (LTS)](https://nodejs.org/en/download/releases/)" version number for NodeJS.  (Do NOT choose the buggy "current" release.)  To install, run something like "`nvm install 6.10.2`" and then "`nvm use 6.10.2`".

    c. Make sure that "C:\Program Files\nodejs" is added to your system PATH.

6.  After installing Note.js LTS, run "`npm install -g npm`" to upgrade your NPM tool to the latest version.

7.  Use your Git tool to clone the **sp-client** repo from this URL (which is
    conveniently also the web site for viewing our VSO work items and other repos):

    https://onedrive.visualstudio.com/DefaultCollection/SPPPlat/_git/sp-client

    It's recommended that your clone target folder should be shallow and without spaces,
    for example:

    *D:\GitRepos\sp-client*

8.  We sometimes use some NPM libraries whose maintainers neglect to provide
    prebuilt binaries.  Instead, they think it's totally cool to invoke the
    full *node-gyp* toolchain to execute a native compiler during the "npm install"
    script.  When this happens, you need a Python 2.x and a working C++ compiler.
    The quick way to satisfy these dependencies is to install:

    `npm install -g windows-build-tools`


## Configuring your NPM registry

1. Create or open your global **.npmrc** file, which exists in your home directory.  Example filename:

   *C:\\Users\\billg\\.npmrc*

2. Delete any lines that you don't remember adding there for some good reason.  Add these lines to your **.npmrc** file (that you opened in step #1):

    ```
    registry=https://onedrive.pkgs.visualstudio.com/_packaging/odsp-npm/npm/registry/
    //onedrive.pkgs.visualstudio.com/_packaging/odsp-npm/npm/registry/:always-auth=true
    ```

3. Go to [https://onedrive.visualstudio.com/SPPPlat/_packaging?feedName=odsp-npm](https://onedrive.visualstudio.com/SPPPlat/_packaging?feedName=odsp-npm).  Make sure the **top left drop-down is set to "odsp-npm"**. Then click "Connect to feed", click "npm" on the left nav.

4. **Ignore instructions for "Windows users".** Instead, click **"Generate npm credentials"**, and append the generated token text to your **.npmrc** file (that you opened in step #1).

5. Your final file contents should look like this:

    **Example only (DO NOT COPY THIS!):**
    ```
    registry=https://onedrive.pkgs.visualstudio.com/_packaging/odsp-npm/npm/registry
    always-auth=true
    ; Treat this auth token like a password. Do not share it with anyone, including Microsoft support.
    ; begin auth token
    //onedrive.pkgs.visualstudio.com/_packaging/odsp-npm/npm/registry/:_authToken=<your generated auth token>
    ; end auth token
    ```

6. **If you have auth issues:** Duplicate the "`_authToken`" line and delete the substring "`registry/`" as shown below.  (This is a workaround for bugs that occur on some Windows PC's.)

    ```
    registry=https://onedrive.pkgs.visualstudio.com/_packaging/odsp-npm/npm/registry
    always-auth=true
    ; Treat this auth token like a password. Do not share it with anyone, including Microsoft support.
    ; begin auth token
    //onedrive.pkgs.visualstudio.com/_packaging/odsp-npm/npm/registry/:_authToken=<your generated auth token>
    //onedrive.pkgs.visualstudio.com/_packaging/odsp-npm/npm/:_authToken=<your generated auth token>
    ; end auth token
    ```

7. Clean your NPM cache (which is always required after any update to .npmrc or Node.js):

    ```
    npm cache clean
    ```

8. Run the following to verify that your registry is configured correctly:

    ```
    npm view @ms/odsp-utilities
    ```

9. The token expires in 90 days and you will get 401s at that time. Just re-run those steps to update your token.

## A word about "npm install -g"

Generally NPM dependencies are specified in the package.json file for each Git repo.
That way, when another dev clones our git repo and runs "`npm install`", the packages
will be installed/updated automatically.  This is great.

However, sometimes you want a command to be available everywhere (e.g. "rush" or "gulp").
You can install it globally using "`npm install -g`".  However, you will be stuck on
whatever version got installed, unless you specifically remember to upgrade it.  When
different people are using different versions, confusion can result.  Thus, if you
encounter build issues, please start by checking whether any of your globally installed
tools might be out of date or conflicting with the repo.

To see what you accidentally installed globally, run "`npm ls -g -depth=0`".


## Next Steps

Great job!  Now that your dev box is set up, please continue to [Building Projects](./BuildingProjects.md).
