# [sp-client](../../../README.md) › Procedures › Publishing API Docs

The SPFx API documentation pipeline looks like this:

*.ts ➡ **[GCB/API Extractor]** ➡ *.api.json ➡ **[api-documenter]** ➡ *.yml ➡ **[docfx]** ➡ [web site](https://docs.microsoft.com/en-us/javascript/api/sp-application-base)

The procedure for releasing documentation works like this:

1. Enlist and rebuild the **web-build tools** repo.

2. Enlist and rebuild the **sp-client** repo.

3. Copy the SPFx *.api.json files from the various projects of interest into this folder:

   > `web-build-tools\libraries\api-documenter\files\input`

   The list of files to copy is tracked
   [in this folder](https://github.com/SharePoint/sp-dev-docs-ref-typescript/tree/master/InputFiles)
   in the docs repo. Don’t copy any extra packages beyond these 11, since our other packages aren’t
   ready to be published yet.

4. Run the **api-documenter tool**.  There’s a handy batch file here:

   > `web-build-tools\libraries\api-documenter\run.cmd`

   (The YAML files will be generated in the "files\output" folder.)

5. Clone the docs repo using this URL:

   > [https://github.com/MicrosoftDocs/officeDocs-SP-Framework](https://github.com/MicrosoftDocs/officeDocs-SP-Framework)

   Login using your GitHub account.  If you don’t have access, **davech** can give you permissions.

6. Copy all the YAML output files from this folder:

   > `web-build-tools\libraries\api-documenter\files\yaml`

   ...and paste it into this folder:

   > `officeDocs-SP-Framework\SP-Framework`

   Also update this folder with a copy of your *.api.json input files, so we have a record of them:

   > `officeDocs-SP-Framework\InputFiles`

7. Commit these changes and push to Git.  You push directly to master, so check the diff carefully to make sure it looks right.

8. Confirm that the workflow succeeded.  The OPS web site is here:  (If you don’t have access, ask davech)

   > [https://ops.microsoft.com/#/sites/Docs/docsets/SP-Framework?tabName=builds](https://ops.microsoft.com/#/sites/Docs/docsets/SP-Framework?tabName=builds)

   (Our docset is “Sp-Framework”.  You should star it as a favorite.)  Then click on the “Builds” tab, and
   you should see a job with your Git commit title.  When it completes, click “Report” to confirm that it
   has 0 errors and 0 warnings.

9. Lastly, go to the review web site and make sure the results look good.  It’s here:

   > [https://review.docs.microsoft.com/en-us/sharepoint/typescript/spfx/?branch=master](https://review.docs.microsoft.com/en-us/sharepoint/typescript/spfx/?branch=master)

   *NOTE: It may take a few minutes for the CDN to sync.  Also, I usually have to press CTRL+F5 on each*
   *page to clear my browser cache.*

10. When you're ready to push these changes to the live web site, contact **davech** or **vesaj** for help.

