# [sp-client](../../../README.md) › Procedures › Publishing JSON Schemas

There are a number of JSON schemas in sp-client, and in order for intellisense to work correctly, they must be
referenced in a `$schema` property of a JSON file. For example:

```JSON
{
  "$schema": "https://dev.office.com/json-schemas/spfx-build/config.schema.json",
  "entries": [
    {
      "entry": "./lib/1737e38a319916b2cf2c48068c6f59b3/f52fabc2e4da8e3a8a819146b3171c10.js",
      "manifest": "./src/1737e38a319916b2cf2c48068c6f59b3/f52fabc2e4da8e3a8a819146b3171c10.manifest.json",
      "outputPath": "./dist/2beef62649d3bb080a38516a884da129.js"
    }
  ],
  "localizedResources": {
    "b988e989abebe80181700fd1b390ad5f": "b988e989abebe80181700fd1b390ad5f/{locale}.js"
  }
}
```

Notice the `$schema` property refers to a URL. Visual Studio Code accepts relative file paths in this property,
such as paths referring to the `node_modules` folder, but other editors do not and the JSON schema specification
does not. This means that we need to publish our schemas online. As you may have noticed in the example above,
the schema is hosted under `https://dev.office.com/json-schemas/`. This example is a Version 1.0 `config.json` file,
which is used by the build system, so its schema is under the `spfx-build` directory of that path. Our naming
convention for schema files is `{file}[.{version}].schema.json`, where the `{version}` is optional. For example, the
Version 2.0 `config.json` schema file is called `config.2.0.schema.json`.

To deploy a schema to https://dev.office.com/json-schemas/:
1. Clone the https://github.com/OfficeDev/devofficecom repository
2. Check out the `devx` branch
3. Fork the branch (to something like `youralias/devx-updating-my-schema`)
4. Navigate to the `src/Orchard.Web/Modules/DevOffice.SharePoint/` directory. This is the SharePoint schema hosting
project
6. In the `src/Orchard.Web/Modules/DevOffice.SharePoint/` directory, notice the `json-schemas` subdirectory. This
is the same directory as is in the URL mentioned above. Inside the `json-schemas` directory, find the directory in
which your schema should live and drop the `*.schema.json` file there.
7. Open the `src/Orchard.Web/Modules/DevOffice.SharePoint/DevOffice.SharePoint.csproj` file and add a `<Content ... />`
node for your schema file under the `<ItemGroup />` with all of the other schema files. This step ensures that
your schema file is deployed.
8. Commit your changes to your branch and push them to GitHub.
9. Open a pull request merging your branch into the `devx` branch and find an approver. Contact @VesaJ to find out
who the approvers are.
10. Once your pull request is accepted and merged in, expect your change to be deployed within two weeks.
