# [sp-client](../../../README.md) › Procedures › Shrinkwrap Reviews

Read this if you made a change that updated **npm-shrinkwrap.json**.

## How to update package.json when using Rush

The *sp-client* repository is built using [Rush](https://aka.ms/rush) which adds some special
requirements when making a change to your project's *package.json* file.

If you want to make changes via the NPM tool (e.g. `npm install --save`), the Rush symlinks
will confuse this tool.  You need to first remove the symlinks, like this:

> C:\GitRepos\sp-client\spfx-core\my-project> **rush unlink**
>
> C:\GitRepos\sp-client\spfx-core\my-project> **npm install --save foo**

In general, the overall process goes like this:

1. If you are introducing a new package that hasn't been used in this repo before, first
   follow the [How To Add A Package Dependency](../HowTo/AddAPackageDependency.md)
   guidance
2. Edit *package.json* manually, or via the commands shown above.
3. Run `rush generate` or `rush generate --lazy`
4. Test your changes, go back to step #1 if needed
5. Run `rush generate` one last time (if you used `--lazy` before)
6. Commit your changes to Git
7. Get approval from the "**Shrinkwrap Approvers**" group

Some notes about Rush generate:

- The `--lazy` option causes `rush generate` to run faster by avoiding the need to
  delete and rebuild your "node_modules" folder from scratch.  This saves time while
  debugging, but it deletes your *npm-shrinkwrap.json* file.  Before merging a PR
  with *package.json* changes, you must run `npm generate` without the `--lazy` option.

- When using `rush link`, your project's *node_modules* folder is NOT determined
  by your *package.json* file, but rather by the shrinkwrapped state from the *common* folder

- Thus, if you forget to run `rush generate` and commit your changes to Git,
  then `rush install` will continue to install the old version (until the next
  time someone runs `rush generate`); the tools currently don't check for this
  mistake

- This shrinkwrapped state is updated by running `rush generate`; as a byproduct,
  it will also upgrade any unconstrained versions (e.g. using a "~" pattern).
  *You need to do appropriate test validation for any changes resulting from
  these upgrades!*

- The `rush generate` command is a superset of `rush install`; after running
  `rush generate`, you can proceed directly to running `rush link`

## "Shrinkwrap Approvers" checklist

If you are in the "** Shrinkwrap Approvers**" VSO group and have been asked
to review someone's PR, here is the checklist of issues we look for:

- Are the *package.json* version references appropriately specified?  Generally
  we should be using locked versions (e.g. "1.2.3") for important external libraries
  (to avoid accidental regressions during `rush generate`), but for local
  libraries we use [tilde versions](https://github.com/npm/node-semver) (e.g. "~1.2.3").
  A "local library" is any project that is listed in the **sp-client**
  [rush.json](../../../rush.json) config file.
  **All `@types` packages should be using locked versions.**

- Are the version references consistent across all projects in our repository?
  For example, if **sp-pages** is using **react** 0.14.8, but **sp-platform-bundles**
  is using **react** 0.14.7, this has two bad consequences:

  1. Extra side-by-side versions will be introduced in the *npm-shrinkwrap.json* tree,
     which slows down our CI builds because `npm install` will do more work.
     Certain toolchain libraries such as **gulp-core-build** have massive dependency
     trees, so side-by-side versioning can create numerous duplicates.

  2. **sp-pages** will be linked to version 0.14.7 at runtime, which is technically
     incompatible with the version that it claimed to require.  This is because
     our **react** bundle uses *runtime versioning*, i.e. all components on the
     browser page get linked to the "latest" version of the library (and our
     module loader doesn't detect mismatches yet).  For libraries that don't use
     runtime versioning (e.g. a private copy of **odsp-shared-react** embedded
     in a web part's bundle), incrementing only one version avoids the need to
     test all the other projects, which can be advantageous.  However, this
     practice has other problems such as runtime bloat and "I thought we already
     fixed that bug last week!?"

- For packages which are shared across multiple Microsoft repositories, such as
  `odsp-datasources`, we should not be introducing side-by-side versions. Additionally,
  we should not pull newer versions of shared repositories if they introduce side-by-side
  versions of SPFx packages.

- When looking at the diff for *npm-shrinkwrap.json*, are there a lot of new
  entries being added?  This is bad, because each entry slows down our CI builds
  by making "npm install" do more work.  It usually results from side-by-side
  versioning as discussed above, however it can also indicate that the person
  is introducing a new library we didn't see before, in which case you should
  make sure they follow the [How To Add A Package Dependency](../HowTo/AddAPackageDependency.md)
  guidance.

- If browser libraries are being upgraded, ask the person whether they performed
  appropriate testing.  The browser libraries are a relatively short list that
  mostly appear in this file:
  [browser-approved-packages.json](../../../common/config/rush/browser-approved-packages.json)

  To determine the test surface, the person should (1) look at the library changelog or
  git history to determine what has changed, then (2) determine which *sp-client*
  projects are using the library.  If many web parts or projects could be
  impacted, the common practice is to add the owners to the PR and then send
  an e-mail with a deadline such as "Please test your project to make sure this
  upgrade doesn't break it; I will merge the PR tomorrow at 5pm, unless someone
  objects."

What NOT to worry about in the diff for *npm-shrinkwrap.json*:

- URLs that flip/flop between "npm-virtual" tand "npm-local".  This is a spurious
  change resulting from an Artifactory issue, where people with publishing permissions
  have a different REST endpoint specified in their .npmrc file.

- Random version changes for packages that do not run in the browser.  For example,
  when *gulp* gets installed, it brings in a massive tree of indirect dependencies
  via [tilde or caret versions](https://github.com/npm/node-semver). We will get
  the latest version whenever someone runs `rush generate`.  In a tree of thousands
  of packages, there are likely to be lots of small upgrades from week to week.
  Fortunately, the risk of regression for toolchain libraries is negligble,
  because if a regression does occur, 99% of the time it will be a conspicuous
  build break, not a runtime bug that could impact real customers.
