# [sp-client](../../../README.md) › Reference › SPFx Assemblies

An SPFx assembly is a single file containing these things:

* the startup code for the SPFx framework
* the sp-loader package
* the sp-loader’s essential dependencies
* any additional packages that may be included to improve performance
* optionally, an application's package and its dependencies.

An SPFx assembly is a not conventional SPFx bundle, because bundles are loaded using the loader, whereas the assembly
is responsible for initializing the loader in addition to an application and its dependencies.

## Creating an assembly for an application

Strictly speaking, the "core component" of an assembly does not need to be an application, but in almost all cases
it will be. To create an assembly for an application, create a `bundle-assembly.json` file in the application project's
`/config` folder with these properties:

```JSON
{
  "assemblyId": "<the ID of the assembly, for example 'b5cca424-63d0-4e35-b6c5-794d04f61206'>",
  "assemblyName": "<the name of the assembly, for example 'sp-pages-assembly'>",
  "rootComponentId": "<the ID of the assembly's core component, for example 'b6917cb1-93a0-4b97-a84d-7cf49975d4ec'>"
}

```

This will produce an additional bundle in the `dist` folder with the name `assemblyName_<locale>.js` for each locale
(or only English in non-production builds) and a manifest for the assembly.

When you create a new assembly, please update the list in the following section, and make sure you add the assembly's
ID to the `tools/vsts-deploy/sources.json` file.

## Assemblies that exist in sp-client

- **Loader Assembly** *(sp-loader-assembly)*: Provides the minimal amount of components, limited to the components
    needed for the SPLoader. This assembly doesn't include other components required by the framework
    (i.e. sp-application-base).
- **Default Assembly** *(sp-default-assembly)*: Provides all components that the framework needs to load an
    application, without any specific application. The application (and its dependencies, if needed) will be loaded from
    their individual bundles.
- **Pages Assembly** *(sp-pages-assembly)*: Optimized for the sp-pages application. This doesn't include the preloaded
    webparts, which need to be loaded separately.
- **Classic Pages Assembly** *(sp-classic-page-assembly)*: Optimized for Classic Pages application. This doesn't include
    the webparts that the application is loading.
- **Workbench Assembly** *(sp-webpart-workbench-assembly)*: Optimized for the SPO-hosted workbench (workbench.aspx).
