# [sp-client](../../../README.md) › Reference › Telemetry

**Table of Contents**
- [Overview](#overview)
- [Data Types](#data-types)
  - [QOS](#qos)
  - [Engagement](#engagement)
  - [Performance Data](#performance-data)
  - [Trace](#trace)
- [View Data In Browser for Debugging](#view-data-in-browser-for-debugging)
  - [Browser Console](#browser-console)
  - [Dev Dashboard](#dev-dashboard)
- [View Data On Dashboards](#view-data-on-dashboards)
  - [BindDat](#bingdat)
  - [Aria Toolchain](#aria-toolchain)
    - [QOS Dashboards On MDM](#qos-dashboards-on-mdm)
    - [Engagment Dashboards On Aria or Interana](#engagment-dashboards-on-aria-or-interana)
      - [Aria](#aria)
      - [Interana](#interana)
    - [Performance Dashboards On MDM](#performance-dashboards-on-mdm)
    - [Trace on MDS or Kusto](#trace-on-mds-or-kusto)
      - [MDS](#mds)
      - [Kusto](#kusto)
- [Realtime Monitoring](#realtime-monitoring)
  - [SMART](#smart)
  - [MDM Alerts](#mdm-alerts)

# Overview
There are four types of telemetry data collected in the client side framework ([QOS](#qos), [Engagement](#engagement), [Performance Data](#performance-data), [Trace](#trace)).
There are two ways to view them on the client side ([Browser Console](#browser-console) and [Dev Dashboard](#dev-dashboard)). There are various ways
 to host dashboards for them (bingdat, Aria, Geneva, Interana, and etc). The recommendations about what tools to use is in the [View Data On Dashboards](#view-data-on-dashboards) session.

# Data Types
Different types of data serve different purposes, and are collected using different APIs.

## QOS

What is QOS and how we measure it? [Quality of Service](https://msft.spoppe.com/teams/SPGroups/monitoring/_layouts/15/start.aspx#/Wiki/Reliability%20Monitoring.aspx)

Make sure you know ACE vs QOS when using BingDat, because ACE is what matters. Naming about QOS on MDM is under construction.

Use [QosMonitor](../../libraries/sp-telemetry/src/Api/QosMonitor.ts) to instrument QOS events in sp-client.

ODB has a note on how to use QOS event. We should align with it because we share the bingdat dashboard logic. [How to use QOS event](https://msft.spoppe.com/sites/spfearch/_layouts/15/WopiFrame.aspx?sourcedoc={3f043feb-461a-48cb-a94c-83c3f8cd05f6}&action=edit&wd=target%28OneDrive%20Next%2Eone%7C449689C6-4948-4E0E-A8E9-B49EAED74A7F%2FODB%3A%20How%20to%20use%20Qos%20event%7C4ED29EFE-4E43-4590-AFFE-FA70444D44AE%2F%29)


## Engagement

Engagement events show how our product is used by users.

Use [EngagementLogger](../../libraries/sp-telemetry/src/Api/Engagement/EngagementLogger.ts) to instrument engagement events in sp-client.

ODB has a note on how to use engagement events. We should also aligh with it because we share the bingdat dashboard logic. [How to use Engagement event](https://msft.spoppe.com/sites/spfearch/_layouts/15/WopiFrame.aspx?sourcedoc={3f043feb-461a-48cb-a94c-83c3f8cd05f6}&action=edit&wd=target%28OneDrive%20Next%2Eone%7C449689C6-4948-4E0E-A8E9-B49EAED74A7F%2FODB%3A%20How%20to%20use%20Qos%20event%7C4ED29EFE-4E43-4590-AFFE-FA70444D44AE%2F%29)


## Performance Data

Performance data contains both server side perf data (passed from server side as part of context) and client perf data.

Use [PerformanceLogger](../..//libraries/sp-telemetry/src/Api/PerformanceLogger.ts)
(a wrapper of [RumOneLogger](https://onedrive.visualstudio.com/OneDriveWeb/_git/odsp-utilities?path=%2Fsrc%2Flocal%2Flogging%2Frumone%2FRUMOneLogger.ts&version=GBmaster&_a=contents))
to instrument performance data in sp-client.

[Performance data format](https://onedrive.visualstudio.com/OneDriveWeb/_git/odsp-utilities?path=%2Fsrc%2Flocal%2Flogging%2Frumone%2FRUMOneSLAPI.ts&version=GBmaster&_a=contents)

## Trace

There are two types of logs. One is from the [TraceLogger](../../libraries/sp-telemetry/src/Api/Trace/TraceLogger.ts)
 which is used by first-party developers, and one is from [Log](../../spfx-core/sp-core-library/src/log/Log.ts) which is used
 by third-party developers. Logs from TraceLogger are sent back to backend for data analysis, while logs from Log are not
 sent back and only kept at the client side.

# View Data In Browser for Debugging

Sometimes a developer needs to see what data is collected for a certain page.

## Browser Console

Browser console only shows logs by default.

To show performance data, run:

        window.sessionStorage["enableRUMOneDebugging"] = true

To show QOS and Engagement data, run:

        window.disableBeaconLogToConsole = false


## Dev Dashboard
After a page is loaded, the key combination Ctrl+shift+` brings out Devdashboard. Devdashboard shows manifest definitions,
performance data for that page, and logs (coming soon).

# View Data On Dashboards

Telemetry data in sp-client is doing double dispatch. It is sent to SLAPI/Cosmos pipeline and Aria Toolchain. SLAPI pipeline is going away when Aria toolchain is fully ready.

SLAPI/Cosmos pipeline stores data in Cosmos which provides long-term storage, and reports data on BingDat dashboards. The reports are collected via Cosmos queries which runs one a day. It is not real-time.
Bingdat dashboards have existed for a while and are fairly stable. They are relatively hard to create or modify.

Aria toolchain contains Aria, Kusto, MDM, MDS, Interana, COSMOS, and etc. It is being developed by MI6 team,  and is the product for the future.
Aria collects data from many client platforms, and forwards data to its tools chains. The data on Aria toolchain provides semi real time monitoring (minutes of delay). Most of the tools only provide short-term data storage, except COSMOS.

## bingdat

Our existing Bingdat dashboards:

* QOS [SPFx](http://bingdat40/d/sppplat/Scenarios.html?Team=SPPPLAT&View=SPFX&Metric=1)

* QOS [Web Parts](http://bingdat40/d/wex/Scenarios.html?team=WEX&view=WebParts&bd_metric=MQ2)

* QOS [SPPages](http://bingdat40/d/wex/Scenarios.html?Team=WEX&View=SPPages&bd_metric=MQ2)

* QOS [Canvas](http://bingdat40/d/WEX/Scenarios.html?team=WEX&view=Canvas&bd_metric=MQ2)

* Performance [EDog](http://bingdat40/d/wex/SPPagesPerformance-edog.html?env=edog),&nbsp;
[MSIT](http://bingdat40/d/wex/SPPagesPerformance-msit.html?env=msit),&nbsp;
[Prod](http://bingdat40/d/wex/SPPagesPerformance-prod.html?env=prod)

* Usage/Engagement [SPPages](http://bingdat40/d/spx/v2AppHits_edog.html?env=edog&bd_appname=U1BQYWdlcw2)


Introduction on [bingdat](https://msft.spoppe.com/teams/SPGroups/monitoring/_layouts/15/start.aspx#/Wiki/Home.aspx)

Introduction on [how to set up a bingdat dashboard](https://msft.spoppe.com/teams/SPGroups/monitoring/_layouts/15/start.aspx#/Wiki/How%20to%20set%20up%20your%20own%20BingDAT%20dashboard.aspx)

To add more QOS events to the existing QOS dashboard: believe it or not, you need to add to the [list](https://msft.spoppe.com/teams/SPGroups/monitoring/_layouts/15/start.aspx#/Lists/Scenario%20Monitoring%20StartSuccess%20IDs/AllItems.aspx)
.  Choose SPPPlat or WEX as the team. It decides on which board the data will show. Do not forget to add the failure Id to the failure Id list as well.

To update bingdat performance dashboard:
1. Update "\\\\bingdat40\TemplateData\CustomCode\spo\SPPagesPerformance.script". Test the scope script first to make sure it dumps out correct data.
2. Update template schema on [SPPages performance -edog](http://bingdat40/WebForms/Templates/EditTemplate.aspx?bdtg=0&bdt=1337),
[SPPages performance -msit](http://bingdat40/WebForms/Templates/EditTemplate.aspx?bdtg=0&bdt=1338),
[SPPages performance -prod](http://bingdat40/WebForms/Templates/EditTemplate.aspx?bdtg=0&bdt=1339) if the columns of the scope script
change. You need to add yourself as owner to those three templates first.
3. Update "\\\\bingdat40\Dashboards\wex\SPPagesPerformance.html", and test if it renders fine on the dashboards.

Note: SPPPLAT team and WEX currently share one performance dashboard and our bingdat dashboards align
with [SPX dashboard](http://bingdat40/d/spx), so most of the work is shared.

## Aria Toolchain

The architecture of Aria toolchain can be found [here](#).
The data distribution on Aria toolchain is:

 - [Aria](https://aria.microsoft.com/) and [Kusto](https://aria.microsoft.com/kusto?tenant=930040d5f2d14c649c8f4a22a062a2af) has all telemetry data.

 - MDM has QOS and Performance data.

 - MDS has QOS and Trace(coming) data.

 - Interana has Engagement data.

 - COSMOS has Engagemetn data.

EDog and MSIT data goes to Aria PPE tenant [ODSP - SPPPLAT_WEX -PPE](https://aria.microsoft.com/dashboards/3633376f-ae1f-4f31-bb84-2cbd4eebca3f/?tenant=930040d5f2d14c649c8f4a22a062a2af&timeframe=Last1Week&granularity=OneDay).


Prod data goes to Aria Prod tenant  [ODSP - SPPPLAT_WEX - PROD](#https://aria.microsoft.com/dashboards/f2b66973-909e-40c3-8c80-e10787814ca1/2b82f6e2-ffb3-40d5-a52b-84cae37770cd?timeframe=Last1Week&granularity=OneDay).

[Playground boards for WEX](https://aria.microsoft.com/dashboards/a139408b-5ad2-481a-8922-628ef105cb7b/?timeframe=Last1Week&granularity=OneDay)  WEX people are welcome to add dashboards for investigation or monitoring.

[Playground boards for SPPPLAT](https://aria.microsoft.com/dashboards/d0bdaa16-df46-431c-aea2-daa3f7a621f3/?timeframe=Last1Week&granularity=OneDay) SPPPLAT people are welcome to add dashboards here for investigation or monitoring.

### QOS Dashboards On MDM

[SPPPLAT QOS](https://jarvis-west.dc.ad.msft.net/dashboard/ODSP_SPPPLATWeb/ClientQos/SPPPLAT%2520QoS%2520Last%252024%2520hours)

Operation QOS https://jarvis-west.dc.ad.msft.net/dashboard/ODSP_SPPPLATWeb/ClientQos/Operation(Name)%2520Qos
In Operation QOS board, click "Override" to add your operation name to show QOS about your operation.

[SPPPLAT SPPages QOS](https://jarvis-west.dc.ad.msft.net/dashboard/ODSP_SPPPLATWeb/ClientQos/SPPPLAT%2520SPPages%2520QoS%2520last%252024%2520hours)

[SPPPLAT Classic Pages QOS](https://jarvis-west.dc.ad.msft.net/dashboard/ODSP_SPPPLATWeb/ClientQos/SPPPLAT%2520ClassicPage%2520QoS%2520last%252024%2520hours)

[SPPPLAT Workbench QOS](https://jarvis-west.dc.ad.msft.net/dashboard/ODSP_SPPPLATWeb/ClientQos/SPPPLAT%2520WorkBench%2520QoS%2520last%252024%2520hours)


### Engagment Dashboards On Aria or Interana

Engagement dashboards can be built on Aria or Interana or COSMOS. The long term recommendation from MI6 is to use Interana. For the short-term, people can use Aria or query data on COSMOS.

#### Aria
Existing dashboards on Aria:

[WebPart Usage](https://aria.microsoft.com/dashboards/f2b66973-909e-40c3-8c80-e10787814ca1/21079c05-86a4-450a-8f48-a2a65f413aa1?timeframe=Last60Day&granularity=OneDay)

[Home Page Usage](https://aria.microsoft.com/dashboards/f2b66973-909e-40c3-8c80-e10787814ca1/48dc2a68-7af5-419e-a4cf-556ff84a303d?timeframe=Last60Day&granularity=OneDay)

[Team News](https://aria.microsoft.com/dashboards/f2b66973-909e-40c3-8c80-e10787814ca1/ef208d48-0171-46a1-a0d3-236cbf5d9dda?timeframe=Last60Day&granularity=OneDay)

#### Interana
We do not have any offical dashboards on [Interana](https://onedrive.interana.com/) yet.

### Performance Dashboards On MDM

Existing performance dashboards:

[Prod](https://jarvis-west.dc.ad.msft.net/dashboard/ODSP_SPPPLATWeb/Performance/Prod)
[MSIT](https://jarvis-west.dc.ad.msft.net/dashboard/ODSP_SPPPLATWeb/Performance/Prodbubble)
[EDog](https://jarvis-west.dc.ad.msft.net/dashboard/ODSP_SPPPLATWeb/Performance/EDogPerf)

[MI6 Perf Board](https://jarvis-west.dc.ad.msft.net/dashboard/share/1DBFB4E?overrides=[{"query":"//*[id='Scenario']","key":"value","replacement":"ODBNext.Files-Files"}]%20)

### Trace on MDS or Kusto

Both MDS and Kusto provide powerful query capability on trace logs. MDS will be the recommended tool when Trace gets into it. Before that, use Kusto to query.

#### MDS
Find MDS here: [MDS](https://jarvis-west.dc.ad.msft.net/82C04D26)


#### Kusto

Kusto Desktop app - Kusto Explorer

Kusto desktop app is very powerful and can be used to analyze the short-term telemetry data.
Install [kusto](https://aria.microsoft.com/kusto?tenant=930040d5f2d14c649c8f4a22a062a2af)

To use it:
- launch Kusto first
- then right click "Connections"
- add data source "https://kusto.aria.microsoft.com:443".
- Choose Client Security: AAD-Federated.

Then you will be able to see all the tenants you have access.
The names of our tenants start with "ODSP - SPPPLAT_WEX". We have two active tenants: "ODSP - SPPPLAT_WEX - PPE" and
"ODSP - SPPPLAT_WEX - Prod". PPE contains telemetry from EDog and MSIT environment. Telemetry from all other environments goes to
the Prod tenant.

Each tenant has six tables:
- ev_qos => QOS events
- ev_engagement => Engagement events
- ev_rumonedataupload => Performance events
- ev_trace => verbose log, uploaded when there is an error.
- ev_beacon => beacon events for logging health

Query it like SQL. Language help can be found [here](https://kusto.azurewebsites.net/docs/queryLanguage/query_language.html).

Example queries:

* List all the errors in the past 10 minutes on prod

        ev_trace | where EventInfo_Time > ago(10m)
        | where Environment == "prod"
        | where WebLog_Type_CaughtError == 1 or WebLog_Type_QosError == 1

* Search for all the traces for one correlationId (remember to set a time range for fast query)

        ev_trace | where EventInfo_Time > ago(2h)
        | where Environment == "prod"
        | where AppInfo_Session == "b20dde9d-d04d-3000-f34b-95edcabdcd6c"

* Calculate EUPL p75 for difference scenarios:

        ev_rumonedataupload
        | where EventInfo_Time >= ago(10d)
        | extend  eupl = toint(RUMOneDataUpload_dictionary_EUPL)
        | where eupl < 100000
        | summarize percentile(eupl, 75)  by bin(todatetime(EventInfo_Time), 4h), RUMOneDataUpload_dictionary_ScenarioId
        | render timechart


# Realtime Monitoring

Existing alerts are based on SMART alerts. MI6 team is migrating SMART alerts to MDM alerts.

## SMART

SMART monitors exist at:

        \\\\daddev\Office\16.0\[version]\src\spo\RealTimeMonitoring\SMART\SmartMonitors

To view them in action, visit [spo-rt](http://spo-rt/RealUser_ScenarioQoS).

## MDM Alerts

MDM alerts can be found [here](https://jarvis-west.dc.ad.msft.net/#/health):