# [sp-client](../../../README.md) › Reference › Third Party Client-side Applications

This document describes how to run the Article Site demo project on
a SharePoint TDS box.  It illustrates the basic principles for running any
client-side application project.


## Installing and Running the Article Site Demo

1.  Create a SharePoint test box in one of these ways:
    - Use [TDSWeb](http://aka.ms/tdsweb/) to allocate a VM using the
      "**SPORel MOSS-1M Farm (Windows Server 2012 R2)**" config
    - You can also the "**SP16 SPO OneBox - Windows 2012 R2**" config, although the URLs below
      will be slightly different
    - You can also use "BB local" to pave/install a 1mfarm box, which gives you
      a debug build

2.  Enable the **ExpFeatureId.SppplatPortalFrameworkFeature** flight by
    running these PowerShell commands on your SharePoint test box:

    ```powershell
    Add-PSSnapIn *sharepoint*
    $farm = Get-SPFarm
    $farm.ServerDebugFlags.Add(50005)
    $farm.Update()
    ```

3.  (Optional) If you want to run the **announcement-list-demo** or will be
    creating a "list-based" client-side application, you will need to copy
    the two ASPX files from this folder:

    [SupportFiles](./SupportFiles)

    ...to this folder on your test box:

    *C:\Program Files\Common Files\microsoft shared\Web Server Extensions\16\TEMPLATE\Pages*

4.  Create a new team site, or use the default site that 1mfarm provisions
    at the root URL:

    http://example.com/

    Log in as the system account (e.g. pkmacct or dozer), and then add yourself as
    a site collection administrator for both the team site and also the App Catalog site
    at this URL:

    http://example.com/sites/AppCatalog/AppCatalog

    Then you need to login using your personal Redmond user account, since the
    system account is forbidden from installing apps.  (If you find that annoying,
    hassle Graham about it heheh.)  If you don't have a Windows session that is already
    logged in as the system user, you can use this handy query string to force the
    web browser to show the login dialog:

    http://example.com/_windows/default.aspx?ReturnUrl=&loginasanotheruser=true

    NOTE: Since Chrome constantly forgets your login info, you might also want to
    make a login bookmark that automatically redirects to your favorite page URL.
    You can use query parameters like this:

    http://example.com/_windows/default.aspx?ReturnUrl=/ClientSidePages/Welcome.aspx&loginasanotheruser=true

5.  Now we will try running the article-site-demo project.  If you didn't
    already follow the instructions from [HowToBuild.md](./HowToBuild.md),
    you should do that now.  Then, run this command to build the .sppkg solution
    that will provision the SharePoint content for the article-site-demo project:

    > C:\GitRepos\sp-client> **cd platform-samples\article-site-demo**
    > C:\GitRepos\sp-client\platform-samples\article-site-demo> **gulp package-solution**

    The output file will be in this folder:

    article-site-demo\sharepoint\solution\ **article-site-application.sppkg**

6.  Navigate to your App Catalog site, which should be a URL like this:

    http://example.com/sites/AppCatalog/AppCatalog

    *NOTE: If the App Catalog is missing, you can provision it using
    Central Administration -> Apps -> Manage App Catalog.*

    Now drag+drop your **article-site-application.sppkg** file from Step 5
    into the "Apps for SharePoint" list, then click the "Deploy" button.
    The "Valid App Package" field should show "Yes":

    ![App Catalog](../Images/HowToRunAnApplication/AppCatalog.png)

    (If not, you will need to look at the ULS logs to diagnose the problem
    with your Feature XML.)

7.  If you want to confirm that the application and web part manifests
    were registered properly, they are stored in a **hidden** SPList with a URL
    like this:

    http://example.com/sites/AppCatalog/Lists/ComponentManifests

8.  Now navigate to your team site URL such as this:

    http://example.com/

    Click Site Contents -> "add an app", and select the "Article Site"
    application that we registered in step 6.  Wait for the timer job to finish
    installing (i.e. the "We're installing your app" message).  Then refresh
    the Site Contents page.  The "ClientSidePages" library should now exist,
    and contain a page called Welcome.aspx with a URL like
    `/ClientSidePages/Welcome.aspx`.

9.  Before clicking the URL, make sure to start the localhost web
    server by running this command in the **platform-samples\article-site-demo**
    folder:

    > gulp serve --nobrowser

10. In order to navigate to the URL using the framework libraries from "gulp serve"
    (rather than whatever the SharePoint box is configured to use), you need to add
    some query parameters to your URL like this:

    http://example.com/ClientSidePages/Welcome.aspx?noredir=true&moduleloader=https://localhost:4321/node_modules/@microsoft/sp-loader/dist/sp-loader.js&debugManifestsFile=https://localhost:4321/temp/manifests.js

    The individual parameters are:

    - *?noredir=true*: if an error occures, don't redirect to SharePoint's error page

    - *&moduleloader=...*: Use the locally built sp-loader.js

    - *&debugManifestsFile=...*: Use the locally built manifests, rather than querying the SharePoint server

    For a full list of our URL query parameters, see [FrameworkQueryParameters.md](./FrameworkQueryParameters.md).

11. The first time you run *article-site-demo*, you will see a dialog
    saying that the application wants to finish provisioning its content.

    ![SimpleProvisioner](../Images/HowToRunAnApplication/SimpleProvisioner.png)

    Click OK, then wait for it to finish, then click OK again.  Then press
    F5 to refresh the page.  The result should look like this:

    ![Article Site Demo](../Images/HowToRunAnApplication/ArticleSiteDemo.png)


## Alternate Method using a the Manifest Timer Job

TODO


## Troubleshooting

-   **Symptom: Browser is reporting 404 for scripts.**

    Note that the names "sp-client-preview" and "sp-client-projects" look very similar.
    Make sure you're running "gulp serve" in the right folder!

-   **Symptom: The page mysteriously stops working.**

    Make sure "gulp serve" is still running in the background.  It dies sometimes.

-   **Symptom: The page fails to render and the ULS logs show "InvalidOperationException"**

    You may need to redo the manifest setup step (Step #5).

-   **Symptom: The page fails to render and the ULS logs show "NotSupportedException"**

    You may have forgotten to enable the flight on your test box (Step #2)

-   **Symptom: The page is blank with the web browser console showing some weird
    error like "Potentially unhandled rejection"**

    Try using the Chrome browser.  Support for other browsers is sometimes lacking.

-   **Symptom: You get a strange 404 error even though the page URL seems totally legit**

    It may be that you created ClientApp.aspx using a different user account
    (e.g. pkmacct), but your browser is now logged in as yourself, and the
    publishing page is in a draft state.  For security reasons, in this situation
    SharePoint reports a 404 error to everyone except the person who originally
    created the page.  The workaround is to log in using the right account.
    The permanent fix is to "publish" the page (i.e. check in a Major Version).
