# [sp-client](../../../README.md) › Reference › Verifying Code Coverage

This document describes how we implement TypeScript code coverage in the **sp-client** repository.
Code coverage information is a key component of building a high quality code base.

# How does it work?

The code coverage enforcement is implemented inside [VerifyCoverageTask](https://onedrive.visualstudio.com/sppplat/_git/sp-client?path=%2Ftools%2Fsp-build-internal-tasks%2Fsrc%2FverifyCoverage%2FVerifyCoverageTask.ts&version=GBmaster).
The code coverage data is generated with open source packages [istanbul](https://www.npmjs.com/package/istanbul)
and [remap-istanbul](https://www.npmjs.com/package/remap-istanbul).

There're 2 enforcement levels:
- File level code coverage enforcement.
- Code churn level code coverage enforcement.

Code churn level code coverage enforcement will only check the new code written by you within a pull request.

# Configure code coverage enforcement for a project

1. Go to the "config" folder of a project, add or update the "verify-coverage.json" file.

  ```
  project/
    ├╴config/
      └╴verify-coverage.json
  ```

2. Set the enforcement threshold and enforced branch based on project needs. Here is an example:

  ```json
  {
    "verifyCodeChurnCoverage": true, // Enable code coverage enforcement for new code (optional, default is true).
    "codeChurnCoverageEnforcedBranch": "master", // Enforce code churn coverage for specified branch (if not set, code churn coverage enforcement will be skipped).
    "minimumCodeChurnCoverage": 80, // Threshold of code churn code coverage enforcement.

    "verifyFileCoverage": true, // Enable code coverage enforcement for files (optional, default is true).
    "minimumStatementCoverage": 55, // Threshold of statement coverage enforcement.

    "skipFoldersOrFiles": [] // Folders or files which will be skipped by enforcement (support regex pattern).
  }
  ```

  If actual code coverage / code churn code coverage is lower than the threshold, build will break.

  When enable code coverage enforcement on a project, if existing code doesn't have adequate code coverage,
  build will break, you may want to skip it temporarily to have build pass.

  The recommended code coverage threshold:
    - For file level: >= 55%
    - For code churn level: >= 80%

3. For all configuration possibilities, check ["tools/sp-build-internal-web/src/schemas/verify.schema.json"](https://onedrive.visualstudio.com/SPPPlat/SPPPlat%20Team/_git/sp-client?path=%2Ftools%2Fsp-build-internal-web%2Fsrc%2Fschemas%2Fverify.schema.json&version=GBmaster).

**Note**: your project needs to be built using '@ms/sp-build-internal-web' to enable code coverage enforcement.

# How to skip code coverage enforcement for pull request build on VSTS in case of emergency?

In VSTS sp-client-pullrequest build definition, set build variable `SKIP_COVERAGE_VERIFICATION` to `true`.

![Skip code coverage enforcement for pull request build on VSTS](../Images/CodeCoverageEnforcement/SkipCodeCoverageEnforcementOnVSTS.png)

**Note**: Only VSO admins can do this.

# Code churn code coverage insights and test results on SPO-RT CodeHealth dashboard

The code churn code coverage and test results will be collected from VSTS pull request build and master build.
Check [SPO-RT CodeHealth](http://spo-rt/CodeHealth/) dashboard to have the insights below:

- [VSTS code churn coverage rank](http://spo-rt/CodeHealth/VstsCodeChurnCoverageRank)
- [VSTS code churn coverage trend for a dev (change UserAlias parameter)](http://spo-rt/CodeHealth/VstsCodeChurnCoverageTrend?UserAlias=yolian)
- [VSTS failed tests rank](http://spo-rt/CodeHealth/VstsFailedTestsRank)