// @public
class A11yAttribute {
  constructor(appPrefix: string, type: A11yAttributeType, params?: string[], value?: string);
  public static getAllFromElement(appPrefix: string, element: HTMLElement): A11yAttribute[];
  public static getFromElementByType(appPrefix: string,
      element: HTMLElement,
      type: A11yAttributeType): A11yAttribute[];
  public static getFromElementOrParentsByType(appPrefix: string,
      element: HTMLElement,
      root: HTMLElement,
      type: A11yAttributeType): A11yAttribute;
  public static getPrefix(appPrefix: string, type: A11yAttributeType): string;
  public readonly name: string;
  public readonly params: string[];
  public readonly prefix: string;
  public setOnElement(element: HTMLElement): void;
  public readonly type: A11yAttributeType;
  public readonly value: string;
}

// @public
enum A11yAttributeType {
  AlertOnFocusIn,
  AlertOnFocusOut,
  Class,
  Id,
  NavigateByHierarchy,
  NavigateOnKey,
  NavigationIsland,
  SkipKeys,
  StopKeys
}

// @public
class A11yManager {
  public addFocusListener(element: HTMLElement,
      direction: 'inward' | 'outward',
      handler: (focusTransition: FocusTransition) => void): number;
  public alert(msg: string): void;
  public readonly config: IA11yManagerConfig;
  public static create(domElement: HTMLElement, config?: IA11yManagerConfig): A11yManager;
  public createA11yAttribute(type: A11yAttributeType, params?: string[], value?: string): A11yAttribute;
  public focusById(id: string): boolean;
  public focusInside(element: HTMLElement): boolean;
  public focusOutOf(element: HTMLElement): void;
  public focusTo(element: HTMLElement): boolean;
  public forgetSavedActiveElement(key: string): void;
  public getElementByA11yId(id: string, root?: HTMLElement): HTMLElement | undefined;
  public getElementsByA11yClass(className: string, root?: HTMLElement): HTMLElement[];
  public static getInstanceById(id: string): A11yManager | undefined;
  public getSavedActiveElement(key: string): HTMLElement;
  public hasFocus(): boolean;
  public readonly id: string;
  public static isElementManaged(domElement: HTMLElement): boolean;
  public manages(element: HTMLElement): boolean;
  public readonly prefix: string;
  public removeFocusListener(id: number): void;
  public restoreFocus(key: string): boolean;
  public readonly root: HTMLElement;
  public saveActiveElementAs(key: string): void;
  public saveLastActiveElementAs(key: string): void;
  public setA11yAttributesOnElement(attributes: A11yAttribute[], element: HTMLElement): void;
  public skipEvent(e: IKeyboardEvent): void;
  public undoFocus(): boolean;
}

// @public
class Focus {
  public static focusInside(element: HTMLElement): boolean;
  public static focusOutOf(element: HTMLElement, root: HTMLElement): boolean;
  public static focusTo(element: HTMLElement): boolean;
  public static getFirstFocusableChild(elem: HTMLElement, ignoreTabIndex: boolean = false): HTMLElement;
  public static getFocusableChildren(element: HTMLElement, ignoreTabIndex: boolean = false): HTMLElement[];
  public static getFocusableDescendants(element: HTMLElement,
      ignoreTabIndex: boolean = false,
      descendants?: HTMLElement[]): HTMLElement[];
  public static getFocusableParent(element: HTMLElement, root?: HTMLElement): HTMLElement;
  public static getFocusableSiblings(element: HTMLElement, root?: HTMLElement): HTMLElement[];
  public static getNextFocusableSibling(element: HTMLElement, root?: HTMLElement): HTMLElement;
  public static getPrevFocusableSibling(element: HTMLElement, root?: HTMLElement): HTMLElement;
  public static isElementFocusable(elem: HTMLElement, ignoreTabIndex: boolean = false): boolean;
}

// @public
class FocusTransition {
  constructor(src: HTMLElement,
      dest: HTMLElement,
      root?: HTMLElement);
  // (undocumented)
  public readonly dest: HTMLElement;
  // (undocumented)
  public forEachElementInPath(callback: (element: HTMLElement, isOutward: boolean) => void): void;
  // (undocumented)
  public readonly root: HTMLElement;
  // (undocumented)
  public readonly src: HTMLElement;
}

// @public (undocumented)
interface IA11yManagerConfig {
  // (undocumented)
  debug?: boolean;
  // (undocumented)
  useHierarchicalNavigation?: boolean;
}

// @public
interface IKeyboardEvent {
  // (undocumented)
  altKey: boolean;
  // (undocumented)
  ctrlKey: boolean;
  // (undocumented)
  keyCode: number;
  // (undocumented)
  preventDefault(): void;
  // (undocumented)
  shiftKey: boolean;
  // (undocumented)
  stopPropagation(): void;
  // (undocumented)
  target: EventTarget;
}

// @public
interface IModifierKeys {
  // (undocumented)
  alt?: boolean;
  // (undocumented)
  ctrl?: boolean;
  // (undocumented)
  shift?: boolean;
}

// @public
class Keyboard {
  public static isEnter(e: IKeyboardEvent): boolean;
  public static isEscape(e: IKeyboardEvent): boolean;
  public static isKey(keyCode: number,
      event: IKeyboardEvent,
      allowedModifiers?: IModifierKeys): boolean;
  public static isShiftTab(e: IKeyboardEvent): boolean;
  public static isTab(e: IKeyboardEvent): boolean;
}

// @public
class ScreenReader {
  public static alert(id: string, message: string, isVisible: boolean = false, className?: string): void;
}

