// @internal
class _DialogManagerConfiguration {
  public allowDialogs: boolean;
  public dispose(): void;
  public readonly domElement: HTMLDivElement;
  public initialize(domElement: HTMLDivElement,
      allowDialogs: boolean = true): void;
  // (undocumented)
  public static readonly instance: DialogManagerConfiguration;
}

// @internal
interface _IApplicationCustomizerContextParameters {
  sequence: number;
}

// @internal
interface _IPlaceholderContentParameters {
  // (undocumented)
  domElement: HTMLDivElement;
  // (undocumented)
  options?: IPlaceholderCreateContentOptions;
  // (undocumented)
  placeholder: Placeholder;
  // (undocumented)
  sequence: number;
}

// @internal
interface _IPlaceholderDefinition {
  domElement: HTMLDivElement;
  name: PlaceholderName;
}

// @internal
class _Placeholder {
  public constructor(definition: IPlaceholderDefinition, manager: PlaceholderManager);
  public createPlaceholderContent(sequence: number, options?: IPlaceholderCreateContentOptions): PlaceholderContent;
  // (undocumented)
  public dispose(): void;
  // (undocumented)
  public readonly name: PlaceholderName;
  // (undocumented)
  public removePlaceholderContent(placeholderContent: PlaceholderContent): void;
}

// @internal
class _PlaceholderManager {
  constructor(serviceScope: ServiceScope);
  public readonly changedEvent: Event<EventArgs>;
  public initializePlaceholders(definitions: IPlaceholderDefinition[]): void;
  public readonly placeholders: Placeholder[] | undefined;
  public registerPlaceholders(): void;
  public static serviceKey: ServiceKey<PlaceholderManager>;
}

// @beta
class ApplicationAccessor {
  constructor(serviceScope: ServiceScope);
  // @internal
  public readonly _layoutChangedEvent: Event<EventArgs>;
  public readonly navigatedEvent: Event<EventArgs>;
}

// @public
class ApplicationCustomizerContext extends ExtensionContext {
  // @internal
  constructor(extensionContextParameters: _IExtensionContextParameters,
      applicationCustomizerContextParameters: IApplicationCustomizerContextParameters);
  public readonly application: ApplicationAccessor;
  public readonly placeholderProvider: PlaceholderProvider;
}

// @alpha
class BaseApplication {
  constructor();
  // @internal
  public _dispose(): void;
  // @internal
  public static _layoutChangedEventName: string;
  // @internal
  public _load(shell: IShell): void;
  // @internal
  protected _loadTheme(): void;
  // @internal
  public static _navigatedEventName: string;
  // @internal
  public _render(domElement: HTMLDivElement | undefined): void;
  public alias: string;
  protected domElement: HTMLDivElement;
  @virtual public getBrowserCompatibility(): IBrowserCompatibility;
  @virtual protected onDispose(): void;
  @virtual protected onLoad(): void;
  @virtual protected onRender(): void;
  protected raiseLayoutChangedEvent(): void;
  protected raiseNavigatedEvent(): void;
  protected shell: IShell;
  public suiteNavConfiguration(): ISuiteNavManagerConfiguration;
}

// @public
class BaseApplicationCustomizer<TProperties> extends BaseExtension<TProperties> {
  protected context: ApplicationCustomizerContext;
}

// @alpha (undocumented)
interface IO365ShellRenderSettings {
  disableDelayLoad?: boolean;
  disableShellPlus?: boolean;
  layout: string;
  top: string;
}

// @beta
interface IPlaceholderCreateContentOptions {
  onDispose?: (placeholderContent: PlaceholderContent) => void;
}

// @alpha
interface IShell {
  // WARNING: Because this definition is explicitly marked as @internal, an underscore prefix ("_") should be added to its name
  // @internal
  applicationCustomizerLoader: ApplicationCustomizerLoader | undefined;
  environment: Environment;
  httpClient: HttpClient;
  // WARNING: Because this definition is explicitly marked as @internal, an underscore prefix ("_") should be added to its name
  // @internal
  manifestPreloads: IPreloadedData;
  pageContext: PageContext;
  // WARNING: Because this definition is explicitly marked as @internal, an underscore prefix ("_") should be added to its name
  // @internal
  pageManager: PageManager;
  serviceScope: ServiceScope;
  spHttpClient: SPHttpClient;
  suiteNavManager: ISuiteNavManager;
}

// @alpha (undocumented)
interface ISuiteNavData {
  // (undocumented)
  AboutMeLink?: ISuiteNavLink;
  ClientData?: string;
  // (undocumented)
  CommunityLink?: ISuiteNavLink;
  // (undocumented)
  CompanyDisplayName?: string;
  // (undocumented)
  CorrelationID?: string;
  // (undocumented)
  CultureName?: string;
  CurrentMainLinkElementID?: string;
  CurrentWorkloadHelpSubLinks?: Array<ISuiteNavLink>;
  CurrentWorkloadSettingsSubLinks?: Array<ISuiteNavLink>;
  CurrentWorkloadUserSubLinks?: Array<ISuiteNavLink>;
  // (undocumented)
  Dimensions?: {
    Top: string;
  }
  // (undocumented)
  FeedbackLink?: ISuiteNavLink;
  // (undocumented)
  FlightName?: string;
  // (undocumented)
  FlipHelpIcon?: boolean;
  // (undocumented)
  HelpLink?: ISuiteNavLink;
  IsAuthenticated?: boolean;
  // (undocumented)
  LegalLink?: ISuiteNavLink;
  // (undocumented)
  LogoNavigationUrl?: string;
  // (undocumented)
  MyProfileUrl?: string;
  // (undocumented)
  O365SettingsLink?: ISuiteNavLink;
  // (undocumented)
  PrivacyLink?: ISuiteNavLink;
  // (undocumented)
  SessionID?: string;
  SignOutLink?: ISuiteNavLink;
  StringsOverride?: ISuiteNavOverrideStrings;
  UserDisplayName?: string;
  UseSPOBehaviors?: boolean;
  WorkloadLinks: Array<ISuiteNavLink>;
}

// @alpha
interface ISuiteNavLink {
  Id: string;
  MenuName?: string;
  // (undocumented)
  ServiceId?: any;
  // (undocumented)
  SubLinks?: any;
  TargetWindow?: string;
  Text?: string;
  Title?: string;
  Url: string;
}

// @alpha
interface ISuiteNavManager extends IComponentVisible {
  loadSuiteNav(suiteNavConfig: ISuiteNavManagerConfiguration): void;
}

// @alpha
interface ISuiteNavManagerConfiguration {
  cacheToken: string;
  currentUICultureName: string;
  disableSuiteNav(): void;
  hamburgerCallback: (() => void) | undefined;
  // (undocumented)
  isSuiteNavDisabled(): boolean;
  modifySuiteNavData: (suiteNavData: ISuiteNavData) => ISuiteNavData;
  o365ShellRenderSettings: IO365ShellRenderSettings;
  suiteNavPostRenderCallback: () => void;
  systemUserKey: string;
  webServerRelativeUrl: string;
}

// @alpha
interface ISuiteNavOverrideStrings {
  // (undocumented)
  l_Shell15_Affordance_Text?: string;
  // (undocumented)
  l_Shell15_Affordance_Title_Text?: string;
  // (undocumented)
  l_Shell15_Help_Text?: string;
  // (undocumented)
  l_Shell15_Help_Title_Text?: string;
  // (undocumented)
  l_Shell15_MeTile_Title_Text?: string;
  // (undocumented)
  l_Shell15_O365_Text?: string;
  // (undocumented)
  l_Shell15_Settings_Text?: string;
  // (undocumented)
  l_Shell15_Settings_Title_Text?: string;
  // (undocumented)
  l_Shell15_UserSettings_Text?: string;
  // (undocumented)
  l_ShellCore_Affordance_Text?: string;
  // (undocumented)
  l_ShellCore_Affordance_Title_Text?: string;
  // (undocumented)
  l_ShellCore_Help_Text?: string;
  // (undocumented)
  l_ShellCore_Help_Title_Text?: string;
  // (undocumented)
  l_ShellCore_MeTile_Title_Text?: string;
  // (undocumented)
  l_ShellCore_O365_Text?: string;
  // (undocumented)
  l_ShellCore_O365_Title_Text?: string;
  // (undocumented)
  l_ShellCore_Settings_Text?: string;
  // (undocumented)
  l_ShellCore_Settings_Title_Text?: string;
  // (undocumented)
  l_ShellCore_UserSettings_Text?: string;
}

// @beta
class PlaceholderContent {
  // @internal
  public constructor(parameters: IPlaceholderContentParameters);
  public dispose(): void;
  public readonly domElement: HTMLDivElement;
  public readonly name: PlaceholderName;
}

// @beta
enum PlaceholderName {
  Bottom = 2,
  Top = 1
}

// @beta
class PlaceholderProvider {
  // @internal
  constructor(serviceScope: ServiceScope, sequence: number);
  public readonly changedEvent: Event<EventArgs>;
  public readonly placeholderNames: ReadonlyArray<PlaceholderName>;
  public tryCreateContent(name: PlaceholderName,
      options?: IPlaceholderCreateContentOptions): PlaceholderContent | undefined;
}

// WARNING: Unsupported export: _spShell
