// @alpha
interface ILogger {
  // @alpha
  logError(message: string): void;
  // @alpha
  logWarning(message: string): void;
}

// @alpha
interface IOAuthToken {
  isGuest: boolean;
  token: string;
}

// @alpha
interface IOAuthUtilityContext {
  // @alpha
  fetchWithDigest(request: Request): Promise<Response>;
  // @alpha
  logger: ILogger;
  // @alpha
  qosMonitor(): IQosMonitor;
}

// @alpha
interface IOAuthUtilityErrorContext {
  // (undocumented)
  correlationId?: string;
  // (undocumented)
  cultureName?: string;
  // (undocumented)
  httpResponseStatus: number;
  // (undocumented)
  message: string;
  // (undocumented)
  redirectUrl?: string;
  serverErrorCode?: number;
}

// @alpha
interface IQosMonitor {
  writeExpectedFailure(tagName?: string, ex?: Error): void;
  writeSuccess(): void;
  writeUnexpectedFailure(tagName?: string, ex?: Error): void;
}

// @alpha
class OAuthUtility {
  public constructor(context: IOAuthUtilityContext);
  public clearCachedToken(resource: string): void;
  public getOAuthToken(resource: string, webUrl: string): Promise<IOAuthToken>;
}

// @alpha
class OAuthUtilityError extends Error {
  constructor(type: OAuthUtilityErrorType, context: IOAuthUtilityErrorContext);
  // (undocumented)
  public readonly correlationId: string;
  // (undocumented)
  public readonly cultureName: string;
  // (undocumented)
  public readonly httpResponseStatus: number;
  // (undocumented)
  public readonly message: string;
  // (undocumented)
  public readonly redirectUrl: string;
  public readonly serverErrorCode: number;
  // (undocumented)
  public readonly type: number;
}

// @alpha
enum OAuthUtilityErrorType {
  malformedResponseBody,
  serverError,
  unknownError
}

// (No packageDescription for this package)
