// @internal
enum _Browser {
  // (undocumented)
  Chrome = 1,
  // (undocumented)
  Edge = 2,
  // (undocumented)
  Firefox = 3,
  // (undocumented)
  IE = 4,
  // (undocumented)
  Other = 0,
  // (undocumented)
  Safari = 5
}

// @internal
class _BrowserDetection {
  public static getBrowserInformation(userAgent?: string): IBrowserInformation;
}

// @internal
class _EventManager {
  public constructor();
  // @internal
  public _raiseEvent < TEventArgs >(eventName: string, eventArgs: TEventArgs): void;
  public static readonly instance: EventManager;
  public registerEvent < TEventArgs >(eventName: string,
      observer: IEventObserver,
      eventHandler: (eventArgs: TEventArgs) => void): void;
  public unregisterEvent < TEventArgs >(eventName: string,
      observer: IEventObserver,
      eventHandler: (eventArgs: TEventArgs) => void): void;
}

// @internal
interface _IBrowserInformation {
  browser: Browser;
  browserVersion?: Version;
  os: OS;
  osVersion?: string;
  userAgent?: string;
}

// @internal
interface _IEnvironmentData {
  type: EnvironmentType;
}

// @internal
interface _ISessionData {
  applicationId: string;
  pageId?: string;
}

// @internal
enum _OS {
  // (undocumented)
  Android = 1,
  // (undocumented)
  IOS = 2,
  // (undocumented)
  Linux = 3,
  // (undocumented)
  Mac = 4,
  // (undocumented)
  Other = 0,
  // (undocumented)
  Windows = 5,
  // (undocumented)
  WindowsPhone = 6
}

// @internal
class _SPFlight {
  public static initialize(flights: number[], debugFlights?: string): void;
  public static isEnabled(flight: number): boolean;
}

// @internal
class _SPKillSwitch {
  public static initialize(killSwitchGuids: { [key: string]: boolean }, debugKillswitches?: string): void;
  public static isActivated(killSwitchGuid: Guid, date: string, featureName: string): boolean;
}

// @alpha
enum BrowserSupportLevel {
  Blocked = 3,
  FullFunctionality = 1,
  None = 0,
  Readable = 2
}

// @public
enum DisplayMode {
  Edit = 2,
  Read = 1
}

// @public
class Environment {
  // @internal
  public static _initialize(data: IEnvironmentData): void;
  public static readonly type: EnvironmentType;
}

// @public
enum EnvironmentType {
  ClassicSharePoint,
  Local,
  SharePoint,
  Test
}

// @beta
class Event<TEventArgs extends EventArgs> {
  // @internal
  public constructor(name: string);
  public add(observer: IEventObserver, eventHandler: (eventArgs: TEventArgs) => void): void;
  public remove(observer: IEventObserver, eventHandler: (eventArgs: TEventArgs) => void): void;
}

// @beta
class EventArgs {
}

// @public
class Guid {
  public static empty: Guid;
  public equals(guid: Guid): boolean;
  public static isValid(guid: string | undefined | null): boolean;
  public static newGuid(randomNumberGenerator?: IRandomNumberGenerator): Guid;
  public static parse(guidString: string | undefined | null): Guid;
  public toString(): string;
  public static tryParse(guid: string | undefined | null): Guid | undefined;
}

// @alpha
interface IBrowserCompatibility {
  supportLevel: BrowserSupportLevel;
  warning: string | undefined;
}

// @beta
interface IDisposable {
  dispose(): void;
  isDisposed: boolean;
}

// @beta
interface IEventObserver extends IDisposable {
  componentId: string;
  instanceId: string;
}

// @alpha
interface ILogHandler {
  // (undocumented)
  error(source: string, error: Error, scope: ServiceScope | undefined): void;
  // (undocumented)
  info(source: string, message: string, scope: ServiceScope | undefined): void;
  // (undocumented)
  verbose(source: string, message: string, scope: ServiceScope | undefined): void;
  // (undocumented)
  warn(source: string, message: string, scope: ServiceScope | undefined): void;
}

// @public
interface IRandomNumberGenerator {
  generate(): number;
}

// @public @deprecated
interface IServiceCollection {
  serviceScope: ServiceScope;
}

// @public
interface ITimeProvider {
  getDate(): Date;
  getTimestamp(): number;
}

// @alpha
class JsonUtilities {
  public static encode(json: string): string | undefined;
}

// @public
class Log {
  // @internal
  public static _initialize(logHandler: ILogHandler): void;
  public static error(source: string, error: Error, scope?: ServiceScope): void;
  public static info(source: string, message: string, scope?: ServiceScope): void;
  public static verbose(source: string, message: string, scope?: ServiceScope): void;
  public static warn(source: string, message: string, scope?: ServiceScope): void;
}

// @public
class RandomNumberGenerator implements IRandomNumberGenerator {
  constructor(serviceScope: ServiceScope);
  public generate(): number;
  public static serviceKey: ServiceKey<IRandomNumberGenerator>;
}

// @public
class ServiceKey<T> {
  public static create < TKey >(name: string,
      serviceClass: { new (serviceScope: ServiceScope); }): ServiceKey<TKey>;
  public static createCustom < TKey >(name: string, defaultCreator: ServiceCreator<TKey>): ServiceKey<TKey>;
  public defaultCreator: ServiceCreator<T>;
  public id: string;
  public name: string;
}

// @public
class ServiceScope {
  public consume < T >(serviceKey: ServiceKey<T>): T;
  public createAndProvide < T >(serviceKey: ServiceKey<T>,
      simpleServiceClass: { new (serviceScope: ServiceScope); }): T;
  public createDefaultAndProvide < T >(serviceKey: ServiceKey<T>): T;
  public finish(): void;
  public getParent(): ServiceScope | undefined;
  public provide < T >(serviceKey: ServiceKey<T>, service: T): T;
  public startNewChild(): ServiceScope;
  public static startNewRoot(): ServiceScope;
  public whenFinished(callback: () => void): void;
}

// @public
class Session {
  // @internal
  public static _changePage(): void;
  // @internal
  public static _initialize(data: ISessionData): void;
  public static readonly applicationId: Guid;
  public static readonly pageId: Guid;
}

// @alpha
class SPError extends Error {
  constructor(errorCode: string,
        message: string,
        logProperties?: {[key: string]: any});
  public _toString(logDebug?: boolean): string;
  public readonly category: string;
  public readonly id: number;
  public innerError: Error | undefined;
  public toString(): string;
  public toStringForUI(): string;
}

// @public
class Text {
  public static format(s: string, ...values: any[]): string;
}

// @public
class TimeProvider implements ITimeProvider {
  constructor(serviceScope: ServiceScope);
  public getDate(): Date;
  public getTimestamp(): number;
  // @beta
  public static serviceKey: ServiceKey<ITimeProvider>;
}

// @public @deprecated
class UrlQueryParameterCollection {
  constructor(url: string);
  public getValue(param: string): string | undefined;
  public getValues(param: string): (string | undefined)[] | undefined;
}

// @alpha
class UrlUtilities {
  public static convertToODataStringLiteral(value: string): string;
  public static removeEndSlash(url: string): string;
}

// @public
class Validate {
  public static isNonemptyString(value: string | undefined | null, variableName: string): void;
  public static isNotNullOrUndefined(value: any, variableName: string): void;
  public static isTrue(value: boolean | undefined | null, variableName: string): void;
}

// @public
class Version {
  public static compare(v1: Version, v2: Version): number;
  public equals(compareWith: Version): boolean;
  public greaterThan(compareWith: Version): boolean;
  public static isValid(versionString: string | undefined | null): boolean;
  public lessThan(compareWith: Version): boolean;
  public readonly major: number;
  public readonly minor: number;
  public static parse(versionString: string | undefined | null): Version;
  public readonly patch: number | undefined;
  public readonly revision: number | undefined;
  public satisfies(compareWith: Version): boolean;
  public toString(): string;
  public static tryParse(versionString: string | undefined | null): Version | undefined;
}

// WARNING: Unsupported export: ServiceCreator
