// @internal
class _GraphHttpClientContext {
  constructor(serviceScope: ServiceScope);
  public getWebServerRelativeUrl(): string;
  public initialize(webServerRelativeUrl: string): void;
  public static serviceKey: ServiceKey<GraphHttpClientContext>;
}

// @alpha
class DigestCache implements IDigestCache {
  constructor(serviceScope: ServiceScope);
  public addDigestToCache(webUrl: string, digestValue: string, expirationTimestamp: number): void;
  public clearAllDigests(): void;
  public clearDigest(webUrl: string): boolean;
  public fetchDigest(webUrl: string): Promise<string>;
  public static serviceKey: ServiceKey<IDigestCache>;
}

// @beta
class GraphHttpClient {
  constructor(serviceScope: ServiceScope);
  public static configurations: IGraphHttpClientConfigurations;
  public fetch(url: string,
      configuration: GraphHttpClientConfiguration,
      options: IGraphHttpClientOptions): Promise<GraphHttpClientResponse>;
  public get(url: string, configuration: GraphHttpClientConfiguration,
      options?: IGraphHttpClientOptions): Promise<GraphHttpClientResponse>;
  public post(url: string, configuration: GraphHttpClientConfiguration,
      options: IGraphHttpClientOptions): Promise<GraphHttpClientResponse>;
  public static serviceKey: ServiceKey<GraphHttpClient>;
}

// @beta
class GraphHttpClientConfiguration extends HttpClientConfiguration, implements IGraphHttpClientConfiguration {
  constructor(flags: IGraphHttpClientConfiguration, overrideFlags?: IGraphHttpClientConfiguration);
  // (undocumented)
  protected flags: IGraphHttpClientConfiguration;
}

// @beta
class GraphHttpClientResponse extends HttpClientResponse {
  public constructor(response: Response);
  // (undocumented)
  @override public clone(): GraphHttpClientResponse;
}

// @public
class HttpClient {
  constructor(serviceScope: ServiceScope);
  public static configurations: IHttpClientConfigurations;
  public fetch(url: string, configuration: HttpClientConfiguration,
      options: IHttpClientOptions): Promise<HttpClientResponse>;
  public get(url: string, configuration: HttpClientConfiguration,
      options?: IHttpClientOptions): Promise<HttpClientResponse>;
  public post(url: string, configuration: HttpClientConfiguration,
      options: IHttpClientOptions): Promise<HttpClientResponse>;
  // @public
  public static serviceKey: ServiceKey<HttpClient>;
}

// @public
class HttpClientConfiguration implements IHttpClientConfiguration {
  constructor(flags: IHttpClientConfiguration, overrideFlags?: IHttpClientConfiguration);
  // (undocumented)
  protected flags: IHttpClientConfiguration;
  @virtual protected initializeFlags(): void;
  @virtual public overrideWith(sourceFlags: IHttpClientConfiguration): HttpClientConfiguration;
}

// @public
class HttpClientResponse implements Response, Body {
  // @internal
  public constructor(response: Response);
  // WARNING: Unable to find a documentation file ("whatwg-fetch.api.json") for the referenced package
  public arrayBuffer(): Promise<ArrayBuffer>;
  // WARNING: Unable to find a documentation file ("whatwg-fetch.api.json") for the referenced package
  public blob(): Promise<Blob>;
  // @alpha
  public readonly body: ReadableStream | null;
  // WARNING: Unable to find a documentation file ("whatwg-fetch.api.json") for the referenced package
  public readonly bodyUsed: boolean;
  // (undocumented)
  @virtual public clone(): HttpClientResponse;
  // WARNING: Unable to find a documentation file ("whatwg-fetch.api.json") for the referenced package
  public formData(): Promise<FormData>;
  // WARNING: Unable to find a documentation file ("whatwg-fetch.api.json") for the referenced package
  public readonly headers: Headers;
  // WARNING: Unable to find a documentation file ("whatwg-fetch.api.json") for the referenced package
  public json(): Promise<any>;
  // (undocumented)
  protected nativeResponse: Response;
  // WARNING: Unable to find a documentation file ("whatwg-fetch.api.json") for the referenced package
  public readonly ok: boolean;
  // WARNING: Unable to find a documentation file ("whatwg-fetch.api.json") for the referenced package
  public readonly status: number;
  // WARNING: Unable to find a documentation file ("whatwg-fetch.api.json") for the referenced package
  public readonly statusText: string;
  // WARNING: Unable to find a documentation file ("whatwg-fetch.api.json") for the referenced package
  public text(): Promise<string>;
  // WARNING: Unable to find a documentation file ("whatwg-fetch.api.json") for the referenced package
  public readonly type: ResponseType;
  // WARNING: Unable to find a documentation file ("whatwg-fetch.api.json") for the referenced package
  public readonly url: string;
}

// @alpha
interface IDigestCache {
  addDigestToCache(webUrl: string, digestValue: string, expirationTimestamp: number): void;
  clearAllDigests(): void;
  clearDigest(webUrl: string): boolean;
  fetchDigest(webUrl: string): Promise<string>;
}

// @beta
interface IGraphHttpClientConfiguration extends IHttpClientConfiguration {
}

// @beta
interface IGraphHttpClientConfigurations {
  v1: GraphHttpClientConfiguration;
}

// @beta
interface IGraphHttpClientOptions extends IHttpClientOptions {
}

// @public
interface IHttpClientConfiguration {
}

// @public
interface IHttpClientConfigurations {
  v1: HttpClientConfiguration;
}

// @public
interface IHttpClientOptions extends RequestInit {
}

// @alpha
interface IOAuth2Token {
  message: string;
  token: string;
}

// @alpha
interface ISPHttpClientBatchConfiguration extends ISPHttpClientCommonConfiguration {
}

// @alpha
interface ISPHttpClientBatchConfigurations {
  v1: SPHttpClientBatchConfiguration;
}

// @alpha
interface ISPHttpClientBatchCreationOptions {
  webUrl?: string;
}

// @alpha
interface ISPHttpClientBatchOptions extends IHttpClientOptions {
}

// @public
interface ISPHttpClientCommonConfiguration extends IHttpClientConfiguration {
  jsonRequest?: boolean;
  jsonResponse?: boolean;
}

// @public
interface ISPHttpClientConfiguration extends ISPHttpClientCommonConfiguration {
  defaultODataVersion?: ODataVersion;
  defaultSameOriginCredentials?: boolean;
  requestDigest?: boolean;
}

// @public
interface ISPHttpClientConfigurations {
  v1: SPHttpClientConfiguration;
}

// @public
interface ISPHttpClientOptions extends IHttpClientOptions {
  webUrl?: string;
}

// @alpha
class OAuth2TokenProvider {
  public constructor(serviceScope: ServiceScope);
  // (undocumented)
  protected static _authContext: adal.AuthenticationContext;
  // WARNING: The type "IDisplayCallHandlerArgs" needs to be exported by the package (e.g. added to index.ts)
  // (undocumented)
  protected _displayCallHandler: ((args: IDisplayCallHandlerArgs) => void) | undefined;
  // @internal
  public _initialize(azureInstanceUrl: string, azureTenantId: string): void;
  // WARNING: The type "IAdalJsModule" needs to be exported by the package (e.g. added to index.ts)
  // (undocumented)
  protected _loadAndConfigureAdalJsModulePromise: Promise<IAdalJsModule> | undefined;
  public getOAuthToken(resourceEndpoint: string): Promise<IOAuth2Token>;
  public static serviceKey: ServiceKey<OAuth2TokenProvider>;
}

// @alpha
class OAuthTokenProvider {
  public constructor(serviceScope: ServiceScope);
  public clearCachedToken(resource: string): void;
  public getOAuthToken(resource: string, webUrl: string): Promise<any>;
  public static serviceKey: ServiceKey<OAuthTokenProvider>;
}

// @public
class ODataVersion {
  public toString(): string;
  public static tryParseFromHeaders(headers: Headers): ODataVersion | undefined;
  public static v3: ODataVersion;
  public static v4: ODataVersion;
}

// @public
class SPHttpClient {
  constructor(serviceScope: ServiceScope);
  public beginBatch(batchCreationOptions?: ISPHttpClientBatchCreationOptions): SPHttpClientBatch;
  public static configurations: ISPHttpClientConfigurations;
  public fetch(url: string, configuration: SPHttpClientConfiguration,
      options: ISPHttpClientOptions): Promise<SPHttpClientResponse>;
  public get(url: string, configuration: SPHttpClientConfiguration,
      options?: ISPHttpClientOptions): Promise<SPHttpClientResponse>;
  public static getWebUrlFromRequestUrl(requestUrl: string): string;
  public post(url: string, configuration: SPHttpClientConfiguration,
      options: ISPHttpClientOptions): Promise<SPHttpClientResponse>;
  public static serviceKey: ServiceKey<SPHttpClient>;
}

// @alpha
class SPHttpClientBatch {
  // @internal
  public constructor(serviceScope: ServiceScope, batchCreationOptions: ISPHttpClientBatchCreationOptions);
  public static configurations: ISPHttpClientBatchConfigurations;
  public execute(): Promise<SPHttpClientBatch>;
  public fetch(url: string, configuration: SPHttpClientBatchConfiguration, options?: ISPHttpClientBatchOptions): Promise<SPHttpClientResponse>;
  public get(url: string, configuration: SPHttpClientBatchConfiguration,
      options?: ISPHttpClientBatchOptions): Promise<SPHttpClientResponse>;
  public post(url: string, configuration: SPHttpClientBatchConfiguration,
      options: ISPHttpClientBatchOptions): Promise<SPHttpClientResponse>;
}

// @alpha
class SPHttpClientBatchConfiguration extends SPHttpClientCommonConfiguration, implements ISPHttpClientBatchConfiguration {
  constructor(flags: ISPHttpClientBatchConfiguration, overrideFlags?: ISPHttpClientBatchConfiguration);
  // (undocumented)
  protected flags: ISPHttpClientBatchConfiguration;
  // (undocumented)
  @override protected initializeFlags(): void;
  // (undocumented)
  @override public overrideWith(sourceFlags: ISPHttpClientBatchConfiguration): SPHttpClientBatchConfiguration;
}

// @public
class SPHttpClientCommonConfiguration extends HttpClientConfiguration, implements ISPHttpClientCommonConfiguration {
  constructor(flags: ISPHttpClientCommonConfiguration, overrideFlags?: ISPHttpClientCommonConfiguration);
  // (undocumented)
  protected flags: ISPHttpClientCommonConfiguration;
  // (undocumented)
  @override protected initializeFlags(): void;
  public readonly jsonRequest: boolean;
  public readonly jsonResponse: boolean;
  // (undocumented)
  @override public overrideWith(sourceFlags: ISPHttpClientCommonConfiguration): SPHttpClientCommonConfiguration;
}

// @public
class SPHttpClientConfiguration extends SPHttpClientCommonConfiguration, implements ISPHttpClientConfiguration {
  constructor(flags: ISPHttpClientConfiguration, overrideFlags?: ISPHttpClientConfiguration);
  public readonly defaultODataVersion: ODataVersion;
  public readonly defaultSameOriginCredentials: boolean;
  // (undocumented)
  protected flags: ISPHttpClientConfiguration;
  // (undocumented)
  @override protected initializeFlags(): void;
  // (undocumented)
  @override public overrideWith(sourceFlags: ISPHttpClientConfiguration): SPHttpClientConfiguration;
  public readonly requestDigest: boolean;
}

// @public
class SPHttpClientResponse extends HttpClientResponse {
  public constructor(response: Response);
  // (undocumented)
  @override public clone(): SPHttpClientResponse;
}

