// @internal
interface _IErrorInformation {
  error?: Error;
  message: string;
}

// @internal
interface _IPreloadedCustomAction {
  clientSideComponentId: string;
  clientSideComponentProperties?: string;
  location: string;
  registrationId?: string;
  registrationType?: string;
  requireSiteAdministrator?: boolean;
  rights?: IODataBasePermission;
  sequence?: number;
  title?: string;
}

// @internal
interface _IPreloadedData {
  clientSideApplicationId?: string;
  contextWebInfo?: IODataContextWebInformation;
  customActions?: IPreloadedCustomAction[];
  item?: IODataListItem;
  list?: IODataList;
  manifests: IClientSideComponentManifest[];
  page?: any;
  quickLaunch?: IODataNavigationNodeCollection;
  spPageContextInfo: _IODataPageContextInfo;
  user?: IODataUser;
  web?: IODataWeb;
}

// @alpha
class DeveloperTools {
  public static initialize(developerToolsLoader: IDeveloperTools): void;
  public static registerDeveloperToolsTab(developerToolsTab: IDeveloperToolsTab): void;
}

// @alpha
interface IDeveloperTools {
  initialize(): void;
  registerDeveloperToolsTab(developerToolsTab: IDeveloperToolsTab): void;
}

// @public
interface ILoadScriptOptions {
  globalExportsName?: string;
}

// @public
interface ISPComponentLoader {
  // @internal
  _forceLocalManifests(): Promise<void>;
  // @internal
  _initialize(preloadedData: IPreloadedData,
      bundledComponents: { [id: string]: Object }): Promise<void>;
  loadComponent < TComponent >(manifest: IClientSideComponentManifest): Promise<TComponent>;
  // @alpha
  loadComponentById < TComponent >(id: string, version?: string): Promise<TComponent>;
  loadCss(url: string): void;
  loadScript < TModule >(url: string, options?: ILoadScriptOptions): Promise<TModule>;
  // @alpha
  readonly manifests: IClientSideComponentManifest[];
  // @alpha
  registerManifests(manifests: IClientSideComponentManifest[]): void;
  // WARNING: Because this definition is explicitly marked as @internal, an underscore prefix ("_") should be added to its name
  // @internal
  startApplication < TApplication >(applicationId: string, preloadedData: IPreloadedData): Promise<TApplication>;
  // @alpha
  tryGetManifestById(id: string, version?: string): IClientSideComponentManifest | undefined;
}

// @public
class SPComponentLoader {
  // @internal
  public static _forceLocalManifests(): Promise<void>;
  // @alpha
  public static getManifests(): IClientSideComponentManifest[];
  public static initialize(componentLoader: ISPComponentLoader): void;
  public static loadComponent < TComponent >(manifest: IClientSideComponentManifest): Promise<TComponent>;
  // @alpha
  public static loadComponentById < TComponent >(id: string, version?: string): Promise<TComponent>;
  public static loadCss(url: string): void;
  public static loadScript < TModule >(url: string, options?: ILoadScriptOptions): Promise<TModule>;
  // @alpha
  public static registerManifests(manifests: IClientSideComponentManifest[]): void;
  // WARNING: Because this definition is explicitly marked as @internal, an underscore prefix ("_") should be added to its name
  // @internal
  public static startApplication < TApplication >(applicationId: string,
      preloadedData: IPreloadedData): Promise<TApplication>;
  // @alpha
  public static tryGetManifestById(id: string, version?: string): IClientSideComponentManifest | undefined;
}

// WARNING: Because this definition is explicitly marked as @internal, an underscore prefix ("_") should be added to its name
// @internal
class SPStarter {
  // @internal
  public static _setBundledComponents(bundledComponents: { [id: string]: Object }): void;
  public static getBrowserCompatibility(): IBrowserCompatibility;
  public static start < TApplication >(preloadedData: IPreloadedData,
      handleFailure?: (errorInformation: IErrorInformation) => void): Promise<TApplication>;
}

// WARNING: Unsupported export: IDeveloperToolsTab
