// @alpha
interface IClientSideApplicationManifest extends IClientSideComponentManifest {
  // WARNING: Because this definition is explicitly marked as @internal, an underscore prefix ("_") should be added to its name
  // @internal
  assemblyId?: string;
  description: ILocalizedString;
  preloadComponents: string[];
  preloadOptions: IPreloadOptions;
  title: ILocalizedString;
}

// @public
interface IClientSideAssemblyManifest extends IClientSideComponentManifest {
  rootComponentId: string;
}

// @beta
interface IClientSideComponentLoaderConfiguration {
  entryModuleId: string;
  exportName?: string;
  internalModuleBaseUrls: string[];
  scriptResources: {
    [ name: string ]: IModuleConfiguration
  }
}

// @public
interface IClientSideComponentManifest {
  alias: string;
  componentType: 'Application' | 'WebPart' | 'Library' | 'Extension';
  id: string;
  // WARNING: Because this definition is explicitly marked as @internal, an underscore prefix ("_") should be added to its name
  // @internal
  isInternal?: boolean;
  loaderConfig: IClientSideComponentLoaderConfiguration;
  manifestVersion: number;
  requiresCustomScript?: boolean;
  // @deprecated
  safeWithCustomScriptDisabled?: boolean;
  version: string;
}

// @beta
interface IClientSideExtensionManifest extends IClientSideComponentManifest {
  extensionType: ExtensionType;
}

// @public
interface IClientSideLibraryManifest extends IClientSideComponentManifest {
}

// @public
interface IClientSideWebPartManifest<TProperties> extends
  IClientSideComponentManifest {
  disabledOnClassicSharepoint?: boolean;
  hiddenFromToolbox?: boolean;
  preconfiguredEntries: IClientSideWebPartManifestEntry<TProperties>[];
  supportsFullBleed?: boolean;
}

// @public
interface IClientSideWebPartManifestEntry<TProperties> {
  dataVersion?: string;
  description: ILocalizedString;
  // WARNING: Unable to find referenced export "@microsoft/sp-module-interfaces:PredefinedGroup"
  // @beta
  group?: ILocalizedString;
  // @beta
  groupId: PredefinedGroup | string;
  iconImageUrl?: string;
  officeFabricIconFontName?: string;
  properties: TProperties;
  title: ILocalizedString;
}

// @public
interface IClientSideWebPartManifestInstance<TProperties> extends IClientSideComponentManifest {
}

// @beta
interface ICommandDefinition {
  ariaLabel?: ILocalizedString;
  iconImageUrl?: string;
  title: ILocalizedString;
  type: 'command';
}

// @beta
interface ICommandSetExtensionManifest extends IClientSideExtensionManifest {
  extensionType: 'ListViewCommandSet';
  items: {
    [ itemId: string ]: ICommandDefinition
  }
}

// @beta
interface IComponentModuleConfiguration extends IModuleConfiguration {
  failoverPath?: string | IPath;
  id: string;
  version: string;
}

// @beta
interface ILocalizedPathModuleConfiguration extends IModuleConfiguration {
  defaultPath: string | IPath;
  paths?: {
    [ locale: string ]: string | IPath
  }
}

// @public
interface ILocalizedString {
  // (undocumented)
  [ locale: string ]: string | undefined;
  // (undocumented)
  default?: string;
  // (undocumented)
  id?: string;
}

// @beta
interface IModuleConfiguration {
  shouldNotPreload?: boolean;
  type: 'component' | 'path' | 'localizedPath';
}

// @beta
interface IPath {
  debug?: string;
  default: string;
}

// @beta
interface IPathModuleConfiguration extends IModuleConfiguration {
  globalDependencies?: string[];
  globalName?: string;
  path: string | IPath;
}

// @alpha (undocumented)
interface IPreloadOptions {
  preloadListItemProperties?: string[];
  preloadListItemQueryOption?: string;
  shouldPreloadItem?: boolean;
  shouldPreloadList?: boolean;
  shouldPreloadQuickLaunch?: boolean;
  shouldPreloadUser?: boolean;
  shouldPreloadWeb?: boolean;
}

// @alpha
enum ManifestType {
  component = 'Component'
}

// WARNING: Unsupported export: ExtensionType
// WARNING: Unsupported export: PredefinedGroup
// WARNING: Unsupported export: PredefinedGroup
// (No packageDescription for this package)
