// TypeScript Type definitions for TAB (core API)

// workaround to access global 'Window' type inside TAB.Window
declare var _GlobalWindow: Window;

declare module TAB {
    var browserIs: BrowserIs;
    var MustTakeScreenshot: boolean;
    var PageErrCount: number;
    var Win: Window;

    function AddTask(task: Task, waiter?: Waiter, taskOptions?: TaskOptions): aSyncQueueEntry;
    function AddTaskClearBrowserCache(win: Window): void;
    function AddTaskClientOrchestration(taskName: string, result: ClientOrchestrationResult, params: any): void;
    function AddTaskClientOrchestrationUsingPOST(taskName: string, result: ClientOrchestrationResult, params: any): void;
    function AddTaskEnsureCurrentPage(urlIn: string, fragmentToCheck?: string): void;
    function AddTaskLoginAsOriginalUser(): void;
    function AddTaskLoginAsAnotherUser(userdetails: any): void;
    function AddTaskNavigate(urlIn: string): void;
    function AndWaiters(waiter1: Waiter, waiter2: Waiter): Waiter;
    function ClearAllDataPops(settingsIn: any, datapopsIn: any, parentWindow: any);
    function ClearCookies(): void;
    function CloseWindow(tabWindow: Window): Task;
    function CombineUrl(baseUrl: string, relativeUrl: string);
    function EnsureAuthentication(): void;
    function GetControl(controlType: any): any;
    function GetControls(controlType: any): Array<any>;
    function GetDomain(idx: number): string;
    function GetUser(idx: number): any;
    function GetUserAlias(idx: number): string;
    function GetUserLogin(idx: number): string;
    function GetUserName(idx: number): string;
    function GetUserPassword(idx: number): string;
    function IsClientOrchestrationEnabled(): boolean;
    function LaunchCCProxy(): Task;
    function LoadPage(urlIn: string): Task;
    function LoadScript(scriptURL: string, scriptFileName: string): Task;
    function LoadScripts(arrScriptUrls: Array<string>, scriptsLoaded: Array<string>): void;
    function LogFromProduct(s: string): void;
    function LoginCore(authParams: any): void;
    function LogOnError(msg: string, url: string, line: number, column: number, error: any): void;
    function MakeTask(arg1: Function, ...args: any[]): Task;
    function MakeWaiter(arg1: Function, ...args: any[]): Waiter;
    function NeutralDomainURL(): string;
    function OpenWindow(twc: WindowContainer): Task;
    function OrWaiters(waiter1: Waiter, waiter2: Waiter): Waiter;
    function PageReady(errOut?: number): Waiter;
    function PageUrlContainsFunc(text: string): boolean;
    function PopulateData(datapopScripts: Array<string>): void;
    function ReloadPage(): Task;
    function RunAsTabTask(fn: Function, waiter?: Waiter, taskOptions?: TaskOptions): aSyncQueueEntry;
    function SetWaitTime(time: number): void;
    function StopCCProxy(): Task;
    function TryCatch(tryBlock: Function, catchBlock: Function): boolean;

    class aSyncQueue {
        constructor(parentIn: aSyncQueue);

        parent: aSyncQueue;
        queue: Array<aSyncQueueEntry>;
        currentEntry: number;
    }

    class aSyncQueueEntry {
        constructor(task: Task, waiter: Waiter);

        callerStackTrace: string;
        mustRun: boolean;
        state: number;

        GetMustRun(): boolean;
        GetTestString(): string;
        GetWaiterString(): string;
        SetMustRun(): void;
    }

    class BrowserIs {
        constructor();

        firefox: boolean;
        ie: boolean;
        chrome: boolean;
    }

    class ClientOrchestrationResult {
        constructor();

        testComplete: boolean;
        testRan: boolean;
        testResult: string;
        testSuccess: boolean;
    }

    class DataPops {
        static All(): any;
        static Clear(datapopName: string): any;
        static Get(datapopName: string): any;
        static Initialize(datapopsIn: any): void;
        static Set(dataPopName: string, data: any): void;
    }

    class Log {
        static AddTaskComment(message: string): void;
        static AddTaskCommentTag(message: string, tag: number): void;
        static AddTaskDidNotRun(desc: string): void;
        static AddTaskFail(desc: string): void;
        static AddTaskFailTag(desc: string, tag: number): void;
        static AddTaskPass(desc: string): void;
        static AddTaskPassTag(desc: string, tag: number): void;
        static AddTaskVerbose(message: string): void;
        static AddTaskVerboseTag(message: string, tag: number): void;
        static AddTaskVerify(success: boolean, message: string): boolean;
        static AddTaskVerifyFunc(func: Function, message: string): boolean;
        static AddTaskVerifyTag(success: boolean, message: string, tag: number): boolean;
        static AddTaskWarning(message: string): void;
        static AddTaskWarningTag(message: string, tag: number): void;
        static Comment(message: string): void;
        static CommentTag(message: string, tag: number): void;
        static CrawlData(data: string): void;
        static CrawlDataTag(data: string, tag: number): void;
        static DidNotRun(desc: string): void;
        static DidNotRunTag(desc: string, tag: number): void;
        static EndLog(phaseName: string): void;
        static EndLogTag(phaseName: string, tag: number): void;
        static ExpectedException(fn: Function, exception?: string): boolean;
        static Fail(desc: string): void;
        static FailTag(desc: string, tag: number): void;
        static GetCurrentStackTraceString(): void;
        static GetErrorMessage(e: any): void;
        static GetTestName(): string;
        static Pass(desc: string): void;
        static PassTag(desc: string, tag: number): void;
        static PerfData(data: string): void;
        static PerfDataTag(data: string, tag: number): void;
        static Screenshot(id: string): void;
        static SendLogToHarness(): Task;
        static SetParentWindow(parentWindow: any): void;
        static SetTestName(testNameIn: string): void;
        static SetTestOwner(testOwnerIn: string): void;
        static SetVerbose(enable: boolean): void;
        static StartLog(phaseName: string): void;
        static TelemetryData(data: string): void;
        static TelemetryDataTag(data: string, tag: number): void;
        static Verbose(message: string): void;
        static VerboseTag(message: string, tag: number): void;
        static Verify(success: boolean, message: string): boolean;
        static VerifyEquals(var1: any, var2: any, message: string): boolean;
        static VerifyEqualsDeep(var1: any, var2: any, message: string): boolean;
        static VerifyEqualsDeepTag(var1: any, var2: any, message: string, tag: number): boolean;
        static VerifyEqualsInternal(var1: any, var2: any, message: string, bDeepEquals: boolean, tag: number): boolean;
        static VerifyEqualsTag(var1: any, var2: any, message: string, tag: number): boolean;
        static VerifyFalse(success: boolean, message: string): boolean;
        static VerifyFalseTag(success: boolean, message: string, tag: number): boolean;
        static VerifyTag(success: boolean, message: string, tag: number): boolean;
        static VersionData(data: string): void;
        static VersionDataTag(data: string, tag: number): void;
        static Warning(desc: string): void;
        static WarningTag(desc: string, tag: number): void;
    }

    class Settings {
        static Initialize(settingsIn: any): void;
        static Get(settingName: string): any;
        static Set(settingName: string, settingValue: any): void;
    }

    class Task {
        constructor(func: Function, args: Array<any>);

        taskOptions: TaskOptions;

        call(): any;
        taskToString(): string;
        Then(nextTask: Task): Task;
        WaitFor(water: Waiter): Task;
        When(waiter: Waiter, nextTask: Task): Task;
    }

    class TaskOptions {
        constructor();

        expectAlert: boolean;
        expectConfirmValue: boolean;
    }

    class Waiter {
        constructor(func: Function, args: Array<any>);

        call(): any;
        PreTaskInit(): void;
        SetPreTaskInit(ptiFunc: Function): void;
        Then(nextTask: Task): Task;
        waiterToString(): string;
        Wait(): Task;
    }

    class Window {
        constructor (doNothing?: boolean);

        close(): void;
        document: Document;
        location: Location;
        name: string;
        window: typeof _GlobalWindow;
    }

    class WindowContainer {
        constructor();

        tabwin: Window;
    }
}
