// TypeScript Type definitions for TAB (TaskLib API)

declare module TAB {
    function AddAjaxTask(xmlhttp: XMLHttpRequest, successCallback?: Function, postBody?: string);
    function AddTaskCapture(tag: string, target: captureTarget, element?: HTMLElement): void;
    function AddTaskCaptureFailure(): void;
    function AddTaskClickElement(searchObj: any, criteria: searchBy, value: any, waiter?: Waiter): void;
    function AddTaskSimulateDragElement(element: HTMLElement, moveX: number, moveY: number, moves: number): void;
    function ClickElement(element: HTMLElement): void;
    function ElementExists(searchObj: any, criteria: searchBy, value: any): boolean;
    function EscapeQuery(str: string): string;
    function FireChangeEvent(element: HTMLElement);
    function FireKeyboardEvent(element: HTMLElement, eventName: string, keyCode: number);
    function FireMouseEvent(element: HTMLElement, mouseEventValue: mouseEvent, mouseEventArg?: MouseEventData): void;
    function GetElement(searchObj: any, criteria: searchBy, value: any): HTMLElement;
    function GetElementOffset(element: HTMLElement): ElementOffset;
    function GetElements(searchObj: any, criteria: searchBy, value: any): Array<HTMLElement>;
    function GetElementPosition(element: HTMLElement): ElementPosition;
    function GetPageDimensions(): Dimensions;
    function GetStyleAttribute(element: HTMLElement, attributeName: string): any;
    function GetTextContent(element: HTMLElement): string;
    function GetUniqueName(str: string): string;
    function HasCssClass(elm: HTMLElement, classNAme: string): boolean;
    function InstantiateInTABWindow(constructorName: string, parameters?: Array<any>): any;
    function MakeAsyncTask(fn: Function): Task;
    function PickRadioButton(controlIds: Array<any>, controlToClick: number): void;
    function SetTextContent(element: HTMLElement, newContent: string): string;
    function StringToXml(strXml: string): any;
    function WaitForElementToExist(searchObj: any, criteria: searchBy, value: any): Waiter;
    function XmlToString(xmlDoc: any): string;

    enum captureTarget {
        screen,
        page,
        element
    }

    enum mouseEvent {
        click,
        mouseDown,
        mouseUp,
        doubleClick,
        mouseMove,
        mouseOver,
        contextmenu,
        mouseEnter
    }

    enum searchBy {
        id,
        trailingId,
        leadingId,
        partialId,
        className,
        partialClassName,
        partialAlt,
        partialContent,
        partialField,
        partialHref,
        partialName,
        partialRole,
        partialSrc,
        tag,
        partialTitle,
        customQuery,
        hasClassName
    }

    class Dimensions {
        width: number;
        height: number;
    }

    class DropdownControl {
        constructor(searchObj: any, criteria: searchBy, value: any);

        HasOptionByTextContent(_value: string): boolean;
        SelectOptionByIndex(elementId: number): void;
        SelectOptionByPartialValue(elementIndex: number, _value: string): void;
        SelectOptionByTextContent(textContent: string): void;
        SelectOptionByValue(_value: string): void;
    }

    class ElementOffset {
        constructor();

        top: number;
        left: number;
    }

    class ElementPosition {
        x: number;
        y: number;
        w: number;
        h: number;
    }

    class Encode {
        static EncodeScriptQuote(str: string): string;
        static HtmlDecode(str: string): string;
        static HtmlEncode(str: string): string;
        static ScriptEncode(str: string): string;
        static ScriptEncodeWithQuote(str: string): string;
        static StrAttrQuote(st: string): string;
    }

    class MouseEventData {
        constructor(clientX: number, clientY: number);

        altKey: boolean;
        bubbles: boolean;
        button: number;
        cancelable: boolean;
        clientX: number;
        clientY: number;
        ctrlKey: boolean;
        detail: number;
        metaKey: boolean;
        relatedTarget: Node;
        screenX: number;
        screenY: number;
        shiftKey: boolean;
        view: Window;
    }
}