/**
 * @copyright Microsoft Corporation. All rights reserved.
 *
 * @file React utility to manipulate the elements.
 */

import * as React from 'react';

export function isReactElement(
  child: React.ReactChild
): child is React.ReactElement<any> { // tslint:disable-line:no-any
  return Boolean(child && (child as JSX.Element).type);
}

export function walk(
  replace: (child: React.ReactChild) => [boolean, React.ReactChild],
  elements: React.ReactChild[]
): React.ReactChild[] {
  return elements.map((child: React.ReactChild) => {
    const [shouldReplace, target]: [boolean, React.ReactChild] = replace(child);
    if (shouldReplace) {
      // The child targets on the replacing element;
      return target;
    } else if (isReactElement(child)) {
      // The child is other type. We are checking its children.
      const originalChildren: React.ReactChild[] = React.Children.toArray(child.props.children);
      const injectedChildren: React.ReactChild[] = walk(replace, originalChildren);
      return <child.type {...child.props} key={child.key} children={injectedChildren} />;
    } else {
      // The child is either string or number or null.
      return child;
    }
  });
}
