import { BrowserUtilities } from '@ms/sp-component-utilities';

/**
 * Utility related to native app rendering which uses WebView.
 * Currently the main purpose is to help deal with link interception when
 * layouts do not use anchor tags for their links.
 */
export default class WebViewUtility {
  public static getOpenDocumentAction(url: string): string | undefined {
    if (!url || !BrowserUtilities.isWebViewHosted()) {
      return undefined;
    }

    return JSON.stringify({
      key: WebViewUtility.documentKey,
      args: {
        command: WebViewUtility.documentCommand,
        url: url,
        version: '1.0'
      }
    });
  }

  public static get webViewActionRole(): string | undefined {
     return BrowserUtilities.isWebViewHosted() ? 'button' : undefined;
  }

  public static get documentKey(): string {
     return 'openDocument';
  }

  public static get documentCommand(): string {
     return 'sitePageUserAction';
  }
}