/**
 * @file IBaseCollectionWebPartOptions.ts
 * @Copyright (c) Microsoft Corporation.  All rights reserved.
 */
import { IButtonOptions } from '@ms/sp-component-utilities';
import { IMessageView } from './IMessageView';
import { IPaginationProps } from '../../pagination/index';

// @todo: Can qosPrefix be derived from the manifest?

// Options that can be changed post initialization via the setOptions function.
export interface IBaseCollectionWebPartLayoutState {
  /*
   * Set to true to render an editable web part title.  Defaults to false.
   */
  showTitle?: boolean;

  /*
   * Set to true to render the title placeholder text in read mode if the user has not modified it.  Default is false.
   */
  displayPlaceholderTitleInReadMode?: boolean;

  /**
   * Configuration of the add button.
   */
  addButton?: IButtonOptions;

  /**
   * Configuration of the "see all" link.
   */
  seeAllLink?: IButtonOptions;

  /**
   * Set to render a pagination control.
   *
   * How to use:
   * 1. Provide an onPageChange callback from the web part's code.
   * 2. In the onPageChange callback update the currentPage value and call the refreshData method.
   * 3. Override getRequest method to construct request parameters per the current page.
   * 4. In the DataProvider requestData method, parse the current page index from the params and
   *    return corresponding sub items.
   */
  paginationProps?: IPaginationProps;

  /**
   * Set to render an informational message.
   */
  messageView?: IMessageView;
}

// Options that are only settable during initialization when the get Options() function is invoked.
export interface IBaseCollectionWebPartOptions extends IBaseCollectionWebPartLayoutState {
  /*
   * Used as a prefix for quality of service monitor tags. Example: 'HighlightedContentWebPart'
   * The following QosMonitors are initiated by the BaseCollectionWebPart code:
   *     {qosPrefix}Mount
   *     {qosPrefix}Render
   */
  qosPrefix: string;

  /*
   * Set to true to add webId, siteId and baseUrl to web part properties based on SharePoint page context variables.
   * Defaults to false.
   */
  requirePageContextVariables?: boolean;

  /*
   * Time delay in milliseconds before an empty response is indicated in the UI via a "We didn't find anything to
   * show here" message.  The delay avoids flashing the message unnecessarily.  Default value is 500.
   */
  delayBeforeNoResultsMessage?: number;

  /*
   * Set to true to enable WebPart result caching.  Defaults to false.
   */
  canCacheResults?: boolean;
}