/**
 * @file IDataProviderInfo.ts
 * @Copyright (c) Microsoft Corporation.  All rights reserved.
 */

import { IDataProvider, IDataProviderOptions } from '@ms/sp-component-utilities';
import { IDataProviderOption } from './IDataProviderOption';

export interface IDataProviderInfo {
  // Unique identifier for the data provider.  For example, this can be 'Search' for a data provider implemented
  // with the SearchDataProvider.ts file.
  id: string;

  // Attributes used in a property pane dropdown for selecting which data provider to use.
  // Can be ignored if there is only one data provider option.
  option?: IDataProviderOption;

  // Attributes for selecting a data provider when more than one are available to a web part.
  isSelected?: boolean;
  title?: string; // localized name of the data provider displayed to the user when selecting data providers

  // This should only be set if your DataProvider needs custom options to be passed in and
  // initialized in its constructor. If this is empty, it will be set automatically with _spHttpClient.
  options?: IDataProviderOptions;

  // Promise returning the class with the requestData function.
  // Use Promise.resolve() if you are not dynamically loading your data provider code.
  //
  // The IDataProvider interface takes generics for the request and response data types; they are
  // defined as any here to avoid having to add generics to IDataProviderInfo signature as
  // that causes problems when declaring an array of IDataProviderInfo objects with each one having
  // it's own custom request and response formats.
  //
  // tslint:disable-next-line:no-any
  loadDataProvider: (options: IDataProviderOptions) => Promise<IDataProvider<any, any>>;

  // This receives the data provider instance so that it doesn't have to be re-allocated on each request.
  dataProvider?: IDataProvider<any, any>; // tslint:disable-line:no-any
}