/**
 * @file ILayoutInfo.ts
 * @Copyright (c) Microsoft Corporation.  All rights reserved.
 */

import * as React from 'react';

import DisplayFieldMap from '../../displayFieldMap/DisplayFieldMap';
import { ILayoutOption } from './ILayoutOption';

// Configure a layout for use by the BaseCollectionWebPart.
export interface ILayoutInfo {
  // Unique identifier for the layout.  For example, this can be 'Card' for a layout implemented with the
  // CardLayout.ts file.
  id: string;

  // Attributes used in a property pane choice group for selecting which layout to use.
  // Can be ignored if there is only one layout option.
  option?: ILayoutOption;

  // Promise returning the component class that does the rendering.
  // Use Promise.resolve() if you are not dynamically loading the code.
  loadLayout: () => Promise<React.ComponentClass<any>>; // tslint:disable-line:no-any

  // Properties to forward to the layout upon initialization.  Defined as any as the set of properties
  // will vary based upon the layout.
  properties?: any; // tslint:disable-line:no-any

  // Default display field maps for when this layout is being used by the identified data provider.
  // For example, you may have a default mapping for a Search API data provider, another for an
  // Activity feed data provider and another for a Graph API data provider.
  displayFieldMaps?: { [dataProviderId: string]: DisplayFieldMap };

  // Set to true to render immediately before data is retrieved.  Using this allows wep parts developers
  // to have their own loading state.
  hasInitialLoadingState?: boolean;
}