/**
 * @copyright Microsoft Corporation. All rights reserved.
 * @file Data source that calls the Bing image search endpoint
 */

import { UriEncoding } from '@ms/odsp-utilities-bundle';
import { HttpClient, HttpClientConfiguration, HttpClientResponse } from '@microsoft/sp-http';

import { IBingImageSearchResponse } from './models/IBingImageSearchResponse';

export default class BingImageSearch {
  private _httpClient: HttpClient;
  private _configuration: HttpClientConfiguration;

  constructor (httpClient: HttpClient) {
    this._httpClient = httpClient;
    this._configuration = new HttpClientConfiguration({});
  }

  /**
   * Perform a Bing image search, with parameters as the same parameters for the query
   *
   * @param query - search text
   * @param count - maximum number of results, defaulting to 50
   * @param offset - batch offset for paged results
   * @param mkt - language/region code for targeted region, e.g., en-US
   */
  public getImages(
    query?: string, count: number = 50, offset?: number, mkt?: string): Promise<IBingImageSearchResponse> {
    const apiUrl: string = 'https://www.bingapis.com/api/v7/images/search';
    const apiKey: string = 'D41D8CD98F00B204E9800998ECF8427E06193A4E';
    const encodedQuery: string = UriEncoding.encodeURIComponent(query);

    let url: string = `${apiUrl}?q=${encodedQuery}&appid=${apiKey}&traffictype=Internal_monitor&count=${count}`;
    if (offset) {
      url += `&offset=${offset}`;
    }
    if (mkt) {
      url += `&mkt=${UriEncoding.encodeURIComponent(mkt)}`;
    }

    return new Promise((resolve, reject) => {
      this._httpClient.get(url, this._configuration)
        .then((response: HttpClientResponse) => response.json())
        .then((value: IBingImageSearchResponse) => {
          resolve(value);
        })
        .catch((error: Error) => {
          reject(error);
        });
    });
  }
}
