/**
 * @copyright Microsoft Corporation. All rights reserved.
 * @file IBingImageSearchResponse.ts
 */

/**
 * Result from the Bing https://api.cognitive.microsoft.com/bing/v7.0/images/search endpoint.
 * The result contains additional content but this only declares what is being used.
 *
 * See: https://docs.microsoft.com/en-us/azure/cognitive-services/bing-image-search/quick-start,
 *      https://docs.microsoft.com/en-us/rest/api/cognitiveservices/bing-images-api-v7-reference
 */

export interface IBingImageSearchThumbnail {
  height: number;
  width: number;
}

/**
 * Properties from Bing Image Search API call
 *
 * Additional properties can be added to be enabled. See API docs
 */
export interface IBingImageSearchItem {
  contentSize: string;
  contentUrl: string;
  encodingFormat: string;
  height: number;
  imageId: string;
  name: string;
  thumbnail: IBingImageSearchThumbnail;
  thumbnailUrl: string;
  width: number;
}

/**
 * Properties returned from Bine Image Search API call
 *
 * Additional properties can be added to be enabled. See API docs
 */
export interface IBingImageSearchResponse {
  nextOffset: number;
  value: IBingImageSearchItem[];
}