/**
 * @Copyright (c) Microsoft Corporation.  All rights reserved.
 */

import { Validate } from '@microsoft/sp-core-library';
import { IDataProvider, IDataProviderConfiguration, IRequest  } from './IDataProvider';

/**
 * Base data provider.
 *
 * @alpha
 */
export abstract class BaseDataProvider implements IDataProvider {

  public readonly configuration: IDataProviderConfiguration;

  constructor (configuration: IDataProviderConfiguration) {
    Validate.isNotNullOrUndefined(configuration, 'configuration');
    Validate.isNotNullOrUndefined(configuration.serviceScope, 'configuration.serviceScope');

    this.configuration = configuration;
  }

  public abstract requestData<TData>(request: IRequest<TData>): Promise<TData>;
}