
import { Validate } from '@microsoft/sp-core-library';
import { BaseDataProvider }  from './BaseDataProvider';
import { IDataProviderConfiguration, IRequest } from './IDataProvider';
import { MySiteWebPartCacheManager } from '../mySiteWebPartCache/MySiteWebPartCacheManager';

/**
 * This is a prototype at the current point. The goal is to build some base code that will
 * help our customers follow the performance best practices to build web parts.
 *
 * @alpha
 */
export default class MysiteCacheDataProvider extends BaseDataProvider {
  private _cacheManager: MySiteWebPartCacheManager;

  constructor(configuration: IDataProviderConfiguration) {
    super(configuration);
    this._cacheManager = configuration.serviceScope.consume(MySiteWebPartCacheManager.serviceKey);
  }

  // Format of data changes per the request so return is an "any".
  public requestData<TData>(request: IRequest<TData>): Promise<TData> {
    Validate.isNotNullOrUndefined(request, 'request');
    Validate.isNotNullOrUndefined(request.fetchData, 'request.fetchData');
    Validate.isNotNullOrUndefined(request.dataReceived, 'request.dataReceived');
    Validate.isNonemptyString(request.cacheKey, 'request.cacheKey');
    Validate.isNonemptyString(request.alias, 'request.alias');

    /* tslint:disable:no-any */
    return this._cacheManager.requestWebPartCacheData(
      request.fetchData as any,
      request.dataReceived as any,
      request.cacheParameters,
      request.cacheKey,
      request.alias) as any;
    /* tslint:enable:no-any */
  }
}