/**
 * @Copyright (c) Microsoft Corporation.  All rights reserved.
 */

import { autobind } from 'office-ui-fabric-react';
import { BaseDataWebPart } from '../BaseDataWebPart';
import { IResponse } from '../IDataProvider';

/**
 * Type that represents the web part properties
 */
export interface ITestDataProperties {
  name: string;
  value: string;
}

/**
 * Web part response data type. In this case it is same as the
 * web part properties, but it does not have to be.
 */
export interface ITestResponseData extends ITestDataProperties {
}

/**
 * This is a prototype at the current point. The goal is to build some base code that will
 * help our customers follow the performance best practices to build web parts.
 *
 * @alpha
 */
export default class TestDatawebPart extends
  BaseDataWebPart<ITestDataProperties, ITestResponseData> {

  public render(): void {
    this.domElement.innerHTML = `<span>{this.properties.name}</span>`;
  }

  /*
   * Return the request data parameter required by the data provider to retrieve events.
   *
   * @override
   */
  protected get cacheParameters (): { [key: string]: Object } { // tslint:disable-line:no-any

    return {
      name: this.properties.name,
      value: this.properties.value
    };
  }

  /**
   * Callback when date is fetched and updated. In case of a cache hit, this callback
   * is called twice. Once for data from the cache and next for data from the actual
   * source. In case of a cache miss, the callback happens only once with data from
   * the actual source only.
   *
   * @override
   */
  @autobind
  protected dataReceived(response: IResponse<ITestResponseData>): void {
    this.properties = response.data;
    this.render();
  }

  /**
   * Fetch the web part data.
   *
   * @override
   */
  @autobind
  protected fetchData(): Promise<ITestResponseData> {
    return (
      new Promise<ITestResponseData>(
        (complete: (response: ITestResponseData) => void, reject: (error: Error) => void) => {
          complete({
            name: 'TestName',
            value: 'TestValue'
          } as ITestResponseData);
      }));
  }
}
