/**
 * @file MappedField.ts
 * @copyright Microsoft Corporation. All rights reserved.
 */

import * as React from 'react';
import { IDataRow } from '@ms/sp-component-utilities';
import { IDisplayField } from './models/IDisplayField';

export default class MappedField {
  private _target: string;
  private _displayField: IDisplayField;
  private _onRenderCell: (item?: IDataRow, index?: number) => React.ReactNode;

  constructor(
    target: string,
    displayField: IDisplayField,
    onRenderCell?: (item?: IDataRow, index?: number) => React.ReactNode) {

    this._target = target;
    this._displayField = displayField;
    this._onRenderCell = onRenderCell;
  }

  public get target(): string {
    return this._target;
  }

  public get sources(): string[] {
    return this._displayField.sources || [];
  }

  public get linkUrls(): string[] {
    return this._displayField.linkUrls || [];
  }

  public get heading(): string {
    return this._displayField.heading || '';
  }

  public get width(): number {
    // tslint:disable-next-line:no-null-keyword
    return this._displayField.width === undefined || this._displayField.width === null ? 100 : this._displayField.width;
  }

  public get onRenderCell(): (item?: IDataRow, index?: number) => React.ReactNode {
    return this._onRenderCell;
  }
}