/**
 * Lazy Loader react component to load child components on demand
 * @Copyright (c) Microsoft Corporation.  All rights reserved.
 *
 * @file LazyLoader.tsx
 */

import { ISPLazyLoaderProps } from './ISPLazyLoaderProps';
import * as React from 'react';
import LazyLoader from './LazyLoader';

/**
 * The Component to be used to lazy load any react component.
 * Usage
 *  1. <SPLazyLoader placeholder={placeholder} offset={200} >{lazyComponent}</SPLazyLoader>
 *  2. <SPLazyLoader height={300} offset={200}>{lazyComponent}</SPLazyLoader>
 */
export default class SPLazyLoader extends React.Component<ISPLazyLoaderProps, {}> {
  public render(): React.ReactElement<ISPLazyLoaderProps> {
    return (
      <LazyLoader
        placeholder={ this.props.placeholder }
        offset={ this.props.offset }
        height={ this.props.height }
        onRendered={ this.props.onRendered }>{this.props.children}</LazyLoader>
    );
  }
}