import * as React from 'react';
import LazyLoader, {ILazyLoaderProps, ILazyLoaderState} from '../LazyLoader';

import { expect } from 'chai';
import { shallow, ShallowWrapper } from 'enzyme';

describe('LazyLoader component tests', () => {

  it('Lazy loader default properties initialized as expected', () => {
    const lazyLoader: ShallowWrapper<ILazyLoaderProps, ILazyLoaderState> =
      shallow<ILazyLoaderProps, ILazyLoaderState>(React.createElement(LazyLoader, undefined));

    expect(lazyLoader.state('isRendered')).to.be.eq(false);
    expect(lazyLoader.prop('height')).to.be.eq(250);
  });

  it('Lazy loader should render custom placeholder', () => {
    const mockLazyLoaderProps: ILazyLoaderProps = {
      height: 200,
      offset: 0,
      placeholder: (
        <div
          className={'lazyloader-test-custom-placeholder'}
          { ...{ height: 489 } /* VSO:391730 Remove this workaround */ }>
          Mocked Placeholder
        </div>
      )
    };

    const lazyLoader: ShallowWrapper<ILazyLoaderProps, ILazyLoaderState> =
      shallow<ILazyLoaderProps, ILazyLoaderState>(React.createElement(LazyLoader, mockLazyLoaderProps));

    expect(lazyLoader.state('isRendered')).to.be.eq(false);
    expect(lazyLoader.prop('height')).to.be.eq(489);

    expect(lazyLoader.find('div.lazyloader-test-custom-placeholder').length).to.be.eq(1);
  });

  it('Lazy loader should render default placeholder', () => {
    const mockLazyLoaderProps: ILazyLoaderProps = {
      height: 234
    };

    const lazyLoader: ShallowWrapper<ILazyLoaderProps, ILazyLoaderState> =
      shallow<ILazyLoaderProps, ILazyLoaderState>(React.createElement(LazyLoader, mockLazyLoaderProps));
    expect(lazyLoader.find('div.office-ui-lazyloader-placeholder').length).to.be.eq(1);

    expect(lazyLoader.state('isRendered')).to.be.eq(false);
    expect(lazyLoader.prop('height')).to.be.eq(234);
  });
});
