/**
 * @copyright Microsoft Corporation. All rights reserved.
 *
 * @file ILessTextProps.ts
 */

import * as React from 'react';

export interface ILessTextProps {
  /**
   * Lines of text to show in the text container.
   */
  lines: number;

  /**
   * Symbol to append to the truncated text.
   */
  omission?: string;

  /**
   * Symbol/Regex used for spliting text when truncating text.
   */
  separator?: string;

  /**
   * Embed full text into title attribute of the text container.
   */
  showTitle: boolean;

  /**
   * Original text to process.
   */
  text: string;

  /**
   * Text container css class.
   */
  className: string;

  /**
   * The delay of resize execution.
   */
  resizeDelayInMilliseconds?: number;

  /**
   * Callback to extecute to determine if need truncate text.
   */
  shouldShowMore?: () => boolean;

  /**
   * Callback for text container click event.
   */
  onClick?: (evt: React.MouseEvent<HTMLDivElement>) => void;
}

export default ILessTextProps;
