import * as React from 'react';
import strings from './loc/Strings.resx';

import styles from './css/LightBox.module.scss';
import {
  css,
  getRTL,
  BaseComponent,
  KeyCodes,
  IBaseProps,
  Icon
} from 'office-ui-fabric-react';

export interface ISlideChangeButtonProps extends IBaseProps {
  onClick: (event?: React.MouseEvent<any>) => void; // tslint:disable-line:no-any
  buttonDirection: ButtonDirection;
  shouldFocus?: boolean;
  offsetPixels?: number;
  isTabStop?: boolean;
  className?: string;
}

export enum ButtonDirection {
  Next,
  Prev
}

const DEFAULT_OFFSET_PIXELS: number = 17;

export default class SlideChangeButton extends BaseComponent<ISlideChangeButtonProps, {}> {

  private _button: HTMLButtonElement;

  public componentDidUpdate(): void {
    if (this.props.shouldFocus) {
      this._async.setTimeout(() => {
        this._button.focus();
      }, 0);
    }
  }

  public render(): JSX.Element {
    const isRight: boolean = this._xor(getRTL(), this.props.buttonDirection === ButtonDirection.Next);
    const isNext: boolean = this.props.buttonDirection === ButtonDirection.Next;
    const offsetValue: number = this.props.offsetPixels !== undefined ? this.props.offsetPixels : DEFAULT_OFFSET_PIXELS;
    const calculatedStyle: React.CSSProperties = isRight ? { right: offsetValue } : { left: offsetValue };
    const buttonTabStop: number = this.props.isTabStop ? (isRight ? 1 : 2) : -1;

    return (
      <div className={ css(styles.indexButtonContainer, this.props.className) } style={ calculatedStyle }>
        <button
          tabIndex={ buttonTabStop }
          ref={ this._resolveRef('_button') }
          className={ css(styles.indexButton, isRight ? styles.rightPositioned : styles.leftPositioned, 'ms-font-l') }
          onClick={ this.props.onClick }
          onKeyPress={ this._handleKeyboardNavigation }
          title={ isNext ? strings.LightBoxButtonNext : strings.LightBoxButtonPrevious }
          data-automation-id={ isRight ? 'next-button' : 'prev-button' }>
          <Icon iconName={ isRight ? 'ChevronRight' : 'ChevronLeft' } />
        </button>
      </div>
    );
  }

  private _xor(a: boolean, b: boolean): boolean {
    return (a || b) && !(a && b);
  }

  /* tslint:disable-next-line:no-any */
  private _handleKeyboardNavigation(event: React.KeyboardEvent<any>): void {
    if (this.props.onClick &&
      event.keyCode === KeyCodes.right && this._xor(getRTL(), this.props.buttonDirection === ButtonDirection.Next) ||
      event.keyCode === KeyCodes.left && !this._xor(getRTL(), this.props.buttonDirection === ButtonDirection.Next)) {
      this.props.onClick();
    }
  }
}