/// <reference types="mocha" />
/* tslint:disable:no-any */
/* tslint:disable:no-empty */

import { assert } from 'chai';
import * as React from 'react';
import { shallow, ShallowWrapper } from 'enzyme';
import LightBox, { ILightBoxProps, ILightBoxState } from '../LightBox';

describe('LightBox tests', () => {
  let lightBox: ShallowWrapper<ILightBoxProps<any>, ILightBoxState>;

  const mockProps: ILightBoxProps<any> = {
    items: ['testItem1', 'testItem2', 'testItem3'] as any[],
    startIndex: 1,
    isShown: true,
    onRenderItem: (item: string, index: number): JSX.Element => { return <div key={ index }>{ item }</div>; },
    onClose: (): void => { }
  };

  beforeEach(() => {
    lightBox = shallow<ILightBoxProps<any>, ILightBoxState>(
      React.createElement(LightBox, mockProps));
  });

  afterEach(() => {
    lightBox.unmount();
  });

  it('should open at specified index', () => {
    assert.equal(lightBox.state('currentIndex'), 1);
  });

  it('should render next item and it should be hidden', () => {
    assert.equal(lightBox.find(`[data-automation-id='next-item']`).length, 1);
    assert.isTrue(lightBox.find(`[data-automation-id='next-item']`).html().indexOf('hidden') > 0);
  });

  it('should render prev item and it should be hidden', () => {
    assert.equal(lightBox.find(`[data-automation-id='prev-item']`).length, 1);
    assert.isTrue(lightBox.find(`[data-automation-id='prev-item']`).html().indexOf('hidden') > 0);
  });
});