/**
 * @file IMySiteWebPartCacheDataItem.ts
 * @copyright Microsoft Corporation. All rights reserved.
 *
 * @remark Describes the MySite WebPartCache Item.
 */

import { Guid } from '@microsoft/sp-core-library';

/**
 * Item that represents a MySite WebPart Cache item.
 */
export interface IMySiteWebPartCacheDataItem {
  /**
   * The unique ID of the WebPart.
   * This is a string value that could be anything but is guaranteed to be unique at page scope.
   */
  webPartInstanceId: string;

  /**
   * The cached result value.
   */
  result: string;

  /**
   * The hashed WebPart properties.
   */
  hash: string;

  /**
   * The SharePoint ID of the item.
   */
  id?: number;

  /**
   * The SharePoint UniqueId of the item.
   */
  uniqueId?: Guid;

  /**
   * The last modified time of the item (when it was written to cache).
   */
  modified: Date;

  /**
   * The cache version of the item (when it was written to cache).
   */
  cacheVersion: number;
}
