/**
 * @file MySiteWebPartCacheManager.ts
 * @copyright Microsoft Corporation. All rights reserved.
 *
 * @remark The MySite WebPart Cache Manager for fetching and retrieving WebPart result data.
 */

import { ServiceKey, ServiceScope } from '@microsoft/sp-core-library';
import { MySiteWebPartCacheDataProvider } from './MySiteWebPartCacheDataProvider';
import { IDataRow } from '@ms/sp-component-utilities';

export class MySiteWebPartCacheManager {

  public static readonly serviceKey: ServiceKey<MySiteWebPartCacheManager> =
  ServiceKey.create<MySiteWebPartCacheManager>(
    'sp-component-utilities:MySiteWebPartCacheManager', MySiteWebPartCacheManager);

  private _mySiteWebPartCacheDataProvider: MySiteWebPartCacheDataProvider;

  public constructor(serviceScope: ServiceScope) {
    this._mySiteWebPartCacheDataProvider = new MySiteWebPartCacheDataProvider(serviceScope);
  }

  /**
   * Returns cached data for the calling WebPart while handling caching and re-rendering.
   *
   * @param requestData - A function that returns an IDataRow[] and makes the service data request.
   * @param dataUpdated - A function that saves the response to the WebPart and then renders.
   * @param config - The current WebPart config.
   * @param webPartInstanceId - The WebPart instanceId.
   * @param webPartAlias - The WebPart alias (manifest alias).
   * @param rerender - Optional: Controls whether or not to re-render on new data after rendering cached data.
   *                   Default: Always re-render.
   * @param secondsToInvalidate - Optional: A number representing the amount seconds to expire cached results.
   *                              Default: Don't expire cached results automatically.
   * @returns - A promise that represents WebPart data returned from the request or cache.
   */
  public requestWebPartCacheData(
    requestData: () => Promise<IDataRow[]>,
    dataUpdated: (response: IDataRow[]) => void,
    config: any, // tslint:disable-line:no-any
    webPartInstanceId: string,
    webPartAlias: string,
    rerender?: boolean,
    secondsToInvalidate?: number): Promise<IDataRow[]> {
      return this._mySiteWebPartCacheDataProvider.getWebPartData(
        requestData,
        dataUpdated,
        config,
        webPartInstanceId,
        webPartAlias,
        rerender,
        secondsToInvalidate);
  }
}