/**
 * @copyright Microsoft Corporation. All rights reserved.
 *
 * @file IPageProps.ts
 */

export enum PageType {
  /**
   * Represents the page button with index which let user goes to specific page.
   */
  Default = 0,

  /**
   * Represents the page button which let user goes to the previous page.
   */
  Prev = 1,

  /**
   * Represents the page button which let user goes to the next page.
   */
  Next = 2
}

export interface IPageProps {
  /**
   * Optional override for page button css class.
   */
  className?: string;

  /**
   * Text value of the page button.
   */
  text: string;

  /**
   * Page number of the page button.
   */
  pageNumber?: number;

  /**
   * Type of the page button.
   */
  pageType: PageType;

  /**
   * Set to true if the page button is disabled.
   */
  disabled: boolean;

  /**
   * Set to true if the page button is selected.
   */
  selected: boolean;

  /**
   * Callback to execute when the page button is clicked.
   */
  onClick?: (pageNumber?: number, pageType?: PageType) => void;
}
