﻿/**
 * @copyright Microsoft Corporation. All rights reserved.
 *
 * @file IPaginationProps.ts
 */

import { PageType } from './IPageProps';

export interface IPaginationProps {
  /**
   * Optional override for pagination css class.
   */
  className?: string;

  /**
   * Optional override for page button css class.
   */
  pageClassName?: string;

  /**
   * Total number of items that need to be paginated.
   */
  itemsCount?: number;

  /**
   * Number of items to display on one paginated page.
   */
  itemsPerPage?: number;

  /**
   * Current pagination page number.
   */
  currentPage?: number;

  /**
   * Total pages to display in pagination.
   */
  pageLength?: number;

  /**
   * To show page number or not.
   */
  showPageNum?: boolean;

  /**
   * To show 'Next' page or not.
   */
  hasNextPage?: boolean;

  /**
   * To show 'Prev' page or not.
   */
  hasPrevPage?: boolean;

  /**
   * Callback for pagination page change.
   */
  onPageChange: (pageNumber: number, pageType: PageType) => void;
}
