﻿/**
 * @copyright Microsoft Corporation. All rights reserved.
 *
 * @file Page.tsx
 */

import * as React from 'react';
import { autobind, css, ActionButton, Icon } from 'office-ui-fabric-react';
import { Locale, StringHelper } from '@ms/odsp-utilities-bundle';

import { IPageProps, PageType } from './IPageProps';
import strings from './Strings.resx';
import styles from './Pagination.module.scss';

export default class Page extends React.PureComponent<IPageProps, {}> {
  public render(): JSX.Element {
    const isPrevPage: boolean = this.props.pageType === PageType.Prev;
    const isNextPage: boolean = this.props.pageType === PageType.Next;
    const pageClassName: string = css(
      {
        [styles.selected]: this.props.selected,
        [styles.prev]: isPrevPage,
        [styles.next]: isNextPage
      },
      this.props.className
    );

    return (
      <ActionButton
        className={pageClassName}
        ariaLabel={this._pageAriaLabel}
        disabled={this.props.disabled}
        onClick={this._handleClick}
      >
        {isPrevPage && <Icon iconName={Locale.isRightToLeft ? 'ChevronRight' : 'ChevronLeft'} aria-hidden='true' />}

        {this.props.text}

        {isNextPage && <Icon iconName={Locale.isRightToLeft ? 'ChevronLeft' : 'ChevronRight'} aria-hidden='true' />}
      </ActionButton>
    );
  }

  @autobind
  private _handleClick(): void {
    this.props.onClick(this.props.pageNumber, this.props.pageType);
  }

  private get _pageAriaLabel(): string {
    let ariaLabel: string;

    if (this.props.pageNumber) {
      ariaLabel = this.props.pageType === PageType.Default
        ? StringHelper.format(strings.Page, this.props.pageNumber)
        : StringHelper.format(strings.PrevNextButtonAriaLabel, this.props.text, this.props.pageNumber);
    } else {
      ariaLabel = StringHelper.format(strings.PrevNextButtonCompactModeAriaLabel, this.props.text);
    }

    return ariaLabel;
  }
}
