/// <reference types="mocha" />
/// <reference types="sinon" />
declare var sinon: sinon.SinonStatic;
/**
 * @copyright Microsoft Corporation. All rights reserved.
 *
 * @file Pagination.test.ts
 */

import { expect } from 'chai';
import { mount, ReactWrapper } from 'enzyme';
import * as React from 'react';

import { IPaginationProps } from '../IPaginationProps';
import Pagination from '../Pagination';
import strings from '../Strings.resx';

describe('<Pagination />', () => {
  let pagination: ReactWrapper<IPaginationProps, {}>;
  let mockProps: IPaginationProps;
  let onPageChangeSpy: sinon.SinonSpy;

  beforeEach(() => {
    onPageChangeSpy = sinon.spy();
  });

  afterEach(() => {
    if (pagination) {
      pagination.unmount();
    }
  });

  it('should display Previous and Next buttons.', () => {
    mockProps = {
      itemsCount: 9,
      itemsPerPage: 3,
      currentPage: 2,
      pageLength: 5,
      showPageNum: true,
      onPageChange: onPageChangeSpy
    };

    pagination = mount<IPaginationProps, {}>(React.createElement(Pagination, mockProps));

    expect(pagination.find('Page')).to.have.length(5);
    expect(pagination.find('Page').first().text()).to.be.contain(strings.PaginationPrev);
    expect(pagination.find('Page').last().text()).to.be.contain(strings.PaginationNext);
  });

});
