import { IBaseProps } from 'office-ui-fabric-react';

/**
 * Used to display a placeholder in case of no or temporary content. Button is optional.
 *
 * @public
 */
export interface IPlaceholderProps extends IBaseProps {
  /**
   * Text description for the placeholder. Appears bellow the Icon and IconText.
   */
  description: string;

  /**
   * Icon className from the MDL2 set. Example: 'Add'.
   */
  icon: string;

  /**
   * Heading displayed against the Icon.
   */
  iconText: string;

  /**
   * Text label to be displayed on button below the description.
   * Optional: As the button is optional.
   */
  buttonLabel?: string;

  /**
   * onClick handler for the button.
   * Optional: As the button is optional.
   */
  onAdd?: () => void;

  /**
   * This className is applied to the root element of content. Use this to
   * apply custom styles to the placeholder.
   */
  contentClassName?: string;

  /**
   * Width of the Webpart which the Placeholder is rendered in
   */
  webpartWidth?: number;

  /**
   * If more than one button, put the extra buttons in extraButtons.
   */
  extraButtons?: IPlaceholderButtonProps[];
}

export interface IPlaceholderButtonProps {
    /**
     * Text label to be displayed on button.
     */
    label: string;

    /**
     * onClick handler for the button.
     */
    onClick: () => void;

    /**
     * Button's description which is for accessibility and not visible to users.
     */
    description: string;
}
