/// <reference types="mocha" />
/// <reference types="sinon" />

import * as React from 'react';
import { expect } from 'chai';
import { shallow, ShallowWrapper } from 'enzyme';
import {
  Icon,
  PrimaryButton
} from 'office-ui-fabric-react';

import { IPlaceholderProps } from '../IPlaceholderProps';

import Placeholder from '../Placeholder';

/* tslint:disable:no-unused-expression */

describe('Placeholder component', () => {
  it('should render passed children', () => {
    const childNode: React.ReactHTMLElement<HTMLTextAreaElement> = React.createElement('textarea', undefined, 'test');
    const placeholder: ShallowWrapper<IPlaceholderProps, {}> = shallow(
      <Placeholder description='' icon='Page' iconText='' children={ childNode } />
    );

    expect(placeholder).not.to.be.null.and.not.undefined;
    expect(placeholder.find('textarea')).to.have.lengthOf(1);
    expect(placeholder.find('textarea').text()).to.equal('test');
  });

  it('should render icon', () => {
    const placeholder: ShallowWrapper<IPlaceholderProps, {}> = shallow(
      <Placeholder description='' icon='Page' iconText='' />
    );

    expect(placeholder.find(Icon)).to.have.lengthOf(1);
  });

  it('should render primary button if label specified', () => {
    const placeholder: ShallowWrapper<IPlaceholderProps, {}> = shallow(
      <Placeholder buttonLabel='Add' description= '' icon='Add' iconText='New' />
    );

    expect(placeholder).not.to.be.null.and.not.undefined;
    expect(placeholder.find(PrimaryButton)).to.have.lengthOf(1);
  });

  it('should not render primary button if no label specified', () => {
    const placeholder: ShallowWrapper<IPlaceholderProps, {}> = shallow(
      <Placeholder buttonLabel='' description= '' icon='Add' iconText='New' />
    );

    expect(placeholder).not.to.be.null.and.not.undefined;
    expect(placeholder.find(PrimaryButton)).to.have.lengthOf(0);
  });

  it('should render extra buttons if specified', () => {
    const placeholder: ShallowWrapper<IPlaceholderProps, {}> = shallow(
      <Placeholder
        buttonLabel='Add'
        description= ''
        icon='Add'
        iconText='New'
        extraButtons={ [ { label: 'Extra', description: 'extra test', onClick: () => {/*noop*/} } ] }
      />
    );

    expect(placeholder.find(PrimaryButton)).to.have.lengthOf(2);
  });
});