/// <reference types="mocha" />

import { expect } from 'chai';

import SetUtilities from './SetUtilities';

/* tslint:disable:no-unused-expression */

describe('Set Utilities', () => {
  describe('set union', () => {
    it('should union two undefined sets', () => {
      const setA: Set<number> = undefined;
      const setB: Set<number> = undefined;
      const union: Set<number> = SetUtilities.union(setA, setB);
      expect(union).to.not.be.undefined;
      expect(union.size).to.be.equal(0);
    });

    it('should union two empty sets', () => {
      const setA: Set<number> = new Set<number>();
      const setB: Set<number> = new Set<number>();
      const union: Set<number> = SetUtilities.union(setA, setB);
      expect(union).to.not.be.undefined;
      expect(union.size).to.be.equal(0);
    });

    it('should union one empty set and one valid set', () => {
      const setA: Set<number> = new Set<number>();
      const setB: Set<number> = new Set<number>([1]);
      const union: Set<number> = SetUtilities.union(setA, setB);
      expect(union).to.not.be.undefined;
      expect(union.size).to.be.equal(1);
      expect(union.has(1)).to.be.true;
    });

    it('should union one undefined set and one valid set', () => {
      const setA: Set<number> = undefined;
      const setB: Set<number> = new Set<number>([1]);
      const union: Set<number> = SetUtilities.union(setA, setB);
      expect(union).to.not.be.undefined;
      expect(union.size).to.be.equal(1);
      expect(union.has(1)).to.be.true;
    });

    it('should union two sets of same number of items, with one duplicate', () => {
      const setA: Set<number> = new Set<number>([1, 2]);
      const setB: Set<number> = new Set<number>([1, 3]);
      const union: Set<number> = SetUtilities.union(setA, setB);
      expect(union).to.not.be.undefined;
      expect(union.size).to.be.equal(3);
      expect(union.has(1)).to.be.true;
      expect(union.has(2)).to.be.true;
      expect(union.has(3)).to.be.true;
    });

    it('should union two sets of same number of items, with no duplicates', () => {
      const setA: Set<number> = new Set<number>([1, 2]);
      const setB: Set<number> = new Set<number>([4, 3]);
      const union: Set<number> = SetUtilities.union(setA, setB);
      expect(union).to.not.be.undefined;
      expect(union.size).to.be.equal(4);
      expect(union.has(1)).to.be.true;
      expect(union.has(2)).to.be.true;
      expect(union.has(3)).to.be.true;
      expect(union.has(4)).to.be.true;
    });

    it('should union two sets of different types', () => {
      const setA: Set<number> = new Set<number>([1, 2]);
      const setB: Set<string> = new Set<string>(['one', 'two']);
      const union: Set<any> = SetUtilities.union(setA, setB);  // tslint:disable-line:no-any
      expect(union).to.not.be.undefined;
      expect(union.size).to.be.equal(4);
      expect(union.has(1)).to.be.true;
      expect(union.has(2)).to.be.true;
      expect(union.has('one')).to.be.true;
      expect(union.has('two')).to.be.true;
    });
  });
});