/// <reference types="mocha" />
/// <reference types="sinon" />
declare var sinon: sinon.SinonStatic;
/* tslint:disable:no-any */
/* tslint:disable:no-empty */

import * as React from 'react';
import { shallow, ShallowWrapper } from 'enzyme';
import { expect } from 'chai';
import TouchArea, { ITouchAreaProps, SwipeDirection } from '../TouchArea';

/* tslint:disable:no-unused-expression */

describe('TouchArea tests', () => {
  let touchArea: ShallowWrapper<ITouchAreaProps, {}>;
  let mockProps: ITouchAreaProps;

  beforeEach(() => {
    mockProps = {
      preventDefault: true,
      onSwipe: (): void => { }
    };
  });

  it('should trigger a swipe left when swiping left with a mouse', () => {
    const swipeSpy: sinon.SinonSpy = sinon.spy((direction: SwipeDirection): void => { });
    mockProps.onSwipe = swipeSpy;
    touchArea = shallow<ITouchAreaProps, {}>(React.createElement(TouchArea, mockProps));
    touchArea.simulate('mousedown', { clientX: 1, clientY: 1, preventDefault: () => { } });
    touchArea.simulate('mouseup', { clientX: 10, clientY: 1, preventDefault: () => { } });
    expect(swipeSpy.calledWith(SwipeDirection.Left)).to.be.true;
  });

  it('should trigger a swipe left when swiping left using touch', () => {
    const swipeSpy: sinon.SinonSpy = sinon.spy((direction: SwipeDirection): void => { });
    mockProps.onSwipe = swipeSpy;
    touchArea = shallow<ITouchAreaProps, {}>(React.createElement(TouchArea, mockProps));
    touchArea.simulate('touchstart', { touches: [{ clientX: 1, clientY: 1 }], preventDefault: () => { } });
    touchArea.simulate('touchmove', { touches: [{ clientX: 10, clientY: 1 }], preventDefault: () => { } });
    expect(swipeSpy.calledWith(SwipeDirection.Left)).to.be.true;
  });

  it('should call preventDefault if specified', () => {
    const preventDefaultSpy: sinon.SinonSpy = sinon.spy((): void => { });
    touchArea = shallow<ITouchAreaProps, {}>(React.createElement(TouchArea, mockProps));
    touchArea.simulate('mousedown', { clientX: 1, clientY: 1, preventDefault: preventDefaultSpy });
    expect(preventDefaultSpy.called).to.be.true;
  });
});