/**
 * @file WebPartChrome.tsx
 * @Copyright (c) Microsoft Corporation.  All rights reserved.
 */
import * as React from 'react';
import {
  ActionButton,
  BaseComponent,
  Link,
  css
} from 'office-ui-fabric-react';

import {
  default as CaptionElement,
  CaptionElementType,
  CaptionElementAlignPosition
} from '../captionElement/CaptionElement';
import { IWebPartChromeProps } from './models/IWebPartChromeProps';
import { Pagination } from '../pagination/index';

import strings from './loc/Strings.resx';
import styles from './WebPartChrome.module.scss';

export default class WebPartChrome extends BaseComponent<IWebPartChromeProps, {}> {

  public static defaultProps: IWebPartChromeProps = {
    isEditMode: false,
    seeAllLink: { show: false },
    addButton: { show: false },
    contentElement: <div />
  };

  private _captionElement: CaptionElement;

  public render(): JSX.Element {
    return (
      <div className={ styles.webPartChrome }>
        <div className={ styles.webPartHeader }>
          { this._renderTitle() }
          { this._renderSeeAllLink() }
        </div>
        { this._renderContent() }
      </div>
    );
  }

  public get title(): string {
    return this._captionElement && this._captionElement.value() || '';
  }

  public focusOnTitle(): void {
    if (this._captionElement) {
      this._captionElement.focusOnCaptionElement();
    }
  }

  private _renderTitle(): JSX.Element {
    return this.props.showTitle && (
      this.props.customTitle ||
      <CaptionElement
        className={ css(styles.title,
        this.props.addButton.show ? styles.titleSmMargin : styles.titleLgMargin) }
        ref={ this._resolveRef( '_captionElement' ) }
        type={ CaptionElementType.Title }
        isEditMode={ this.props.isEditMode }
        captionText={ this.props.title }
        onCaptionTextChange={
          this.props.onTitleChange ? this.props.onTitleChange :
          (ev: React.FormEvent<HTMLTextAreaElement>) => { /* NO OP */ } }
        placeholder={ this.props.placeholder }
        alignPosition={ this.props.titleAlignment || CaptionElementAlignPosition.PageDirection }
    />);
  }

  private _renderContent(): JSX.Element {
    return (
      <div className={ styles.content }>
        { this._renderAddButton() }
        { this.props.contentElement }
        { this.props.pagination && <Pagination { ...this.props.pagination } /> }
    </div>);
  }

  private _renderSeeAllLink(): JSX.Element {
    return this.props.seeAllLink.show && this.props.seeAllLink.href &&
      <Link
        aria-label={ this.props.seeAllLink.ariaLabel || strings.SeeAllLinkAriaLabel }
        className={ styles.seeAll }
        disabled={ this.props.seeAllLink.disabled }
        onClick={ this.props.seeAllLink.onClick }
        data-interception='propagate'
        { ...{ href: this.props.seeAllLink.href } /* VSO:391730 Remove this workaround */ }>
        { this.props.seeAllLink.label || strings.SeeAllLink }
      </Link>;
  }

  private _renderAddButton(): JSX.Element {
    return this.props.addButton.show &&
      <ActionButton
        ariaLabel={ this.props.addButton.ariaLabel || strings.AddButtonAriaLabel }
        className={ styles.add }
        disabled={ this.props.addButton.disabled }
        href={ this.props.addButton.href }
        iconProps={ { iconName: 'CalculatorAddition' } }
        data-interception='propagate'
        onClick={ this.props.addButton.onClick }>
        { this.props.addButton.label || strings.AddButtonLabel }
      </ActionButton>;
  }
}