/**
 * @file IWebPartChromeProps.ts
 * @Copyright (c) Microsoft Corporation.  All rights reserved.
 */

import { IBaseProps } from 'office-ui-fabric-react';

import { IButtonOptions } from '@ms/sp-component-utilities';
import { IPaginationProps } from '../../pagination/IPaginationProps';
import { CaptionElementAlignPosition } from '../../captionElement/CaptionElement';

export interface IWebPartChromeProps extends IBaseProps {
  /**
   * Set to true to render the Title area for edit or the title for read when there is title text.
   */
  showTitle?: boolean;

  /**
   * Title rendered for the web part.
   */
  title?: string;

  /**
   * Specify the text should be one of page direction aligned or center aligned with RTL considered.
   * Page direction aligned is left aligned adjusted to right aligned for RTL.
   */
  titleAlignment?: CaptionElementAlignPosition;

  /**
   * Function for the web part to receive an entered title. If not provided, the title will not be
   * rendered as editable.
   */
  onTitleChange?: (ev: React.FormEvent<HTMLTextAreaElement>) => void;

  /**
   * Custom title element. Used to replace the standard web part title with a breadcrumb display.
   */
  customTitle?: JSX.Element;

  /**
   * Placeholder text printed lightly when a title entry is not provided. This may be used to suggest
   * what the default title would be if one is not provided.
   */
  placeholder?: string;

  /**
   * Should be set to true when the page is in edit mode. The "Add" button is hidden in edit mode to
   * prevent the user from navigating away from an unsaved page.
   */
  isEditMode: boolean;

  /**
   * Configuration of the "see all" link.
   */
  seeAllLink?: IButtonOptions;

  /**
   * Configuration of the add button.
   */
  addButton?: IButtonOptions;

  /**
   * Content to render in the web part.
   */
  contentElement: JSX.Element;

  /**
   * Properties for rendering the pagination component.
   */
  pagination?: IPaginationProps;
}