import * as React from 'react';
import {
  default as CaptionElement,
  CaptionElementType,
  CaptionElementAlignPosition,
  ICaptionElementProps
} from '../captionElement/CaptionElement';

export interface IWebPartTitleProps {
  edit: boolean;
  title: string;
  onTitleChange: (ev: React.FormEvent<HTMLTextAreaElement>) => void;
  placeholder: string;
  alignPosition?: CaptionElementAlignPosition;
  className?: string;
}

/**
 * Title for various webparts.
 */
export default class WebPartTitle extends React.Component<IWebPartTitleProps, {}> {
  public static defaultProps: IWebPartTitleProps = {
    edit: false,
    title: undefined,
    onTitleChange: () => { /* no-op */ },
    placeholder: undefined,
    className: undefined,
    alignPosition: CaptionElementAlignPosition.PageDirection
  };

  private _captionElement: CaptionElement;

  constructor(props: IWebPartTitleProps) {
    super(props);
  }

  public render(): JSX.Element {
    const captionElementProps: ICaptionElementProps = {
      type: CaptionElementType.Title,
      isEditMode: this.props.edit,
      captionText: this.props.title,
      onCaptionTextChange: this.props.onTitleChange,
      placeholder: this.props.placeholder,
      alignPosition: this.props.alignPosition,
      className: this.props.className
    };
    this._captionElement = new CaptionElement(captionElementProps);

    return this._captionElement.render();
  }

  public value(): string {
    return this._captionElement.value();
  }

  public focusOnWebPartTitle(): void {
    this._captionElement.focusOnCaptionElement();
  }
}