﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.SharePoint.Client;
using Microsoft.SharePoint.Client.WebParts;
using System.Security;
using System.Web;

namespace TryCSOM
{
    /// <summary>
    /// Class to help test provisioning client side web parts on classic sharepoint pages
    /// using CSOM APIs in SharePoint online.
    /// </summary>
    class ProvisionClientSideWebPart
    {
        /// <summary>
        /// Client side web part xml schema.
        /// </summary>
        static string webPartSchemaXml = @"<?xml version=""1.0"" encoding=""utf-8""?>
            <webParts>
              <webPart xmlns=""http://schemas.microsoft.com/WebPart/v3"">
                <metaData>
                  <type name=""Microsoft.SharePoint.WebPartPages.ClientSideWebPart, Microsoft.SharePoint, Version=16.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c"" />
                  <importErrorMessage>Cannot Import WebPart</importErrorMessage>
                </metaData>
                <data>
                  <properties>
                    <property name=""Title"" type=""string"">{0}</property>
                    <property name=""Description"" type=""string"">{1}</property>
                    <property name=""IconUrl"" type=""string"">{2}</property>
                    <property name=""ClientSideWebPartId"">{3}</property>
                    <property name=""ClientSideWebPartData"" type=""string"">{4}</property>
                  </properties>
                </data>
              </webPart>
            </webParts>";

        static void Main(string[] args)
        {
            // Step1: Create a client context
            // NOTE: the site URL, password and user login should be changed before running this code.
            ClientContext context = new ClientContext("https://a830edad9050849canary022811.sharepoint.com/sites/site1");
            SecureString pwd = new SecureString();
            pwd.AppendChar('a');
            pwd.AppendChar('b');
            pwd.AppendChar('c');
            pwd.AppendChar('d');
            context.Credentials = new SharePointOnlineCredentials("admin@a830edad9050849canary022811.onmicrosoft.com", pwd);

            // The SharePoint web at the URL.
            Web web = context.Web;

            // Step1: get all the web parts on a page

            // We want to retrieve the web's properties.
            context.Load(web);

            // Step2: Load the appropriate page. Note, a single column web part page works well with this sample code.
            File f = web.GetFileByServerRelativeUrl("/sites/site1/SiteAssets/Page2.aspx");
            context.Load(f);

            // Get web parts from the page.
            LimitedWebPartManager webPartManager = f.GetLimitedWebPartManager(PersonalizationScope.Shared);
            context.Load(webPartManager);
            WebPartDefinitionCollection webParts = webPartManager.WebParts;
            context.Load(webParts, wp => wp.Include(w => w.ZoneId, w => w.Id, w => w.WebPart.Title, w => w.WebPart.ZoneIndex, w => w.WebPart.Properties, w => w.WebPart.ExportMode, w => w.WebPart.Hidden, w => w.WebPart.IsClosed));

            // Checkout the page before adding web parts.
            f.CheckOut();
            bool checkedOut = true;

            // Execute the query to the server.
            context.ExecuteQuery();

            try
            {
                /* Format the web part xml string.
                string myWPString = String.Format(
                    webPartSchemaXml,
                    "My SiteActivityWebPart",
                    "My site activity description",
                    "",
                    "eb95c819-ab8f-4689-bd03-0c2d65d47b1f",
                    @"&lt; div data-sp-webpart="""" data-sp-webpartdataversion=1.0 data-sp-webpartdata=""&amp;#123;&amp;quot;id&amp;quot;&amp;#58;&amp;quot;eb95c819-ab8f-4689-bd03-0c2d65d47b1f&amp;quot;,&amp;quot;instanceId&amp;quot;&amp;#58;null,&amp;quot;title&amp;quot;&amp;#58;&amp;quot;Site activity&amp;quot;,&amp;quot;description&amp;quot;&amp;#58;&amp;quot;Show recent actitvies in the current site&amp;quot;,&amp;quot;version&amp;quot;&amp;#58;&amp;quot;0.0.1&amp;quot;,&amp;quot;properties&amp;quot;&amp;#58;&amp;#123;&amp;#125;,&amp;quot;htmlProperties&amp;quot;&amp;#58;null&amp;#125;""&gt;&lt;div data-sp-componentid=""""&gt;eb95c819-ab8f-4689-bd03-0c2d65d47b1f&lt;/div&gt;&lt;div data-sp-htmlproperties=""""&gt;&lt;/div&gt;&lt;/div&gt;");
                 */

                // Step3: Add the web part to the page and checkin the page.
                // This is where you need to add the web part XML from the workbench
                string myWPString = @"<webParts>
  <webPart xmlns=""http://schemas.microsoft.com/WebPart/v3"">
    <metaData>
      <type name=""Microsoft.SharePoint.WebPartPages.ClientSideWebPart,
        Microsoft.SharePoint, Version=16.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c"" />
      <importErrorMessage>Cannot Import WebPart</importErrorMessage>
    </metaData>
    <data>
      <properties>
        <property name=""Title"" type=""string"">Quick chart</property>
        <property name=""Description"" type=""string"">Visualize your key data in a bar or pie chart.</property>
        <property name=""IconUrl"" type=""string""></property>
        <property name=""ClientSideWebPartId"">91a50c94-865f-4f5c-8b4e-e49659e69772</property>
        <property name=""ClientSideWebPartData"" type=""string"">&lt;div data-sp-webpart="""" data-sp-webpartdataversion=""1.0"" data-sp-webpartdata=""{&amp;quot;id&amp;quot;:&amp;quot;91a50c94-865f-4f5c-8b4e-e49659e69772&amp;quot;,&amp;quot;instanceId&amp;quot;:&amp;quot;0625bcf9-9e20-4605-8ca0-f8c67eba33ef&amp;quot;,&amp;quot;title&amp;quot;:&amp;quot;Quick chart&amp;quot;,&amp;quot;description&amp;quot;:&amp;quot;Visualize your key data in a bar or pie chart.&amp;quot;,&amp;quot;dataVersion&amp;quot;:&amp;quot;1.0&amp;quot;,&amp;quot;properties&amp;quot;:{&amp;quot;data&amp;quot;:[{&amp;quot;id&amp;quot;:&amp;quot;7CFFD4B0-436E-430D-94C5-A4F9D22DB3FE&amp;quot;,&amp;quot;label&amp;quot;:&amp;quot;Tiger&amp;quot;,&amp;quot;value&amp;quot;:&amp;quot;10&amp;quot;,&amp;quot;valueNumber&amp;quot;:10},{&amp;quot;id&amp;quot;:&amp;quot;b1cbabca-88ea-4013-8a98-263b5c951784&amp;quot;,&amp;quot;label&amp;quot;:&amp;quot;Bear&amp;quot;,&amp;quot;value&amp;quot;:&amp;quot;20&amp;quot;,&amp;quot;valueNumber&amp;quot;:20},{&amp;quot;id&amp;quot;:&amp;quot;eefc18a7-12d1-4d65-a516-91ff4611c8dd&amp;quot;,&amp;quot;label&amp;quot;:&amp;quot;Panda&amp;quot;,&amp;quot;value&amp;quot;:&amp;quot;4&amp;quot;,&amp;quot;valueNumber&amp;quot;:4},{&amp;quot;id&amp;quot;:&amp;quot;7170f2a1-9248-4342-9d97-35e8383911c3&amp;quot;,&amp;quot;label&amp;quot;:&amp;quot;Deer&amp;quot;,&amp;quot;value&amp;quot;:&amp;quot;10&amp;quot;,&amp;quot;valueNumber&amp;quot;:10}],&amp;quot;type&amp;quot;:1,&amp;quot;isInitialState&amp;quot;:false,&amp;quot;xAxisLabel&amp;quot;:&amp;quot;&amp;quot;,&amp;quot;yAxisLabel&amp;quot;:&amp;quot;&amp;quot;}}""&gt;&lt;div data-sp-componentid=""""&gt;91a50c94-865f-4f5c-8b4e-e49659e69772&lt;/div&gt;&lt;div data-sp-htmlproperties=""""&gt;&lt;/div&gt;&lt;/div&gt;</property>
      </properties>
    </data>
  </webPart>
</webParts>";

                // Import the web part XML.
                WebPartDefinition webPartDefinition = webPartManager.ImportWebPart(myWPString);

                // Add the web part and execute the query.
                WebPartDefinition webPart = webPartManager.AddWebPart(webPartDefinition.WebPart, "FullPage", 0);
                context.Load(webPart);
                context.ExecuteQuery();

                // Checkin the page.
                f.CheckIn("Added new web part", CheckinType.MajorCheckIn);
                context.ExecuteQuery();
            }
            catch (Exception e)
            {
                if (checkedOut)
                {
                    f.CheckIn("Failed to add", CheckinType.MajorCheckIn);
                    context.ExecuteQuery();
                }
            }
        }
    }
}
