'use strict';

const build = require('@microsoft/gulp-core-build');

const editAndCopyResources = build.subTask('copyAndEditResources', (gulp, buildConfig, completeCallback) => {
  if (buildConfig.production) {
    const internalBuild = require('@ms/sp-build-internal-web');
    const path = require('path');
    const packageJson = require('./package.json');
    const through = require('through2');
    const merge = require('merge2');

    const cdnBasePath = internalBuild.getCDNBasePath();
    const dropLocation = path.join(__dirname, '..', '..', 'tools', 'vsts-deploy', 'app-min');

    const streams = Object.keys(packageJson.dependencies).map((dependency) => {
      const externalLibraryPath = path.join('node_modules', dependency);

      return gulp.src(path.join(externalLibraryPath, 'dist', '**'))
        .pipe(through.obj(function (file, enc, callback) {

          if (!file.isNull() && file.contents) {
            if (file.path.endsWith('.manifest.json')) {
              const contents = file.contents.toString(enc);
              let manifest = JSON.parse(contents);
              manifest.loaderConfig.internalModuleBaseUrls = [cdnBasePath];
              manifest = internalBuild.modifyFirstPartyManifest(manifest);
              file.contents = new Buffer(JSON.stringify(manifest));

              // removing '.manifest' from the filename before dropping it into app-min folder.
              const fileName = path.basename(file.path);;
              const newFileName = fileName.replace('.manifest', '');
              file.path = file.path.replace(fileName, newFileName);
            }

            this.push(file);
          }

          callback();
        }));
    });

    return merge(streams).pipe(gulp.dest(dropLocation)).on('finish', () => completeCallback());
  } else {
    completeCallback();
  }
});

build.task('default', editAndCopyResources);

build.initialize(require('gulp'));