﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Newtonsoft.Json;

namespace ClientAppProvisioner
{

    [JsonObject]
    internal class ClientJsonMetadata
    {
        public ClientJsonMetadata(string type)
        {
            this.Type = type;
        }

        [JsonProperty("type")]
        public string Type
        {
            get;
            private set;
        }
    }

    [JsonObject(MemberSerialization.OptOut)]
    internal abstract class ClientJsonObjectBase
    {
        [JsonProperty(PropertyName = "__metadata")]
        public ClientJsonMetadata Metadata
        {
            get
            {
                return new ClientJsonMetadata(this.MetadataTypeName);
            }
        }

        protected abstract string MetadataTypeName { get; }
    }

    [JsonObject]
    class PagePreloadCommandsJson
    {
	public bool ShouldPreloadWeb { get; set; }
	public bool ShouldPreloadUser { get; set; }
	public bool ShouldPreloadList { get; set; }
	public bool ShouldPreloadItem { get; set; }
    }


    /*
        {
            "manifest" : {
                "ApplicationId": "454c9834-e233-487d-8def-3e37c862dd61",
                "BootstrapModuleName": "lib/local/publishingApp/PublishingApp",
                "LoaderConfig":"{SYSTEMJS-STRINGIFIED CONFIG}",
                "PreloadCommands": {
		    "ShouldPreloadWeb": true,
		    "ShouldPreloadUser": true,
		    "ShouldPreloadList": true,
		    "ShouldPreloadItem": true
                 }
            }
        }
    */
    [JsonObject]
    class UploadAndDeployManifestJson 
    {
        public UploadAndDeployManifestJson()
        {
            this.PreloadCommands = new PagePreloadCommandsJson();
        }

        //protected override string MetadataTypeName
        //{
        //    get { return "SP.ClientRenderedPageManifest"; }
        //}

        public string ApplicationId { get; set; }
        public string BootstrapModuleName { get; set; }
        public string LoaderConfig { get; set; }
        public PagePreloadCommandsJson PreloadCommands { get; private set; }
    }

    [JsonObject]
    class UploadAndDeployBodyJson 
    {
        public UploadAndDeployBodyJson()
        {
            this.Manifest = new UploadAndDeployManifestJson();
        }

        [JsonProperty(PropertyName = "manifest")]
        public UploadAndDeployManifestJson Manifest { get; private set; }

    }


    /*
     $.ajax({
  url: "http://<site url>/_api/web/lists(guid'da58632f-faf0-4a78-8219-99c307747741')
    /fields",
  type: "POST",
  data: { '__metadata': { 'type': 'SP.Field' }, 'FieldTypeKind': 20, 'Title':
    'Stakeholders', 'SchemaXml': '<Field Type=\"UserMulti\" Required=\"TRUE\"
    UserSelectionMode=\"PeopleAndGroups\" UserSelectionScope=\"0\" Mult=\"TRUE\" />' },
  headers: { 
    "X-RequestDigest": <form digest value>,
    "accept": "application/json; odata=verbose",
    "content-type": "application/json;odata=verbose",
    "content-length": <length of body data>
  },
  success: successHandler,
  error: errorHandler
     */

    [JsonObject]
    class CreateFieldBodyJson : ClientJsonObjectBase
    {
        public static class FieldType
        {
            public const int Text = 2;
        }

        public CreateFieldBodyJson()
        {
        }

        protected override string MetadataTypeName
        {
	    get { return "SP.Field"; }
        }

        /// <summary>
        /// https://msdn.microsoft.com/en-us/library/office/microsoft.sharepoint.client.fieldtype.aspx
        /// </summary>
        public int FieldTypeKind { get; set; }

        public string Title { get; set; }

        public string SchemaXml { get; set; }
    }


}
