﻿using Newtonsoft.Json;
using Newtonsoft.Json.Serialization;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SPClientPackageExplorer
{
    public enum DataFileVersion
    {
        V1000 = 1000,
        // Added TrackedPackage.Starred
        V1001,
        // Added TrackedPackage.Note
        V1002,
        // Added NotApplicable designation
        V1003,
        LATEST = V1003
    }

    class DataFile
    {
        public DataFileVersion Version { get; set;  }

        public List<string> RootFolders = new List<string>();

        public readonly TrackedPackageSet TrackedPackageSet = new TrackedPackageSet();

        public string Filename;

        public DataFile()
        {
        }

        public void Clear()
        {
            this.Version = DataFileVersion.LATEST;
            this.RootFolders.Clear();
            this.TrackedPackageSet.Clear();
        }

        public void SaveToFile()
        {
            this.Version = DataFileVersion.LATEST;

            using (StringWriter stringWriter = new StringWriter())
            {
                using (JsonWriter jsonWriter = new JsonTextWriter(stringWriter))
                {
                    jsonWriter.Formatting = Formatting.Indented;
                    jsonWriter.WriteStartObject();

                    jsonWriter.WritePropertyName("Version");
                    jsonWriter.WriteValue((int)this.Version);

                    jsonWriter.WritePropertyName("RootFolders");
                    jsonWriter.WriteStartArray();
                    foreach (string rootFolder in RootFolders)
                    {
                        jsonWriter.WriteValue(rootFolder);
                    }
                    jsonWriter.WriteEndArray();

                    jsonWriter.WritePropertyName("TrackedPackageSet");
                    this.TrackedPackageSet.WriteJson(jsonWriter);

                    jsonWriter.WriteEndObject();
                }
                stringWriter.Flush();
                File.WriteAllText(this.Filename, stringWriter.ToString());
            }
        }

        public void LoadFromFile()
        {
            this.Clear();
            try
            {
                dynamic dataFileJson = JsonConvert.DeserializeObject(File.ReadAllText(this.Filename));

                if (dataFileJson["Version"] != null)
                {
                    this.Version = (DataFileVersion)dataFileJson.Version;
                }
                else
                {
                    this.Version = DataFileVersion.V1000;
                }

                // TODO: Schema validation

                    foreach (string rootFolder in dataFileJson.RootFolders)
                {
                    this.RootFolders.Add(rootFolder);
                }

                this.TrackedPackageSet.ReadJson(dataFileJson.TrackedPackageSet, this);

                this.Version = DataFileVersion.LATEST;
            }
            catch {
                this.Clear();
                throw;
            }
        }
    }
}
