// Type definitions for pseudoloc 1.1.0
// Definitions by: nickpape

declare namespace pseudoloc {
  export interface IPseudolocOptions {
    /**
     * Specifies the string that should be prepended to the beginning of pseudolocalized strings. The prepended
     * and appended strings help to locate strings that have been cut off or improperly concatenated
     * together - localized strings should use tokens for data since different languages have
     * different word orders.
     */
    prepend?: string;

    /**
     * Specifies the string that should be appended to the end of pseudolocalized strings. The prepended and
     * appended strings help to locate strings that have been cut off or improperly concatenated together.
     */
    append?: string;

    /**
     * Specifies the token delimiter.Any characters between token delimiters will not be pseudolocalized.
     * Tokens are used to replace data within localized strings. You can either specify a single delimiter or
     * use startDelimiter and endDelimiter to specify the delimiters seperately.
     */
    delimiter?: string;
    startDelimiter?: string;
    endDelimiter?: string;

    /**
     * Extends the width of the string by the specified percentage. Useful if you will be localizing into
     * languages such as German which can be 30% longer than English. Enter as a fraction (e.g. 0.3 for 30%)
     */
    extend?: number;

    /**
     * Specifies an override character that all characters in the string will be replaced with.
     * Used to easily spot unlocalized strings.
     */
    override?: string;
  }

  export interface IPseudoloc {
    /** Working configuration for pseudoloc */
    option: IPseudolocOptions;
    /** Resets the configuration to the default */
    reset: () => void;
    /** Pseudolocalizes a string using the current options */
    str: (text: string) => string;
    /** Pads a string with additional characters */
    pad: (text: string, percent: number) => string;
  }
}

declare module "pseudoloc" {
  const pseudoloc: pseudoloc.IPseudoloc;
  export = pseudoloc;
}
