# sp-visual-tests

## Introduction
sp-visual-tests is a collection of visual regression tests for projects in sp-client. The tests are
performed using [PhantomCSS](https://github.com/Huddle/PhantomCSS), which utilizes CasperJS and PhantomJS
to take screenshots of an application's UI. Screenshots can be diff'ed by PhantomCSS and the tests will fail
if a diff exceeds the allowed difference threshold.

## Getting Started
1. Create a folder under `screenshots` with the name of your project.
2. Create a test suite under `tests` using {projectName}.visual.test.js (TypeScript support pending.)
3. Set the `screenshotRoot` value of the PhantomCSS config in the test suite file to the folder
created in step 2.

You can follow `tests\canvas.visual.test.js` as an example test suite to get started. Additional API
information can be found at:
- [PhantomCSS](https://github.com/Huddle/PhantomCSS)
- [CasperJS](http://docs.casperjs.org/)

Below is a super bare bones test suite example:

```javascript
const fs = require('fs');
const phantomcss = require('phantomcss');

casper.test.begin('Canvas Workbench Visual Tests', function(test) {
  phantomcss.init({
    addLabelToFailedImage: false,
    casper: casper,
    failedComparisonsRoot: fs.absolute(fs.workingDirectory + '/screenshots/canvas/failures'),
    libraryRoot: fs.absolute(fs.workingDirectory),
    prefixCount: true,
    rebase: casper.cli.get('rebase'),
    screenshotRoot: fs.absolute(fs.workingDirectory + '/screenshots/canvas'),
  });

  // Specify the URL to use for the test
  casper.start('http://localhost');

  // Configure viewport size
  casper.viewport(1024, 768);

  // casper.then is a unit of action
  casper.then(function() {
    // Click on a button and take a screenshot
    casper.click('button');

    // Use css selector to specify portion of screen to caputure and provide a title to the image
    phantomcss('.dialog', 'Button Clicked Dialog');
  });

  // Compare screenshots
  casper.then(function() {
    phantomcss.compareAll();
  });

  // Kick off the tests
  casper.run(function() {
    casper.test.done();
  });
}
```

## Running the tests (Windows)
If you are on Windows there is a convenient batch script to help run the tests. Mac users can use the
batch script to write their command to run the script :)

> If the test suite is targeting a localhost instance you will need to have it running before you start
PhantomCSS, i.e. `gulp serve --nobrowser`.

```
> casperjs.bat test tests\yourtestsuite.js
```

If this is the first time you are running the tests PhantomCSS will generate baseline images. Otherwise,
PhantomCSS will diff the new images against the baseline images. If there are any differences between the
new image and baseline, PhantomCSS will report a test failure. If you go to the screenshots folder, there
will be the new image and a diff image. By default, the diff'ed areas are pink but you can configure to a
new color.

> The baseline images should be checked into Git. (Note: as the `screenshots` folder gets more populated
we will need to store the images elsewhere but link them to Git.)

Occasionally, the diff generated could be intentional. It is recommended to check the diff to confirm,
and check in a new baseline (rename `{title}.diff.png` to `{title.png}`).

## Known Issues
### Class Name Selectors
Using class name selectors is fragile. We would want to move to a better solution with auto generated
test identifiers.

### Debugging
Hopefully we can build more knowledge in this area as we create more tests.

### Test Suite in JavaScript
Low priority but having the ability to write type safe tests would be nice.
